#!/usr/bin/perl -w
#
#  Feed script for CD Sender
#
#  -------------------------------------------------------------
#  Author: Gerald Klinkl
#  -------------------------------------------------------------
#  Perl script for simulating incoming frames which can be
#  forwarded using CD Sender. Frame files must not be modified
#  when the script is running. Only frames which exists when
#  the script is started are used to simulate incoming frames
#
#  Usage: cdFeed.pl config_file mode delay stream(s)
#  -------------------------------------------------------------
#  ToDo:
#  -------------------------------------------------------------
#
#  $Log: cdFeed.pl,v $
#  Revision 1.4  2012-11-03 gerald
#  Expand environment variables and strftime % characters
#  in configuration parameters
#
#  Revision 1.3  2010-01-12 17:21:24  gerald
#  Fix too long line
#
#  Revision 1.2  2009-11-16 14:38:54  gerald
#  Add new command line parameter 'delay'
#
#  Revision 1.1  2009-08-31 11:52:55  gerald
#  Add utility script which is able "feeds" a CD Sender with synthetic or
#  real dat
#

unless (@ARGV > 1)
       { die "usage: $0 config_file stream(s)\n"; }

my $configFile = shift(@ARGV);
my $delay = shift(@ARGV);
my %stations;
my $rec;
my $stream;
my $indexLine;

###########################################################
# Create records which describe a stream
###########################################################

while (@ARGV)
{
   $stream = shift(@ARGV);
   &debug("create record for $stream");
   $rec = {};
   $rec->{name} = $stream;
   $rec->{files} = [];
   $rec->{clfOffset} = 0;
   $rec->{currClfFile} = undef;
   $stations{ $rec->{name} } = $rec;
}

###########################################################
# Read config value from config file
###########################################################

sub readConfigValue()
{
   my $configVar = $_[0];
   my $configValue = `egrep "^$configVar=" $configFile | sed -e 's/.*=//'`;
   chop($configValue);
   $configValue =~ s/\$\(([^\)]+)\)/$ENV{$1}/g;

   $configValue = `date +$configValue`;
   # don't use debug() here, debug value might not be read so far
   # print("$_[0]=$configValue");
   chop($configValue);
   if ($_[1] eq 'strict' && (!defined $configValue || $configValue eq ''))
   {
      die "$_[0] not found in $configFile";
   }

   return($configValue);
}

###########################################################
# Read config file
###########################################################

unless (-r $configFile)
       { die "can't open config file $configFile\n"; }
my $port = `egrep '^port=' $configFile | sed -e 's/.*=//'`;
chop($port);
my $indexFilePath = &readConfigValue('cdrecvIndexFilePath', 'strict');
my $filePath = &readConfigValue('filePath', 'strict');
my $feedFilePath = &readConfigValue('feedFilePath', 'strict');
my $feedIndexFilePath = &readConfigValue( 'feedIndexFilePath', 'strict');
my $feedMode = `egrep '^feedMode=' $configFile | sed -e 's/.*=//'`;
chop($feedMode);
my $debugLevel = &readConfigValue( 'debugLevel', 'nonstrict');

###########################################################
#  Debug function
###########################################################

sub debug()
{
   print "DEBUG:$_[0]\n" if ($debugLevel);
}

###########################################################
#  Dump content of all known station records
###########################################################

sub checkRecords()
{
   for $stream (keys %stations)
   {
      print "stream: $stream\n";
      for $indexLine ( @{ $stations{$stream}{files} } )
      {
         print "\t$indexLine\n";
      }
      print "\tcurrent file: [";
      print $stations{$stream}->{currClfFile}
            if $stations{$stream}->{currClfFile};
      print "] - $stations{$stream}->{clfOffset}\n";

   }
}

###########################################################
#  Try to open next clf file
###########################################################

sub openNextClf()
{
   local $actStream = $_[0];
   my $feedFile;

   &debug("try to open next file for $actStream");
   $stations{$actStream}{currClfFile} = undef;
   $stations{$actStream}{clfOffset} = 0;
   $feedFile = shift @{ $stations{$actStream}{files} };
   return unless defined ($feedFile);
   $feedFile =~ s/$filePath/$feedFilePath/;
   $_ = $feedFile;
   local ($fPath, $fDate, $fExt) = m/(.*\/)[^.]+\.(\d+)\.\d+=cd(.*)/;
   $fExt =~ s/^1$/10/;
   &debug("path [$fPath], date part [$fDate], ext [$fExt]");
   &debug("feed line: $feedFile");
   $indexFile = "$feedIndexFilePath/_$port.$fDate.idx";
   # Add clf file to index file
   open(FEEDINDEX,">>$indexFile") or die "can't open $indexFile";
   print FEEDINDEX "$feedFile\n";
   close FEEDINDEX;
   # Create intial clf file
   system "mkdir -p $fPath" unless -w $fPath;
   s/=.*/.clf/;
   &debug("New clf is $_");
   open(FEEDCLF, ">$_") or die "can't open $_";
   # Link old with new binary file
   local $feedBin = $_;
   $feedBin =~ s/\.clf/.$fExt/;
   local $bin = $feedBin .= 'bin';
   $bin =~ s/$feedFilePath/$filePath/;
   &debug("New bin is $feedBin");
   system "ln -s $bin $feedBin" unless -r $feedBin;
   # open original clf file
   s/$feedFilePath/$filePath/;
   $stations{$actStream}{currClfFile} = $_;
}

###########################################################
#  First pass: add list of clf files to each stream record
###########################################################

# Get list of index file
my @indexFiles = `ls $indexFilePath/_$port.*.idx`;
chop(@indexFiles);

foreach $indexFile (@indexFiles)
{
   &debug("processing $indexFile");
   open(INDEX,"<$indexFile") or die "can't open $indexFile";
   while(<INDEX>)
   {
      chop;
      $indexLine = $_;
      s/=.*//;    # extract file name without extension
      &debug("filename is $indexLine");
      s/.*\///;   # extract stream name
      s/\..*//;
      $stream = $_;
      &debug("stream is $stream");
      # Add file name to streams record
      if ($stations{$stream})
         { push @{ $stations{$stream}->{files} }, $indexLine; }
      else { &debug("skipping $stream"); }
   }
   close(INDEX);
}

&checkRecords();
&debug("filePath: $filePath");
&debug("feedFilePath: $feedFilePath");

###########################################################
# and do some initial work:
#   remove all index files for the given port in the
#   target directory and create target directory if
#   needed.
###########################################################

system "rm -f $feedIndexFilePath/_$port.*.idx";
system " mkdir -p $feedIndexFilePath"  unless -w $feedIndexFilePath;

###########################################################
#   create index file and initial clf files for
#   each stream
###########################################################

for $stream (keys %stations)
{
   &debug("initalizing stream: $stream");
   &openNextClf($stream);
}

&checkRecords();


###########################################################
#  Second pass: main loop; walk to the list of streams
#               and add a frame from the original clf file
#               to the "feed" clf file every "duration"
#               interval
###########################################################

for(my $stop = 0; $stop == 0;)
{
   $stop = 1;
   for $stream (keys %stations)
   {
      # for each station check if the clf file must be changed
      &debug("processing stream: $stream");
      $_ = $stations{$stream}{currClfFile};
      next unless defined($_);
      open(CLF, "<$_") or die "can't open $_";
      &debug("Current clf is $_, offset ". $stations{$stream}{clfOffset});
      seek CLF, $stations{$stream}{clfOffset}, 0 or die "can't seek";
      # try to read the next line
      my $line = <CLF>;
      if ( ! defined($line))
      {
         close CLF;
         &openNextClf($stream);
         redo;
      }
      $stations{$stream}{clfOffset} = tell CLF;
      s/$filePath/$feedFilePath/;
      open(FEEDCLF, ">>$_") or die "can't open $_";
      print FEEDCLF $line if $line;
      close FEEDCLF;
      close CLF;
      $stop = 0;
   }
   if (0 != $delay) { sleep ($delay); }
}

&checkRecords();
