#!/usr/bin/perl -w
#
#  Perl script to transform command responses into human
#  readable text
#
#  ---------------------------------------------------------
#  Author: Gerald Klinkl
#  Last modification $Date: 2011-10-11 12:23:20 $
#  $Revision: 1.8 $
#  ---------------------------------------------------------
#  This script is used to transform the output from CD 
#  Controller into a human-friendly form. It accepts the
#  same command line arguments as CD Controller and adds
#  additional text to the command response.
#
#  Usage: cdcon.pl conﬁg_ﬁle frame_destination:[IP]:port command
#  ---------------------------------------------------------
#  ToDo:
#  ---------------------------------------------------------
#
#  $Log: cdcon.pl,v $
#  Revision 1.8  2011-10-11 12:23:20  ctbto
#  Fix incorrect description of configuration parameters
#  (explain tool is now used)
#
#  Revision 1.7  2006-09-06 12:45:18  gerald
#  Sender specific configuration values have been moved
#  from station structure to sender extension structure
#
#  Revision 1.6  2005/07/07 09:56:36  gerald
#  Add new states stopped and wait2send
#
#  Revision 1.5  2004/11/12 14:37:50  gerald
#  Sync to new command output
#
#  Revision 1.4  2004/11/08 16:48:55  gerald
#  Fix broken cdcon.pl output
#
#  Revision 1.3  2003/12/16 14:41:52  gerald
#  Fix missing connection state text
#
#  Revision 1.2  2003/12/11 16:18:40  gerald
#  Add text for "set config params" command and fix use of IP
#  address
#
#  Revision 1.1  2003/12/09 10:55:22  gerald
#  Add tool to make cdcon output readable for humans
#
#


my ($configFile, $command, $ip);
my $ok = 'no';

my @states = ('int. error', 'unconnected', 'connecting',
              'waiting', 'connected', 'wait for send', 'stopped');
my @types = ('int. error', 'unknown', 'CD1.0', 'CD1.1');
my @a_states = ('unknown', 'no keys', 'off', 'active');
my @apps = ('CD Receiver', 'CD Sender', 'CD Controller', 'Other');

unless (@ARGV > 2)
{
   die "usage: $0 config_file station:[IP]:port command";
}

$configFile = shift(@ARGV);
$peerInfo = shift(@ARGV);
$command = shift(@ARGV);

my ($nu, $app, $version, $date, $started, $running, $stopped);
my $is_init = 0;

sub printThreads()
{
    ($nu, $nu, $app, $version, $date, $started,
     $running, $stopped) = split(/ /, $_[0], 8);
    chop($stopped);
    $app = $apps[$app];
    print "$app $version, started at $date, " .
          "$started threads started, $running running, $stopped stopped\n";
}

sub checkInPath()
{
   foreach (split(':', $ENV{'PATH'})) { return(1) if (-x "$_/$_[0]"); }
   return 0;
}

sub printConfig()
{
   my $configParams;

   (undef, undef, $val1) = split(/ /, $_[0], 3);
   print "$_[0]";
   chop($val1);
   $configParams = 0 + $val1;

   my $explain = '../src/tools/explain';
   $explain = 'explain' if &checkInPath('explain');
   open (CMD2, "$explain -s $configParams |") or
        die "can't execute $explain";

   print <CMD2>;
}
my ($station, $state, $type, $port, $a_state);

my $format0 = '%-17s %-11s %-7s %6s %-8s';
my @f0strings = ('Name', 'State', 'Type', 'Port', 'A-State');
my $line0 = '----------------------------------------------------';

sub printGst0()
{
   printf $format0 . "\n", @f0strings;
   printf $line0 . "\n";
   ($station, $state, $type, $port, $a_state, $nu) = split(/ /, $_[0], 6);
   $state = $states[$state];
   $type = $types[$type];
   $a_state = $a_states[$a_state];
   printf $format0 . "\n\n", $station, $state, $type, $port, $a_state;
   $nu;
}

my $format1 = ' %9s %9s %9s %9s %9s';
my @f1strings = ('ConRq', 'CR-WARN', 'CR-ERR', 'WARNINGS', 'ERRORS');
my $line1 = '--------------------------------------------------';
my ($val1, $val2, $val3, $val4, $val5);

sub printGst1()
{
   $nu = &printGst0("$_[0]");
   printf $format1 . "\n", @f1strings;
   printf $line1 . "\n";
   ($val1, $val2, $val3, $val4, $val5, $nu) = split(/ /, $nu, 6);
   printf $format1 . "\n\n", $val1, $val2, $val3, $val4, $val5;
   $nu;
}

sub printStationOverview()
{
   if ($is_init == 0)
   {
      $is_init = 1;
      printf $format0 . ' ' . $format1 . "\n", @f0strings, @f1strings;
      printf $line0 . ' ' . $line1 . "\n";
   }
   ($station, $state, $type, $port, $a_state, $nu) = split(/ /, $_[0], 6);
   $state = $states[$state];
   $type = $types[$type];
   $a_state = $a_states[$a_state];
   printf $format0 . ' ' . $format1, $station, $state, $type, $port, $a_state,
          split(/ /, $nu, 5);
}

#----------------------------------------------------

my $format2 = ' %9s %9s %9s %9s';
my @f2strings = ('Frames-R', 'Frames-S', 'Bytes-R', 'Bytes-S');
my $line2 = '----------------------------------------' ;

sub printGst2()
{
   $nu = &printGst1("$_[0]");
   printf $format2 . "\n", @f2strings;
   printf $line2 . "\n";
   ($val1, $val2, $val3, $val4, $nu) = split(/ /, $nu, 5);
   printf $format2 . "\n\n", $val1, $val2, $val3, $val4;
   $nu;
}

#----------------------------------------------------

my $format3 = ' %9s %9s';
my @f3strings = ('Sys-Conf', 'D-Level');
my $line3 = '---------------------' ;

sub printGst3()
{
   $nu = &printGst2("$_[0]");
   printf $format3 . "\n", @f3strings;
   printf $line3 . "\n";
   printf $format3 . "\n", split(/ /, $nu, 2);
}

my $cdcon = '../src/cdcon/cdcon';
$cdcon = 'cdcon' if &checkInPath('cdcon');
open (CMD, "$cdcon $configFile $peerInfo $command |") or
     die "can't execute $cdcon";

while (<CMD>)
{
   if ($_ =~ /^000 OK/)
   {
      $ok = "yes";
      if ($command eq "GST") { &printThreads($_); next; }
      elsif ($command =~ /^SCP=/) { &printConfig($_); next; }
      elsif ($command =~ /^GCP/) { &printConfig($_); next; }
      else  { print "$_"; next; }
   }
   if ($ok eq 'yes')
   {
      if ($command =~ /^GST=1/) { &printStationOverview($_); }
      elsif ($command =~ /^GST=2/) { &printGst2($_); }
      elsif ($command =~ /^GST=3/) { &printGst3($_); }
      else { print "$_"; }
   }
   else
   {
      print "$_";
   }
}

close (CMD);
