#!/usr/bin/perl -w

#
#  Script to generate a human readable list of gaps in a
#  given clf file
#
#  ----------------------------------------------------------
#  Author: Gerald Klinkl, SIS SI CD, gerald.klinkl@atos.net
#  Last modification $Date: 2006/01/31 16:47:31
#  $Revision: 1.5
#  ----------------------------------------------------------
#  ToDo:
#  ----------------------------------------------------------
#  $Log: check4gaps.pl,v
#
#

use POSIX qw(strftime);
use strict;

my $debugLevel = 0;
our %stations;

sub debug()
{
   print "DEBUG:$_[0]\n" if ($debugLevel);
}

sub getNFR()
{
   my $actFrame;
   my $oldFrame = 0;
   my $nfr = 0;
   my $actStation = $_[0];
   my $frames = $actStation->{FRAMES};

   # compute nominal frame rate
   my %timeDiff;
   my $high = 0;

   for my $actFrame (sort keys %$frames)
   {
      if ($oldFrame != 0)
      {
         &debug("actFrame: $actFrame, oldFrame: $oldFrame");
         $timeDiff{$actFrame - $oldFrame}++;
      }
      else
      {
         $actStation->{LOW} = $actStation->{HIGH} = 
                              $actFrame; # initialize high/low marks
      }
      $oldFrame = $actFrame;
   }
   for my $nbr (reverse sort keys %timeDiff )
   {
      &debug("diff $nbr, number $timeDiff{$nbr}");
      if ($timeDiff{$nbr} > $high)
      {
         $nfr = $nbr;
         $high = $timeDiff{$nbr};
      }
   }
   &debug("$actStation->{NAME}: nominal frame rate is $nfr");
   $actStation->{NFR} = $nfr;
}

sub getTimeAsText()
{
   my ($seconds, $mSeconds) = split ('\.', $_[0], 2);
   my $txt = strftime("%Y/%m/%d %H:%M:%S", gmtime($seconds));
   if (!defined($mSeconds)) { $mSeconds = 0; }
   return("$txt.$mSeconds");
}

sub writeStat()
{
   my $fh = shift;
   my $actStation = shift;
   my $end = 0;
   my $nfr = $actStation->{NFR};
   my $start = 0;
   my $gaps = $actStation->{GAPS};

   print $fh "+++ Station:  $actStation->{NAME}, " .
             "$actStation->{COUNT} frames, " .
             "Nominal frames rate (computed): $actStation->{NFR}\n";


   if ($actStation->{COUNT} < 10)
   {
      print "** Too few frames to print reasonable values *** \n\n";
      return;
   }

   print $fh "Oldest frame: $actStation->{LOW} [" . 
              &getTimeAsText($actStation->{LOW}) . "]\n";
   print $fh "Newest frame: $actStation->{HIGH} [" . 
              &getTimeAsText($actStation->{HIGH}) . "]\n";

   if ($actStation->{NFR} > 0)
   {
      print $fh "Gaps:\n";
      for my $actFrame (sort keys %$gaps)
      {
         &debug("start/end/act $start/$end/$actFrame");
         if ($start == 0) { $start = $end = $actFrame; next; }
         if (($end + $nfr) != $actFrame) { 
            print $fh "  $start - $end  [" . &getTimeAsText($start) . " - " .
                                             &getTimeAsText($end) . "]\n"; 
            $start = $end = $actFrame; 
         }
         else { $end += $nfr; }
      }
      if ($start != 0) { 
            print $fh "  $start - $end  [" . &getTimeAsText($start) . " - " .
                                             &getTimeAsText($end) . "]\n"; 
      }
   }

   print $fh "Frames with duplicated nominal time: $actStation->{DUPLICATE}\n";
   print $fh "Frames with unexpected nominal time: $actStation->{STRANGE}\n\n";
}

sub processFile()
{
   my $fileName = $_[0];
   my ($frames, $allframes);

   &debug("file name: $fileName");
   my $name = $fileName;
   $name =~ s/\..*//;
   $name =~ s/.*\///;

   open(FH, "<$fileName") or die "cannot open station $fileName ";

   while (<FH>)
   {
      my($fs, $type, $of, $lng, $ft, $sor, $nt, $fpc, $span, $cm, $fts) = 
         split(' ', $_);

      my $actStation = $stations{$name};
      if (!defined ($actStation)) { $actStation = &createStation($name) }

      next if ($ft !~ /[<>][dD]$/);
      ($nt, undef) = split ('\.', $nt, 2); # drop fraction part
      $actStation->{COUNT}++;
      $actStation->{FRAMES}{$nt} = $cm;

      if (defined($actStation->{ALLFRAMES}{$nt}))
      {
         $actStation->{ALLFRAMES}{$nt}++;
      }
      else
      {
         $actStation->{ALLFRAMES}{$nt} = 1;
      }

      if ($actStation->{RECS} == 10) {
         if ($actStation->{NFR} == 0) {
            &getNFR($actStation);
         }
         &processFrames($actStation); 
         $actStation->{RECS} = 0;
      } 
      else { $actStation->{RECS}++; }
   }
   close FH;
}

sub createStation()
{
   my $station = { NAME => $_[0],
                   NFR => 0,
                   LOW => 0,
                   HIGH => 0,
                   RECS => 0,
                   FRAMES => {},
                   ALLFRAMES => {},
                   GAPS => {},
                   DUPLICATE => 0,
                   COUNT => 0,
                   STRANGE => 0,
                 };

   $stations{$_[0]} = $station;
   &debug("New station: $_[0]");
   return($station);
}

sub processFrames()
{
   my $actStation = shift;
   my $actFrame;
   my $oldFrame = 0;
   my $pLow = $actStation->{LOW};
   my $pHigh = $actStation->{HIGH};
   my $nfr = $actStation->{NFR}; # nominal frame rate
   my $frames = $actStation->{FRAMES}; # nominal frame rate

   if ($nfr > 0)
   {
      for my $actFrame (sort keys %$frames )
      {
         &debug("1:low/actFrame/high: $pLow/$actFrame/$pHigh");
         # check for first frame
         if ($pLow == $actFrame && $pHigh == $actFrame) { next; } # first frame
      
         # check if the covered area enlarge seamless 
         if (($pHigh + $nfr) == $actFrame) { $pHigh = $actFrame; next; }
         if (($pLow - $nfr) == $actFrame) { $pLow = $actFrame; next; }

         # check if this is a frame besides the nfr
         if (substr($actFrame, -1) ne '0') {
            &debug("unexpected nfr $actFrame");
            $actStation->{STRANGE}++;
            next;
         }

         # check if a gap is filled
         if ($actFrame <= $pHigh && $actFrame >= $pLow) 
         {
            if (!$actStation->{GAPS}{$actFrame}) { 
               &debug("frame seen before $actFrame"); 
               $actStation->{DUPLICATE}++;
            }
            else {
               delete $actStation->{GAPS}{$actFrame};
               &debug("delete gap $actFrame");
            }
            next;
         }

         &debug("low/actFrame/high: $pLow/$actFrame/$pHigh");
#        sleep(1);
         my ($start, $end);

         # create new gap(s)
         if ($actFrame > $pHigh) { $start = $pHigh + $nfr; $end = $actFrame; 
                                  $pHigh = $actFrame; }
         if ($actFrame < $pLow) { $start = $actFrame + $nfr;  $end = $pLow; 
                                  $pLow = $actFrame; }
   
         for (my $gap = $start; $gap < $end; $gap += $nfr)
         {
             $actStation->{GAPS}{$gap} = 1;
             &debug("add gap $gap");
         }
      }
      &debug("low/high: $pLow/$pHigh");
      $actStation->{LOW} = $pLow;
      $actStation->{HIGH} = $pHigh;
   }
   # empty hash now
   %$frames = ();
}

sub processAllFrames()
{
   my $actFrame;
   my $oldFrame = 0;
   my $pLow = 0;
   my $pHigh = 0;
   my $actStation = shift;
   my $nfr = $actStation->{NFR}; # nominal frame rate
   $actStation->{GAPS} = ();
   my $allframes = $actStation->{ALLFRAMES};

   if ($nfr > 0)
   {
      for my $actFrame (sort keys %$allframes )
      {
         &debug("1:low/actFrame/high: $pLow/$actFrame/$pHigh");

         # check if the low/high water mark has to be adjusted
         if ($pHigh == 0) { $pHigh = $pLow = $actFrame; }
         if ($pHigh < $actFrame) { $pHigh = $actFrame; }
         if ($pLow > $actFrame) { $pLow = $actFrame; }

         # check if this is a frame besides the nfr
         if (substr($actFrame, -1) ne '0') {
            &debug("unexpected nfr $actFrame");
            $actStation->{STRANGE}++;
            next;
         }

         # Check for duplicates
         $actStation->{DUPLICATE} += $$allframes{$actFrame} - 1;

         if ($oldFrame == 0) { $oldFrame = $actFrame; next; }

         # Check for gap. No need to delete gap when sorted
         if (($oldFrame + $nfr) != $actFrame) 
         {
            $actStation->{GAPS}{$actFrame} = 1;
         }
         $oldFrame += $nfr;
      }
      &debug("low/high: $pLow/$pHigh");
      $actStation->{LOW} = $pLow;
      $actStation->{HIGH} = $pHigh;
   }
}

if (scalar(@ARGV) != 1) { die "usage: $0 [clf_file]\n"; }

my $fileName = shift(@ARGV);
&processFile($fileName);

for my $name (sort keys %stations )
{
   my $actStation = $stations{$name};
   &processFrames($actStation);
   &writeStat(\*STDOUT, $actStation);
   &processAllFrames($actStation);
   &writeStat(\*STDOUT, $actStation);
}
