#!/usr/bin/perl -w
#
# *****************************************************
#  International Data Centre                         
#  Comprehensive Test Ban Treaty Organization         
#  Vienna                                             
#  Austria                                            
# *****************************************************
#
#  Tool for creating data time ordered index for frame files
#
#  -------------------------------------------------------------
#  Author: Gerald Klinkl, SIS PRO HPS, gerald.klinkl@siemens.com
#  Last modification $Date: 2011-05-16 11:47:53 $
#  $Revision: 1.5 $
#  -------------------------------------------------------------
#  Perl script for creating data time ordered index for frame files.
#  The script is designed to read the extended stream mode output
#  from CD Report and create the data time ordered index out of it
#
#  Usage: dataTimeOrder.pl config_file
#  -------------------------------------------------------------
#  ToDo:
#  -------------------------------------------------------------
#
#  $Log: createCdx.pl,v $
#  Revision 1.5  2011-05-16 11:47:53  gerald
#  Fix incorrect offset lines and unexpected halts when
#  frame spawns midnight
#
#  Revision 1.4  2010-06-08 13:19:12  gerald
#  Add CSF start time and protocol version to cdx file
#
#  Revision 1.3  2010-04-16 09:03:59  gerald
#  Adjust debug output priorities and read debug level
#  from configuration file
#
#  Revision 1.2  2010-03-11 15:17:37  gerald
#  Fix external to internal station name mapping
#
#  Revision 1.1  2010-03-04 13:01:43  gerald
#  Rename dataTimeOrder.pl to createCdx.pl
#
#  Revision 1.2  2010-01-21 17:04:34  gerald
#  Add mapping of external site names to internal ones
#
#  Revision 1.1  2009-10-19 09:39:45  gerald
#  Add script for creation of data time ordered index files
#
#  Revision 1.1  2009-08-31 11:45:34  gerald
#

use POSIX qw(strftime);

my $debugLevel = 0;

sub debug()
{ print STDERR "DEBUG:$_[1]\n" if ($_[0] <= $debugLevel); }

unless (@ARGV == 1)
       { die "usage: $0 config_file\n"; }

#----------------------------------------------------------------------
# Read and process the config file
#----------------------------------------------------------------------

my $configFile = shift(@ARGV);

unless (-r $configFile)
       { die "can't open config file $configFile\n"; } 

my $filePath = `egrep '^filePath=' $configFile | sed -e 's/.*=//'`;
chop($filePath);

my $sensorFile = `egrep '^sensorFile=' $configFile | sed -e 's/.*=//'`;
chop($sensorFile);

if (defined($ENV{DEBUG_LEVEL}))
{
    $debugLevel = $ENV{DEBUG_LEVEL};
}
else
{
   my $debugL = `egrep '^debugLevel=' $configFile | sed -e 's/.*=//'`;
   chop($debugL);

   if ($debugL =~ /(\d+)/)
   {
       $debugLevel = $1;
   }
}

#----------------------------------------------------------------------
# Process sensor info file. Read external station names in associative
# array
#----------------------------------------------------------------------

my %siteTable;

open(SENSORINFO,"<$sensorFile") or die "can't open sensor info [$sensorFile]";
while(<SENSORINFO>)
{
   local(undef, $intSite, $extSite, undef) = split;
   if ($extSite eq $intSite) { next; }
   &debug(5, "$extSite $intSite");
   $siteTable{$extSite} = $intSite;
}
close(SENSORINFO);
&debug(5, "filepath=$filePath");

#----------------------------------------------------------------------
# Main loop: process input from CD report
#----------------------------------------------------------------------

my ($stream, $prot, $SCL, $cpc, $c_len, $startTime, $dtype, $calper, 
    $calib, $status, $timespan, $nbrOfSamples, $file, $path, $offsetInFile, 
    $ii, @checkSlot, $fileAndPath);

while (<STDIN>)
{
   &debug(2, "$_");
   # Read output from CD Report in extended stream mode
   # Fields: *stream, *protocol version, *site/channel/location, *channel 
   #         parsing code, offset of the ﬁrst byte of the channel
   #         relative to the start of the frame,
   #         *channel length (CD-1.1) or packet length (CD-1.0),
   #         offset to authentication key identiﬁer, authentication switch,
   #         compression ﬂag, sensor type, option ﬂag, *calibration factor,
   #         *calibration period, *uncompressed data format type, *nominal 
   #         time (CD-1.1) or time stamp (CD-1.0) of channel in epoch format,
   #         *subframe time length, *number of samples, channel status size,
   #         *channel status information, data size, subframe count for CD-1.1
   #         or “NA” for CD-1.0, authentication key identiﬁer or -1 for CD-1.0,
   #         authentication size, *ﬁle name and path of the binary ﬁle,  
   #         *offset of the channel from the beginning of the  binary ﬁle
   my $actLine = $_;
   ($stream, $prot, $SCL, $cpc, undef, $c_len, undef, undef, undef, 
    undef, undef, $calib, $calper, $dtype, $startTime, $timespan, 
    $nbrOfSamples, undef, $status, undef, undef, undef, undef, $fileAndPath, 
    $offsetInFile) = split(' ', $actLine);

   my $startTimeF = $startTime;
   # Time span might not be aligned to a second so it is rounded 
   # to the nearest second. That gives the best results.
   my $secondsLeft = 0 + sprintf("%.0f",($timespan/1000));

   while ($secondsLeft > 0)
   {
      # Compute start of day time for this data
      # Start time might not be aligned to a second. To align it to
      # a second a truncat policy is used. If this causes troubles
      # use round here! 
      $startTime = int($startTime);
      my $startOfDay = $startTime - ($startTime % 86400);

      # Compute the path variable for the channel index file 
      # Channel index file is named: 
      #         filePath/STREAM/SITE_CHANNEL.YYYYDDMM.0000.cdx
      my $outPath = strftime($filePath, gmtime($startOfDay));
      my $outFileDate = strftime("%Y%m%d", gmtime($startOfDay));
      $outPath = "$outPath/$stream";
      &debug(5, "outPath=$outPath");

      # Create the output directory if it does not exist
      ! -d $outPath and system "mkdir -p $outPath";

      my ($site, $channel, undef) = split ('/', $SCL);
      # Map External to internal site if different
      if (defined($siteTable{$site})) { $site = $siteTable{$site}; } 
      # Compute the output file name
      my $outFile = "$outPath/${site}_${channel}.$outFileDate.0000.cdx";
      &debug(2, "$outFile");

      if ($fileAndPath =~ /(.*\/)(.+)/)
      {
         $path = $1;
         $file = $2;
      }
      else
      { 
         $path = './';
         $file = $fileAndPath;
      }
      $path = $ENV{PWD} . '/' if ($path eq './');

      # Create a sysmlink back to the original binary file 
      # if it does not exists
      ! -l "$outPath/$file" and symlink("$path$file", "$outPath/$file");

      # Check if the output file exists
      if (-f $outFile) 
      { 
         # Open the output file
         open(FP, "+<$outFile") or die "cannot open $outFile";
         &debug(3, "$outFile exists");
      }
      else 
      { 
         # Create and initialize the output file
         open(FP, "+>$outFile") or die "cannot create $outFile";
         for ($ii = 0; $ii < 86400; $ii++)
         { print FP "0         \n"; }
         &debug(3, "$outFile does not exist");
      }

      #  Don't use my $fileSize = -s FP; here. The file output is
      #  buffered!!
      seek FP, 0, 2 or die "can't seek to end of file";
      my $fileSize = tell FP;
   
      #  Run some basic checks on the file
      if ($fileSize < (86400 * 11))
      {
         die "$outFile has an invalid file (filesize)" 
      }
   
      # Compute seconds from start of day
      my $secInDay = $startTime % 86400;
      # Compute index slot offset
      my $outOff = $secInDay * 11;
      &debug(5, "offset [$outOff] secInDay [$secInDay]");
      # Check if the frame crosses midnight
      my $actSpan = $secondsLeft;
      if (($secInDay + $secondsLeft) >= 86400)
      {
         $actSpan =  86400 - $secInDay;
         $startTime += $actSpan;
      }
      $secondsLeft -= $actSpan;

      # Seek to first index slot
      $ii = 0;
      seek FP, $outOff, 0 or die "can't seek to $outOff";
      while (<FP>)
      {
         # Read slot to check if this is the first data for this time periode
         chop;
         &debug(5, "slot=[$_]");
         push(@checkSlot, $_);
         $ii++;
         last if ($ii >= $actSpan);
      }

      # All slots read 
      if ($ii > $actSpan)
      {
         die "internal error when reading slots [$actLine][$ii][$actSpan]";
      }

      # Seek back to first index slot
      seek FP, $outOff, 0 or die "can't seek to $outOff";
      for ($ii = 0; $ii < $actSpan; $ii++)
      {
         if (shift(@checkSlot) != 0)
         {
            print "Warning, duplicate data for $file NT: $startTimeF ignored\n";
            seek FP, 11, 1 or die "can't seek 11 bytes";
            next;
         }
         # Write channel line offset to all affected slots
         printf FP "%-10d\n", $fileSize;
      }

      seek FP, 0, 2 or die "can't seek to end of file";
      # Write channel line
      print FP "$file $offsetInFile $c_len 0x$cpc $timespan $dtype $calper " .
               "$calib $nbrOfSamples $startTimeF $status $prot\n";

      &debug(3, "outOff=$outOff fileSize=$fileSize");
      &debug(3, "file=$file path=$path");
      close (FP);
   }
}
