#!/usr/bin/perl -w

#
#  Script to create/update frameproduct entries for binary
#  files
#
#  ----------------------------------------------------------
#  Author: Gerald Klinkl gerald.klinkl@ctbto.org
#  ----------------------------------------------------------
#  This script updates the file size of binary files in the
#  frame product table if needed. This is necessary if a non data
#  frame has been received at the end of the day. In this case
#  cd2wng does not update the binary file size in the
#  frameproduct table, because cd2wng considers only data frames.
#  When this script is started, it checks the file size of binary
#  files for the last 7 days by default.
#
#  usage: updateFrameproduct.pl config_file [look_back_days]
#  ----------------------------------------------------------
#  ToDo:
#  ----------------------------------------------------------
#  $Log: updateFrameproduct.pl,v
#
#  2013-10-08  gerald
#  Do not longer issue a warning when an orphaned file is detected
#  which does not contain data frames (ISI-178)
#
#  2013-08-08  gerald
#  Do not longer try to add an entry in the frameproduct table
#  when an orphaned file is detected, log a warning instead.
#  Remove fixed return code -1 from getFrid() function
#
#  2013-05-22  Henrik
#  Use Local::PAR if it exists to read PAR files (for PAR files with depth).
#  Properly parse the dbAccount string into user and optional password.
#  Remove hard-coded debugLevel=2
#  Allow driver override from default ODBC using environment DBI_DRIVER
#  Allow data source override from config using environment DBI_DSN
#  Add support for mwf (type 54) files
#
#  2012-01-02  gerald
#  Initial version
#

use strict;
use DBI;
use IO::Select;
use IO::Socket;
use Time::Local;

my $debugLevel = 0;

sub debug()
{
   print "DEBUG:$_[1]\n" if ($debugLevel >= $_[0]);
}

unless (@ARGV > 0)
{ die "usage: $0 config_file [lookback_days]\n"; }

#----------------------------------------------------------------------
# Read and process the config file
#----------------------------------------------------------------------

my $configFile = shift(@ARGV);

unless (-r $configFile)
       { die "can't open config file $configFile\n"; }

my $filePath;
my $dsn;
my $dbaccount;
my $debugL;

eval {  # check if Local::Par exists/loads
    require Local::Par;
};
if ( $@ ) {  # Local::Par does not exist/load
    $filePath = `egrep '^filePath=' $configFile | sed -e 's/.*=//'`;
    chomp($filePath);
    $dsn = `egrep '^dbConnectName=' $configFile | sed -e 's/.*=//'`;
    chomp($dsn);
    $dbaccount = `egrep '^dbAccount=' $configFile | sed -e 's/.*=//'`;
    chomp($dbaccount);
    $debugL = `egrep '^debugLevel=' $configFile | sed -e 's/.*=//'`;
    chomp($debugL);
}
else {  #  Local::Par exists/loads
    my %par = Local::Par::getpar("par=$configFile");
    $filePath = $par{'filePath'} || '';
    $dsn = $par{'dbConnectName'} || '';
    $dbaccount = $par{'dbAccount'} || '';
    $debugL = $par{'debugLevel'};
}

my $dbuser = '';
my $dbpw = '';
if ( defined($dbaccount) and ($dbaccount =~ m|([^/@]*)(/([^@]*))?(@.*)?| )) {
    $dbuser = $1;
    if (defined($3)) {
        $dbpw = $3;
    }
}

if (defined($ENV{DEBUG_LEVEL}))
{
    $debugLevel = $ENV{DEBUG_LEVEL};
}
else
{
   if (defined($debugL) and ($debugL =~ /(\d+)/))
   {
       $debugLevel = $1;
   }
}

my $lookBack = 7;
$lookBack = shift(@ARGV) if (@ARGV > 0);

STDOUT->autoflush;

#----------------------------------------------------------------------
# Connect to the database
#----------------------------------------------------------------------

my $driver = '';  # use DBI_DRIVER environment variable at DBI->connect
if (not(defined($ENV{'DBI_DRIVER'}))) {
    $driver = 'ODBC';  # default
}
if (defined($ENV{'DBI_DSN'})) {
    $dsn = $ENV{'DBI_DSN'};
}

&debug(2, "driver=$driver dbConnectName=$dsn, dbAccount=$dbaccount,
       dbuser=$dbuser, dbpw=$dbpw");
my $dbh = DBI->connect("dbi:$driver:$dsn", $dbuser, $dbpw);
die "Can't connect -> $DBI::err\n$DBI::errstr\n$DBI::state" unless ($dbh);
$dbh->{RaiseError} = 1;

my $ONE_DAY = 60 * 60 *24;
my $actTime = time;
my $startTime = $actTime - ($lookBack * $ONE_DAY);
my $file;

$actTime = $startTime + $ONE_DAY;
&debug(2, "filePath=$filePath");
&debug(2, "actTime=$actTime, startTime=$startTime");

while ($startTime < $actTime)
{
  my $actDir = `date +$filePath -d\@$startTime`;
  chop ($actDir);
  &debug(2, "actTime=$actTime, startTime=$startTime");
  &debug(2, "actDir=$actDir");

  opendir(my $DH, $actDir) || die "cant open $actDir";
  my $dfile;

  my $sqlstr = "SELECT frid, dsize FROM frameproduct WHERE dir=? AND dfile=?";
  my $sth = $dbh->prepare($sqlstr);          # prepare the query

  #-------------------------------------------------------------------
  # Scan file path for binary files
  #-------------------------------------------------------------------
  while ($dfile = readdir $DH)
  {
     next unless $dfile =~ /(\.1[01]bin|\.mwf)/;
     next unless $dfile !~ /^_U_/;

     my $size = -s "$actDir/$dfile";
     &debug(3,"$actDir/$dfile  $size");
     $sth->execute($actDir, $dfile) or die "Unable to execute: $DBI::errstr";
     my $frid = -1;
     my $dsizeDB = 0;
     my $rows = 0;
     my @dbrow;
     while (@dbrow = $sth->fetchrow_array)
     {  # retrieve one row
        $rows++;
        ($frid, $dsizeDB) = @dbrow;
        &debug(3, "frid=$frid, dsizeDB=$dsizeDB");
     }

     my $time = $actTime - ($actTime % $ONE_DAY);
     my $endtime = $time + $ONE_DAY - 1;

     if ($rows == 0)
     {
        # Check if there is any data frame in the clf file
        if ($dfile =~ /\.1[10]bin/)
        {
           my $foundDataFrame = 0;
           my $clfile = $dfile;
           $clfile =~ s/\.1[10]bin/.clf/;
           open (CLF,"<$actDir/$clfile") or die
                "Can't open $actDir/$clfile for reading";
           while (<CLF>)
           {
              if (/ >[dD] /) { $foundDataFrame = 1; last; }
           }
           close CLF;
           print "$clfile\n" unless $foundDataFrame;
           next unless $foundDataFrame;
        }

        &debug(0, "Warning: orphaned file detected: $actDir/$dfile, skip file");
        next;

        my ($station, undef, undef, $type) = split('\.', $dfile);
        $type = 31 if ($type eq '10bin');
        $type = 32 if ($type eq '11bin');
        $type = 54 if ($type eq 'mwf');

        $frid = &getFrid('frid');
        $sqlstr = "INSERT INTO frameproduct VALUES ($frid, $type, " .
                  "'$actDir', '$dfile', 0, $size, $time, $endtime, " .
                  "'$station', '-', 'ufp-0.99.1', 1, 1, 0, sysdate)",
        &debug(2, $sqlstr);
        my $sth2 = $dbh->prepare($sqlstr);          # prepare the query
        $sth2->execute or die "Unable to execute: $DBI::errstr";
     }
     elsif ($rows == 1)
     {
        next if ($dsizeDB == $size);

        &debug(0, "Binary file size mismatch detected: $actDir/$dfile [$dsizeDB/$size]");
        $sqlstr = "update frameproduct set endtime=$endtime, dsize=$size where frid=$frid";
        &debug(2, $sqlstr);
        my $sth2 = $dbh->prepare($sqlstr);          # prepare the query
        $sth2->execute or die "Unable to execute: $DBI::errstr";
     }
     else
     {
        die "Duplicate entry in frameproduct detected for :$actDir/$dfile";
     }
  }
  closedir $DH;

  $startTime += $ONE_DAY;
}

#----------------------------------------------------------------------
# Get ID for frameproduct table
#----------------------------------------------------------------------

sub getFrid()
{
   my $sqlstr = "SELECT keyvalue FROM lastid WHERE keyname='$_[0]' " .
                "FOR UPDATE OF keyvalue WAIT 3";
   my $sth = $dbh->prepare($sqlstr);          # prepare the query
   $sth->execute or die "Unable to execute: $DBI::errstr";

   my $frid = -1;
   ($frid) = ($sth->fetchrow_array)[0];

   if ($frid == -1)
   {
      $dbh->rollback or die "Unable to rollback: $DBI::errstr";
      die "Unable to get frid";
   }

   $frid += 1;

   $sqlstr = "UPDATE lastid SET keyvalue='$frid' ".
             "WHERE keyname=$_[0]";

   $sth = $dbh->prepare($sqlstr);
   $sth->execute or die "Unable to execute: $DBI::errstr";
   $dbh->commit or die "Unable to commit: $DBI::errstr";
   return($frid);
}

exit 0;
