/*
 * FILE NAME  idcstreamlog_topic.h
 *
 *
 * SYNOPSIS
 *      Prototypes and constants for the libidcstreamlog.a clients.
 *
 * AUTHORS
 *
 *      Martin Ertl, Jan Kianicka (2015-11-19)
 *
 * MODIFICATIONS
 *
 */

#ifndef IDCSTREAMLOG_TOPIC_H
#define IDCSTREAMLOG_TOPIC_H

/*
 * Preferred order of includes,
 * according to the Google Style Guide:
 *
 *      1. Public .h file we are extending (if any).
 * 	2. C system files.
 * 	3. Other libraries' .h files.
 * 	4. Our project's .h files.
 *
 * Order files alphabetically within each section.
 */

#include <stddef.h>              /* size_t, ssize_t */
#include <stdio.h>               /* FILE */

#include "idcstreamlog_cookie.h" /* idcstreamlog_cookie_t */
#include "idcstreamlog_logger.h" /* idcstreamlog_logger_t */
#include "idcstreamlog_core.h"   /* idcstreamlog_severity_t */

/*** Type definitions *******************************************************/

/**
 * Abstract base class.
 */
typedef struct idcstreamlog_topic_       idcstreamlog_topic_t;
typedef struct idcstreamlog_topic_class_ idcstreamlog_topic_class_k;

/**
 * Function type.
 */
typedef ssize_t (*idcstreamlog_write_fn)(idcstreamlog_cookie_t *cookie,
					 char const *buf, size_t size);

/*** IdcStreamLog_Topic class ***********************************************/

/* Structure definitions */

/**
 * Virtual function table (vtable) structure.
 */
struct idcstreamlog_topic_class_
{
    /* Creation functions */
    idcstreamlog_topic_t *(*new)(char const *name);
    idcstreamlog_topic_t *(*bless)(idcstreamlog_topic_t *self);
    idcstreamlog_topic_t *(*construct)(idcstreamlog_topic_t *self,
				       char const *name, size_t num_streams);
    idcstreamlog_topic_t *(*destruct)(idcstreamlog_topic_t *self);
    /* Builder functions */
    void (*add_logger)(idcstreamlog_topic_t *self, idcstreamlog_logger_t *l,
		       idcstreamlog_severity_t smin,
		       idcstreamlog_severity_t smax);
    void (*copy_logger)(idcstreamlog_topic_t *self, idcstreamlog_logger_t *l,
			idcstreamlog_topic_t *proto);
    void (*unlink_logger)(idcstreamlog_topic_t *self, idcstreamlog_logger_t *l);
    /* Accessor/setter functions */
    idcstreamlog_logger_t *(*find_logger)(idcstreamlog_topic_t *self,
					  idcstreamlog_logger_t *lin);
    /* Service functions */
    void (*open)(idcstreamlog_topic_t *self);
    void (*close)(idcstreamlog_topic_t *self);
};

/**
 * Object structure.
 */
/* @todo(ME) workaround for SysTopic; make the object private */
struct idcstreamlog_topic_
{
    idcstreamlog_topic_class_k *class;   /* must be first */
    /**
     * Topic attributes; common for all topic types
     */
    char const                 *name;
    char const                 *topic;
    FILE                       **streams;
    idcstreamlog_cookie_t      **cookies;
    idcstreamlog_write_fn      *write_fns;
    size_t                     num_streams;
    void                       *private;
};

/* Global object declarations */

/**
 * Global class object.
 */
extern idcstreamlog_topic_class_k IdcStreamLog_Topic;

/* Interface (selector) function declarations */

/**
 * Virtual destructor.
 */
idcstreamlog_topic_t *idcstreamlog_topic_destruct(idcstreamlog_topic_t *self);

/**
 * Polymorphic de-allocator.
 */
void idcstreamlog_topic_delete(idcstreamlog_topic_t **self);

/**
 * Builder functions.
 */
void idcstreamlog_topic_add_logger(idcstreamlog_topic_t *self,
				   idcstreamlog_logger_t *l,
				   idcstreamlog_severity_t smin,
				   idcstreamlog_severity_t smax);
void idcstreamlog_topic_copy_logger(idcstreamlog_topic_t *self,
				    idcstreamlog_logger_t *l,
				    idcstreamlog_topic_t *proto);
void idcstreamlog_topic_unlink_logger(idcstreamlog_topic_t *self,
				      idcstreamlog_logger_t *l);

/**
 * Accessor/setter functions.
 */
void idcstreamlog_topic_set_topic(idcstreamlog_topic_t *self,
				  char const *topic);
idcstreamlog_logger_t *
idcstreamlog_topic_find_logger(idcstreamlog_topic_t *self,
			       idcstreamlog_logger_t *lin);

/**
 * Service functions.
 */
void idcstreamlog_topic_open(idcstreamlog_topic_t *self);
void idcstreamlog_topic_close(idcstreamlog_topic_t *self);

/* Interface function declarations */

/**
 * Accessor/setter functions.
 */
char const *idcstreamlog_topic_get_name(idcstreamlog_topic_t *self);

#endif
