#ifndef _AUTH_H
#define _AUTH_H

/**
 * @author    Gerald Klinkl
 *
 * Revision 1.55  2016-01-18 GK
 * Add flag to verifyData to signal an unexpected key related
 * error, mostly triggered by unsupported key sizes (see ISI-421)
 *
 * Revision 1.54  2014-08-19 gerald
 * Re-write CRL handling code to support more than one CRL
 *
 * Revision 1.53  2013-06-25 gerald
 * Add function which cleans up library (necessary for OpenSSL
 * engine support)
 */

#include <arpa/inet.h>  /* needed for standard integer types!! */
#include <openssl/dsa.h>
#include <openssl/x509.h>
#include "misc.h"

/* ==== Defines and data structure for key cache ============================ */

#define NBR_ALIASES  3  /**< Number of allowed alias names */

/**
 * structure for a (location specific) public key cache
 *
 * The public key cache structure contains all known public keys for a given
 * location (== station) or root CA. All key caches are linked to
 * a (location specific ) public key chain. Each station should have at
 * least one public key cache assigned. Alias names are used if the
 * data provider does not connect with a real station name (e.g. PPT connects
 * as PPTS and PPTM)
 */
typedef struct _keyCache {
#define LEN_STATION_NAME 8
   char kcName[LEN_STATION_NAME + 1]; /**< name of key cache (station name) */
   char aliasNames[NBR_ALIASES][LEN_STATION_NAME + 1]; /**< aliases for
                                            this station name (stream name) */
   uint16_t nbrOfCachedKeys;   /**< number of cached keys */
   unsigned char  authState;   /**< authentication state, see below */
#define         AUTH_INIT      ((unsigned char)0)
#define         AUTH_NOKEYS    ((unsigned char)1) /**< No keys available */
#define         AUTH_NONE      ((unsigned char)2) /**< switched off */
#define         AUTH_CACHED    ((unsigned char)3) /**< keys cached */
   unsigned char globalPreLoad; /**< Global preload value, see below */
#define         NO_DIRECTIVE   ((unsigned char)0) /**< No directive; NONE
                                                     directives are stored
                                                     in authState */
#define         PRE_ARM        ((unsigned char)1) /**< Global ARM directive */
#define         PRE_PEM        ((unsigned char)2) /**< Global PEM directive */
#define         PRE_LDAP       ((unsigned char)3) /**< Global LDAP directive */
   time_t lastCacheCheck;      /**< time of last cache check */
   /*@owned@*//*@null@*/
   struct _CDkey *firstKey;    /**< reference to first key in cache */
   /*@owned@*//*@null@*/
   struct _keyCache *nextKeyCache; /**< reference to next public key cache */
} keyCache_t;

/**
 * structure for a public key
 *
 * The public key structure contains the state, the validation period
 * and the state of the key, a link to the next key for that station,
 * and a link to an alternate key (only used for CD-1.0)
 */
typedef struct _CDkey {
#define LEN_SLC_NAME    26   /**< Max.: length of site+location+channel name */
   char  slcName[LEN_SLC_NAME+1];  /**< key search name.  Extracted from
                                        common name */
#define LEN_CN 26
   char  CN[LEN_CN+1];    /**< common name taken from certificate */
   /*@null@*//*@owned@*/
   X509 *cert;            /**< pointer to certificate */
   /*@null@*//*@owned@*/
   X509_CRL *crl;         /**< pointer to CRL; only if root CA certificate */
   EVP_PKEY *pubkey;   /**< pointer to public key retrieved from certificate */
   uint16_t keyType;      /**< type of key, see below */
#define         KEY_NONE        0       /**< Authentication is switch of for
                                             location and/or site */
#define         KEY_ARM         1       /**< Authentication key is located on
                                             local disc, ARM format */
#define         KEY_PEM         2       /**< Authentication key is located on
                                             local disc, PEM format */
#define         KEY_LDAP        3       /**< Authentication key is located on
                                             LDAP server, DEM format */
#define         KEY_ALIAS       4       /**< Used internal for aliases */
   uint32_t keyID;                      /**< ID of the public key */
   uint32_t serialNumber;               /**< Certificates serial number */
   /* the next 3 values could be modified by the certificate thread
      during lifetime of the receiver if the certificate is revoked */
   double notBefore;         /**< start time certificate */
   double notAfter;          /**< end time certificate */
   char   state;             /**< certificate state, see below */
#define         CERT_UNCHECKED    ('\0') /**< not signed (by our root CA) */
#define         CERT_CHECKED      ('\1') /**< signed by our root CA */
#define         CERT_REVOKED      ('\2') /**< not used yet */
   /*@owned@*//*@null@*/
   struct _CDkey *nextKey;       /**< reference to next public key in cache */
   /*@dependent@*//*@null@*/
   struct _CDkey *alternateKey;  /**< reference to alternate public key
                                      in cache. Same slcName but different
                                      common name, eg. CF-01 CF-02. Only
                                      one alternate key name is currently
                                      supported */
} CDkey_t;

#define NO_KEYID  (uint32_t)0xFFFFFFFF   /**< Invalid keyID in certificate */

/* ==== Macro definitions ================================================== */

/* ARM keys have no validity period */
#define IS_IN_KEYDATE(a,b) (KEY_ARM == (b)->keyType || \
                            KEY_NONE == (b)->keyType || \
                            ((a) >= (b)->notBefore && (a) <= (b)->notAfter))

#define IS_IN_KEYDATE2(a1,a2,b) (KEY_ARM == (b)->keyType || \
                                 KEY_NONE == (b)->keyType || \
                      (((b)->notBefore <= (a1) && (a1) <= (b)->notAfter) || \
                       ((b)->notBefore <= (a2) && (a2) <= (b)->notAfter)))

/* Check for AUTH_NONE if cdreceiver should re-load
   the cache on each connection */
#define HAVE_TO_LOAD_CERTIFCATES(pStation) \
     (AUTH_INIT == (pStation)->keyCache->authState && \
       (0 != ((pStation)->systemConf & CHANNEL_AUTH) || \
        (0 != ((pStation)->systemConf & FRAME_AUTH) && \
         CD1_1 == (pStation)->type)))

/* ==== Function prototypes ================================================ */

extern void initAuthentication();
/*@only@*//*@null@*/
extern EVP_PKEY *loadPrivateKey(const char *privateKeyName);
extern int loadLocalKeys4Station(keyCache_t **keyCache, const char *directory,
                                 const int type, const char *stationName,
                                 uint32_t logLoading);
extern int checkLocalKeys4Station(keyCache_t *keyCache, const char *directory,
                                 const int type, const char *stationName,
                                 uint32_t logLoading);
/*@null@*//*@dependent@*/
extern CDkey_t *getKeyByID(const keyCache_t *keyCache, uint32_t keyID,
                           double timeStamp);
/*@null@*//*@dependent@*/
extern CDkey_t *getKeyBySerial(const keyCache_t *keyCache, uint32_t keyID,
                               double timeStamp);
/*@null@*//*@dependent@*/
extern CDkey_t *getKeyByName(const keyCache_t *keyCache,
                             const char *station, const char *site,
                             const char *channel, double timeStamp);
/*@null@*//*@dependent@*/
extern CDkey_t *getKeyByCN(const keyCache_t *keyCache,
                           const char *cN, double sTime, double eTime);
extern int verifyData(EVP_PKEY *publicKey, const char *data, size_t dataLen,
           /*@null@*/const char *signature, const uint32_t sigSize,
           /*@out@*/int *flag);
extern int signData(EVP_PKEY *privateKey, const char *data,
                    size_t dataLen, /*@out@*/ char *signature, size_t *sigLen);
extern int cacheNone(keyCache_t *keyCache, const char *keyName,
          unsigned int numFields, split_t *fields, uint32_t logLoading);

extern int cacheCert(keyCache_t *keyCache, /*@only@*/X509 *cert,
                     const char *stationName, uint32_t logLoading);
extern /*@shared@*/ const char * getAuthType(unsigned char type);
extern uint32_t getKeyIdFromCert(X509 *cert, const char *subject);
extern int initCertChain(uint32_t logLoading);
extern int addAlias(keyCache_t *keyCache, char *stationName);
/*@null@*//*@dependent@*/ extern
keyCache_t *getKeyCache(const char *stationName);
/*@null@*//*@dependent@*/ extern
keyCache_t * checkKeyCache(const char *stationName);
/*@null@*//*@only@*/
extern X509_CRL *loadCRLfromFile(const char *crlPath, const char *Cname,
                                 /*@out@*/int *retVal);
extern void cleanupAuthentication(void);
extern int verifyCrlSignature (X509_CRL *crl2check);
#endif /* _AUTH_H */
