#ifndef _FILEAPI_H
#define _FILEAPI_H

/**
 * @internal
 * @author    Gerald Klinkl
 *
 * Revision 1.25  2014-11-10 GK
 * Add and update function references for doxygen
 *
 * Revision 1.24  2013-23-26 gerald
 * Add syncTimeInterval
 */

#include "schedule.h"
#include "prepare.h"

/* ==== Defines and data structures ========================================= */

#define DUMMY_DC "DUMMY"   /**< name of the dummy data consumer used by the
                                file reader API */

/**
 * Parameters for sender or cd2wng like applications
 */
typedef struct _senderParams
{
   /* ---------------------  read from config file ---------------------------*/
   char      dataConsumerFile[LIBPAR_BUFSIZ]; /**< name of data consumer file */
   char      productionLineFile[LIBPAR_BUFSIZ]; /**< name of production line
                                                     file */
   uint16_t  sleepTimeSched;          /**< sleep time for scheduler thread */
   int32_t   lookBackTimeSched;       /**< scheduler thread look back time for
                                       old data */
   uint16_t  sleepTimeWork;   /**< sleep time for worker threads before check
                                 for new data */
   uint16_t  sleepTimeRetry;  /**< sleep time for worker threads before check
                                 for new data in unconnected state or before
                                 try to re-connect */
   uint16_t  refListStoreInterval; /**< Number of frames sent before store
                                      internal reference list*/
   uint16_t syncTimeInterval; /**< Seconds between syncing wfdisc records */
   char refListStorePath[LIBPAR_BUFSIZ]; /**< path to store internal
                                              reference list */
   char indexFilePath[LIBPAR_BUFSIZ];    /**< File path for index file */
   char dbConnectName[LIBPAR_BUFSIZ];  /**< ODBC connection name */
   char dbConnectPassword[LIBPAR_BUFSIZ];  /**< ODBC connection password */
   char dbAccount[LIBPAR_BUFSIZ];  /**< ODBC connection account */
   char waveformType[LIBPAR_BUFSIZ]; /**< waveform type (s3 or s4, CD2W only) */
   char sensorFile[LIBPAR_BUFSIZ];  /**< sensor file (only for CD2W) */
   char fpIDs[LIBPAR_BUFSIZ];  /**< frame product IDs (only for CD2W) */
   int32_t lookBackTimeWorker;  /**< worker thread look back time (CD2W only) */
   uint16_t sampleRateTolerance; /**< Sample rate tolerance in % (CD2W only) */
   uint32_t channelStatusMask; /**< Mask for channel status bits (CD2W only) */
   uint32_t detectionThreshold; /**< Minimum sample rate for constant data
                                    and sensor noise check (CD2W only) */

   /* ------------------ not read from config file ---------------------------*/
   /*@null@*/
   struct _dcParams    *pDataConsumer; /**< Pointer to data consumer */
   uint16_t  nbrOfdataConsumer;  /**< number of data consumers */
   /*@null@*//*@only@*/
   struct _prodLine    *pProdLine; /**< Pointer to first production line */
   uint16_t nbrOfprodLines;  /**< number of production lines */
   time_t startDate;   /**< Start data fixed interval mode */
   time_t endDate;     /**< End data fixed interval mode */
#define OPER_MODE_AUX         (uint32_t)(1<<0)   /**< run in auxiliary mode */
#define OPER_MODE_CONTINUOUS  (uint32_t)(1<<1)   /**< run in continuous mode */
#define OPER_MODE_FIXED       (uint32_t)(1<<2)   /**< run in fixed interval
                                                      mode */
#define CREATE_WFDISC_RECORDS (uint32_t)(1<<3)   /**< create wfdisc records */
#define CREATE_QUAL_RECORDS   (uint32_t)(1<<4)   /**< create quality records */
#define CREATE_CLF_RECORDS    (uint32_t)(1<<5)   /**< DEPRECATED: create clf
                                                      records in cd2wng */

#define ALL_OPER_MODES (OPER_MODE_AUX | OPER_MODE_CONTINUOUS | OPER_MODE_FIXED)

#define PROCESS_MODE_LIFO     (uint32_t)(1<<8)  /**< send recent data first;
                                                     sorted by nominal time */
#define PROCESS_MODE_FIFO     (uint32_t)(1<<9)  /**< send oldest data first;
                                                     sorted by nominal time */
#define PROCESS_MODE_ORDERED  (uint32_t)(1<<10)  /**< send oldest data first;
                                                     sorted by nominal time;
                                                     send only if no gap  */
#define ALL_PROCESS_MODES (PROCESS_MODE_LIFO | PROCESS_MODE_FIFO | \
                           PROCESS_MODE_ORDERED)
#define BUGGY_ACKNACK_FRAMES  (uint32_t)(1<<11)  /**< sender expects to receive
                                                   buggy ackNack frames. Auto-
                                                   acknowledgement activated */

   uint32_t senderMode;  /**< Senders operating mode */
   /*@null@*//*@owned@*/
   void *sndAPIAppPointer;  /**< Pointer which can be used for application
                                 specific extensions, eg. C2W's DB handle */
   /*@null@*//*@owned@*/
   void *sndAPIAppPointer2; /**< Pointer which can be used for application
                                 specific extensions, eg. C2W's DB handle */
} senderParams_t;

/* ==== Function prototypes ================================================= */

extern /*@dependent@*/configParams_t *
initFileReaderAPI(/*@null@*/const char *configFile, const char *appName);
extern /*@only@*//*@null@*/intRefListParams_t *
getNextFrameDescription(station_t *pStation);
extern int
readFrameByIrl(station_t *pStation, intRefListParams_t *pRefListElement,
               /*@null@*/intRefListParams_t *pLastElementRead, FILE **pfp,
              /*@out@*/uint16_t *event, /*@out@*/int *newDFF);
extern int
readDffByIrl(station_t *pStation, intRefListParams_t *pRefListElement,
               /*@null@*/intRefListParams_t *pLastElementRead, FILE **pfp,
               /*@out@*/uint16_t *event);
extern void
parseFrame(station_t *pStation, uint32_t strictMode, int need2parseDFF,
           long receivedTime, uint32_t bytesRead);

#endif /* _FILEAPI_H */
