#ifndef _MESSAGEAPI_H
#define _MESSAGEAPI_H

/**
 * @internal
 * @author    Gerald Klinkl
 *
 * Revision 1.8  2014-11-03 GK
 * Add and update function references for doxygen
 *
 * Revision 1.7  2014-05-22 GK
 * Add "unsupported data type" error
 */

#include "cd_common.h"

/* ==== Defines and data structures ========================================= */

#define SSI_DRM_DATASIZE_TOO_LARGE  -1
#define ERROR_ALGO                  -2
#define ERROR_INVALID_ARG           -3

#define API_NO_ERROR            0      /**< Request successful */
#define API_W_UNSUPPORTED       (1<<0) /**< Unsupported message block found*/
#define API_W_UNKNOWN           (1<<1) /**< Unknown message block found */
#define API_E_FILE              (1<<2) /**< Can't open/stat file */
#define API_E_NOMEM             (1<<3) /**< Insufficient memory */
#define API_E_FILE_OFFSET       (1<<4) /**< Can't seek to offset in file */
#define API_E_STARTTIME         (1<<5) /**< Invalid start time */
#define API_E_INVALID_WID2      (1<<6) /**< Invalid WID2 block */
#define API_E_NO_STREAM         (1<<7) /**< No stream found for site/chan */
#define API_E_BUFFER_OVERRUN    (1<<8) /**< Output buffer too small */
#define API_E_OPENBUFFER        (1<<9) /**< Can't open buffer:SAPI */
#define API_E_INITCHANNEL       (1<<10) /**< Can't initialize channel:SAPI*/
#define API_E_DECOMPRESS_ERROR  (1<<11) /**< Data decompression failed */
#define API_E_NO_STATION        (1<<12) /**< Station not found in site */
#define API_E_WRITEFRAME        (1<<13) /**< Can't write frame:SAPI */
#define API_E_SETCHANNELDATA    (1<<14) /**< Can't set channel data:SAPI */
#define API_E_NO_BUFFER         (1<<15) /**< Unexpected NULL pointer */
#define API_E_NOT_INITIALIZED   (1<<16) /**< API not initialized */
#define API_W_NO_DATA_FOUND     (1<<17) /**< No data found */
#define API_E_INVALID_CDX       (1<<18) /**< Invalid line in cdx file */
#define API_E_INTERNAL          (1<<19) /**< Internal error */
#define API_E_INVALID_WFDISC    (1<<20) /**< Invalid wfdisc found in DB */
#define API_E_ODBC              (1<<21) /**< Problem with DB (connection) */
#define API_E_INVALID_REQUEST   (1<<22) /**< Invalid waveform request */
#define API_W_DATA_INCOMPLETE   (1<<23) /**< Data not complete available */
#define API_E_NO_CHANNEL        (1<<24) /**< Channel not found in sitechan */
#define API_E_CSF               (1<<25) /**< No CD-1.1 channel subframe */
#define API_E_DATA_TYPE         (1<<26) /**< Unsupported data type */
/* Don't forget to update also CdMsgErrorText[] in messageAPI.c */

/**
 * Channel index structure
 *
 * The Channel index structure is used to reference a particular
 * channel in the binary files
 */

typedef struct _cdx_info
{
#define LEN_CDX_FILENAME    ((64+32)*2)
   char cdx_fileName[LEN_CDX_FILENAME+1]; /**< filename and path for cdx
                      file (twice the size a wfdisc file name length) */
   char cdx_binFile[LEN_CDX_FILENAME+1]; /**< filename and path for binary
                      file (twice the size a wfdisc file name length) */
   size_t cdx_lenPath; /**< Length of the path info in cd_binFile */
   uint32_t cdx_channelOffset; /**< channel offset taken from cdx file */
   int cdx_channelLength; /**< channel length taken from cdx file */
   unsigned int cdx_cpc; /**< channel parsing code taken from cdx file */
   double cdx_startTime; /**< nominal time (CD-1.1) or time stamp (CD-1.0)
                              of channel */
   time_t cdx_fileStartTime; /**< start of day time offset (00:00:00) */
   int cdx_timespan; /**< subframe time length in seconds */
   uint32_t cdx_nextCsfTime; /**< start time next CSF found in cdx file */
   char cdx_dtype[3]; /**< uncompressed data type of channel data */
   float cdx_calibFactor; /**< calibration factor taken from wfdisc */
   float cdx_calibPeriod; /**< calibration period taken from wfdisc */
   long cdx_nbrOfSamples; /**< number of samples in CSF */
   int cdx_returnCode; /**< message API return code */
   int cdx_isFirst; /**< signals that no CSF is read in the current DAT2 block
                         so far */
} cdx_info_t;


/* ==== Function prototypes ================================================= */

extern /*@dependent@*/ configParams_t *
initMessageAPI(const char *configFile);
extern int messageAPI(FILE *fp, int *ret);
extern void
computeChecksum(int *intBuf, int num_of_samples, /*@out@*/int *checksum);
extern int int_to_INT(int lngIn, int *iIn, int lout, /*@out@*/char *cOut);
extern int INT_to_int(int lngIn, char *cBuf, int lout, /*@out@*/int *iout);
extern int int_to_CM6(int lngIn, int *iBuf, int lout, /*@out@*/char *cOut);
extern int CM6_to_int(int lngIn, char *cBuf, int lout, /*@out@*/int *iout);
extern int B64_encode(int lngIn, unsigned char *cIn, int lout,
                      /*@out@*/char *cOut);

extern int getCdxFileName(const char *stream,
                          const char *site, const char *channel,
                          const int channelStartTime, cdx_info_t *pCdxInfo);
extern int getCdxInfo(int channelStartTime, int channelEndTime,
                      cdx_info_t *pCdxInfo, FILE *fp);
/*@dependent@*/extern const char *getMessageErrorAsText(int code);
#endif /* _MESSAGEAPI_H */
