#ifndef _MISC_H
#define _MISC_H

/**
 * @author    Gerald Klinkl
 *
 * Revision 1.48  2014-05-26 gerald
 * Add macro to swap 8 byte values
 *
 * Revision 1.47  2013-10-16 gerald
 * Add generic function to convert broken down UTC time
 * to epoch time (FreeBSD patch)
 *
 */

#include <stdio.h>
#include <string.h>
#include <netinet/in.h>

/* ==== Data structures ===================================================== */

#define MAX_FIELDS    5

typedef struct _split
{
   const char *pString[MAX_FIELDS];   /* pointer to N-th field */
   size_t lenString[MAX_FIELDS];  /* length of N-th field */
   size_t strlen;                 /* number of inspected bytes */
   size_t separators;             /* number of delimiters */
} split_t;

typedef struct _rwFlags
{
   int            sockFlags; /* for socket operations     */
   int32_t        timeout;   /* timeout; -1 is infinitive */
   int32_t        resetDataTimeout;   /* timeout; -1 is infinitive */
   /*@null@*//*@shared@*/
   struct timeval *firstTime; /* time of first read/write  */
   /*@dependent@*/ const unsigned char  *shutdownFlag;
} rwFlags_t;

typedef struct _layout  /* used to define structure layout for writing
                           information to a text file */
{
  const char *text;         /* describing text of element */
  int   elementLength;      /* length of element */
#define INT16U  1       /* 16 bit unsigned integer */
#define INT32U  2       /* 32 bit unsigned integer */
#define INT32S  3       /* 32 bit signed integer */
#define INT64S  5       /* 64 bit signed integer */
#define INT16S  6       /* 16 bit signed integer */
#define ARRAY   7       /* Character array, size is in previous int if length
                           is -1 */
#define IP_ADDR 8       /* IP address (32 bit integer)*/
  int type;             /* Type of element, see above */
} layout_t;

typedef struct _translate  /* used to move bits in an integer */
{
  uint32_t inValue;           /* value to translate */
  uint32_t outValue;          /* result of translation */
} translate_t;

#define ONE_DAY         86400

/* ==== Prototypes ========================================================== */

extern int dumpBlock(FILE *fp, const char *pData, uint32_t dataLen);
extern int makeDirectoryDeep(char *dirPath, const int mode, uint16_t hint);
extern void getDateString(time_t time2conv, /*@out@*/ char *dateString,
            size_t size);
extern void getDate(/*@out@*/ char *dateString, size_t size);
/*@dependent@*//*@null@*/
extern const char *getStr(const char *buffer, const char *searchStr);
extern unsigned int splitString(const char *pString, size_t length,
                                char delimiter, /*@out@*/ split_t *result);
extern int checkDirectory( const char *dirName);
extern int createSocket();
extern int getListenSocket(in_addr_t portAddr, uint16_t portNumber,
                           int maxRequests);
extern int readWithTimeout(int sd, /*@out@*/char *buffer, int len,
                               rwFlags_t *flags);
extern int writeWithTimeout(int sd, const char *buffer, int len,
           rwFlags_t *flags);
extern int getAddressInfo(int sd, /*@out@*/const char **IPaddr,
                          /*@out@*/uint16_t *port);
extern int waitForConnect(int sd, /*@out@*/ struct sockaddr_in *from,
                          int32_t timeout, const unsigned char *shutdownFlag);
extern void CRCinit();
extern uint64_t CRC64(const unsigned char *buffer, const uint32_t len);
extern void double2ascDate(double dDate, FILE *fp);
extern void double2ascDateOut(double dDate, /*@out@*/char *buffer);
extern time_t ascDate2int(const char *aDate, const char *aFormat);
extern double ascDate2double(const char *aDate);
extern double ascDate2double2(const char *aDate, const char *format);
extern void double2ascDateOutF1(double dDate, /*@out@*/char *buffer);
extern long getJdateAsNumber(double time);
extern int checkStrZero(const char *buffer, int length);
extern int checkAlphaNum(const char *buffer, uint32_t length);
extern int checkNum(const char *buffer, uint32_t length);
extern int connectWithTimeout(int sd, in_addr_t remoteIPaddr,
                              uint16_t remotePort, rwFlags_t *flags,
                              int *sockVal);
extern void getTimeStamp(/*@out@*/char *buffer);
extern void getToday(/*@out@*/char *buffer);
extern void writeLayout2File(/*@dependent@*/ const char *buffer, int bufferLen,
                             layout_t *layoutDesc, size_t tableSize, FILE *fp);
extern int raiseFdLimit();
extern void delayedClose(int sd);
extern uint64_t getSequenceNbr();
extern int hasExtension(const char *fileName, const char *extension);
extern time_t cd_timegm(struct tm *tm_time);
/*@dependent@*/ extern const char *
getTextByIndex(size_t index, const char *textArray[], size_t textEntries);
extern uint32_t translateInt(uint32_t inInt, translate_t ttable[],
                             size_t tblSize);

/* ==== Some defines and macros              ================================ */

/* make x divisible by 4 */
#define PAD(x) ((x) % 4 == 0 ? (x) : (x) + (4 - ((x) % 4)))
/* make x divisible by 20 */
#define PAD20(x) ((x) % 20 == 0 ? (x) : (x) + (20 - ((x) % 20)))

#define CONVERT2DOUBLE(in, tmp, out) \
        memcpy(&(tmp), (in), sizeof(uint64_t));\
        tmp = ntohll((tmp)); \
        (out) = 0.0; \
        memcpy(&(out), &(tmp), sizeof(double));

#define SET_NOMINAL_TIME(nominalTime, tmp, out) \
        (tmp) = 0; \
        memcpy(&(tmp), &(nominalTime), sizeof(double)); \
        tmp = htonll((tmp)); \
        memcpy(out, &(tmp), sizeof(uint64_t));

#define REMOVE_NL(buffer) if (NULL != (buffer)) \
      {  \
         int lng; \
         lng = (int)strlen((buffer)) - 1; \
         if ('\n' == (buffer)[lng])  \
         { \
             buffer[lng] = '\0'; /* overwrite LF in string */ \
         }\
      }

#define SD_NONE       ((unsigned char) 0) /* no shutdown pending*/
#define SD_NORMAL     ((unsigned char) 1) /* normal shutdown */
#define SD_FAST       ((unsigned char) 2) /* fast shutdown */
#define SD_STOP       ((unsigned char) 3) /* stop request (similar normal) */

/* ==== Add define to swap long long on little endian architectures ========= */

#ifdef WORDS_BIGENDIAN
 #define ntohll(x)        (x)
#else
 #ifdef HAVE_BYTESWAP_H
  #define ntohll(x)        __bswap_64(x)
 #else
  #define ntohll(x) \
     ((((x) & 0xff00000000000000ull) >> 56) \
      | (((x) & 0x00ff000000000000ull) >> 40) \
      | (((x) & 0x0000ff0000000000ull) >> 24) \
      | (((x) & 0x000000ff00000000ull) >> 8) \
      | (((x) & 0x00000000ff000000ull) << 8) \
      | (((x) & 0x0000000000ff0000ull) << 24) \
      | (((x) & 0x000000000000ff00ull) << 40) \
      | (((x) & 0x00000000000000ffull) << 56))
 #endif
#endif /* WORDS_BIGENDIAN */

#define htonll(x)  ntohll(x)

#define SWAP32(x) \
     ((((x) & 0xff000000ul) >> 24) \
      | (((x) & 0x00ff0000ul) >> 8) \
      | (((x) & 0x0000ff00ul) << 8) \
      | (((x) & 0x000000fful) << 24))

#define SWAP64(x) \
     ((((x) & 0xff00000000000000ull) >> 56) \
      | (((x) & 0x00ff000000000000ull) >> 40) \
      | (((x) & 0x0000ff0000000000ull) >> 24) \
      | (((x) & 0x000000ff00000000ull) >> 8) \
      | (((x) & 0x00000000ff000000ull) << 8) \
      | (((x) & 0x0000000000ff0000ull) << 24) \
      | (((x) & 0x000000000000ff00ull) << 40) \
      | (((x) & 0x00000000000000ffull) << 56))
/*
 * If you encounter problems with PXA255 cpu based Motavista Linux,
 * which can be configured to use middle endian, try the ntholl()
 * definition sent by Geunyoung Kim below
 *
 * #define ntohll(x) \
 *    (__extension__ ( { union \
 *        { __extension__ unsigned long long int __ll; \
 *          unsigned long int __l[2];  \
 *        } __w, __r; \
 *        __w.__ll = (x); \
 *        __r.__l[0] = __bswap_32 (__w.__l[0]); \
 *        __r.__l[1] = __bswap_32 (__w.__l[1]); \
 *       __r.__ll; }))
 */

#endif /* _MISC_H */
