
/*******************************************************/
/*  International Data Centre                          */
/*  Comprehensive Test Ban Treaty Organization         */
/*  Vienna                                             */
/*  Austria                                            */
/*******************************************************/

/**
 * @author    Johannes Schreitl
 *
 * $Log: prepare.h,v $
 *
 * Revision 1.17  2014-02-19 GK
 * Move function deleteWithThatFileObject() used by cd2wng
 * and cdsend to library
 *
 * Revision 1.16  2013-04-09 gerald
 * Fix include dependencies
 *
 * Revision 1.15  2013-04-08 gerald
 * Fix frame acknowledgement when last frame sent is
 * not acknowledged. Fixes also sending to many frames
 * when restart and buggyAckNackFrames is OFF
 *
 * Revision 1.14  2011-12-05 17:01:37  ctbto
 * Add unacknowledged frames high water mark to internal
 * reference list header
 *
 * Revision 1.13  2010-10-13 11:41:16  gerald
 * Fix wrong comment
 *
 * Revision 1.12  2009-11-06 16:10:38  gerald
 * Add channel mask and frame time length to internal
 * reference list structure and remember the bitmask for
 * all channels (read from the clf file) in the internal
 * reference list header (for timely ordered sending mode)
 *
 * Revision 1.11  2009-07-20 12:51:01  gerald
 * Move openIntRefList() from sender and cd2w code to libCD
 *
 * Revision 1.10  2009-04-06 15:00:28  gerald
 * Fix sending/processing frames outside the given
 * interval when running in fixed interval mode
 *
 * Revision 1.9  2009-01-22 15:19:59  gerald
 * Move saveIntRefList() from sender/cd2w code to libCD and
 * fix creating a file object for each line in the re-start file
 *
 * Revision 1.8  2008/07/28 08:29:03  gerald
 * Add file ID to clfFileObj structure (required by cd2w)
 *
 * Revision 1.7  2007/09/13 16:01:43  gerald
 * Increase maximum number of frames which are allowed being
 * queued for sending to cover 14 days (10 frames per minute)
 * (requested by Gonzalo)
 *
 * Revision 1.6  2007/07/16 08:15:34  gerald
 * Keep internal reference list bounded to seven days
 * (with a maximum of 10 frames/minute)
 *
 * Revision 1.5  2007/06/28 07:34:52  gerald
 * Report all incoming frames at the file reader API and report
 * also the frame parsing code
 *
 * Revision 1.4  2007/04/04 12:09:49  gerald
 * Fix too short format string when reading the restart file
 *
 * Revision 1.3  2007/02/19 13:59:18  gerald
 * Add new define for adding also virtual connection event
 * frames to IRL
 *
 * Revision 1.2  2007/01/24 09:05:50  gerald
 * Add receive/send time span to file reader API
 *
 * Revision 1.1  2006/03/28 08:58:54  gerald
 * Move functions used previously only by cd sender to libCD. These
 * functions are needed by the new Data Parsing Programming Interface
 *
 * Revision 1.26  2006/03/07 12:18:40  gerald
 * Continue work for Data Parsing Programming Interface. Rework
 * parseClfFile() and other fucntions to become more independent
 * from CDsender
 *
 * Revision 1.25  2005/10/12 12:36:13  gerald
 * Remove static declaration from some functions used in db_access.c
 * and try to read clf entries from the DB when the clf file can't
 * be opened
 *
 * Revision 1.24  2005/10/05 12:22:14  gerald
 * Add comments and fix wrong size of frameset when scanning a
 * clf file
 *
 * Revision 1.23  2005/09/19 13:01:47  gerald
 * Remove comment signs
 *
 * Revision 1.22  2005/09/14 13:48:04  gerald
 * Change debug level of messages when a frame is unlinked
 * from the send history list
 *
 * Revision 1.21  2005/09/14 13:41:55  gerald
 * Add macro to unlink a frame from the sending history list
 *
 * Revision 1.20  2005/07/07 14:30:46  gerald
 * Move ascDate2int to libCD
 *
 * Revision 1.19  2005/06/16 07:58:17  gerald
 * Add additional forward and backward links to the internal reference list.
 * These links are used to store the sending order necessary for the
 * go back N frames feature
 *
 * Revision 1.18  2005/06/06 11:44:43  gerald
 * Add frame sent count
 *
 * Revision 1.17  2005/05/20 09:05:31  gerald
 * Fix sending too many frames during restart when sender was in
 * LIFO or FIFO mode
 *
 * Revision 1.16  2005/03/07 12:50:59  gerald
 * Add moveFrame2top() function and one new irl frame state 'requested,
 * not sent) needed for 'data request' command
 *
 * Revision 1.15  2004/11/08 16:54:13  gerald
 * Fix missing DFFs when sorting in steady state
 *
 * Revision 1.14  2004/10/07 17:29:14  gerald
 * Add function prototype for sorting and add some structure
 * members for correct restart after sorting
 *
 * Revision 1.13  2004/09/27 12:09:39  gerald
 * Enhance sendIrlData() to allow sending when frames not sorted by
 * arrival time (first patch for sending sorted by nominal time)
 *
 * Revision 1.12  2004/09/02 08:10:35  gerald
 * Add reference to last intRefList element sent to intRefList
 * header. Rename some intRefList structure members.
 *
 * Revision 1.11  2004/06/08 13:39:51  gerald
 * Fix splint annotation
 *
 * Revision 1.10  2004/05/28 18:09:47  gerald
 * Reduce memory consumption when in catchup phase
 *
 * Revision 1.9  2004/03/26 08:41:55  gerald
 * Fix wrong internal reference list file when in catch up or
 * fixed interval mode and a connection is closed or the sender is
 * stopped before the whole clf file is sent. This makes a format
 * change of the internal reference list file necessary
 *
 * Revision 1.8  2004/03/23 15:37:06  gerald
 * Fix some splint warnings
 *
 * Revision 1.7  2004/03/05 13:59:57  gerald
 * Add re-connect feature and improve CPU time consumption
 * Some code clean-up's
 *
 * Revision 1.6  2004/02/25 12:26:54  gerald
 * Add new function definitions
 *
 * Revision 1.5  2004/02/13 15:10:42  gerald
 * Fix some splint warnings, replace tbs with clf in text and
 * names, add CD conform logging
 *
 * Revision 1.4  2004/01/19 16:09:36  karol
 * Simple continuous mode.
 *
 * Revision 1.3  2003/11/07 10:57:59  gerald
 * Remove own defines and use common defines
 *
 * Revision 1.2  2003/05/19 13:00:25  johannes
 * send data in frame borders
 *
 * Revision 1.1  2003/05/07 06:49:58  johannes
 * first issue
 *
 *
 */

#ifndef _PREPARE_H
#define _PREPARE_H

#include "frames.h"
#include "libpar.h"

#define OLD_FILE             0 /* clf file parsed before shutdown; re-start */
#define RESTART_FILE         1 /* clf file parsed during shutdown; re-start */
#define NEW_FILE             2 /* clf file created after shutdown; re-start */
#define NEW_FILE_EXT         3 /* clf file created after shutdown but add
                                  also virtual connect/disconnect frames;
                                  re-start */

#define UNSORTED             0  /* Sort mode when re-start file is written */
#define FIFO                 1  /* Sort mode when re-start file is written */
#define LIFO                 2  /* Sort mode when re-start file is written */

#if 128 > LIBPAR_BUFSIZ
     /* Oops! Seems that length for file names in restart files has
        been defined too large.  Filename buffer is too small.
        Stop compilation with an error */
     #error Check the file name length of filename and path
#endif

#ifndef NBR_RESENT_FRAMES
#define NBR_RESENT_FRAMES        10 /* Number of frames before last frame
                                       sent which should be re-send after
                                       re-start */
#endif /* NBR_RESENT_FRAMES */

#ifndef MAX_UNACKED_FRAMES
#define MAX_UNACKED_FRAMES     1000 /* Maximum number of unacknowledged frames
                                       before sender starts to
                                       auto-acknowledged */
#endif /* MAX_UNACKED_FRAMES */

#define CONFIGURED_REFLIST_ENTRIES    (10*60*24*14) /* sufficient for 14 days
                                                       if a frame arrives
                                                       every 6 seconds */

/* (last) process mode, last clf file parsed and offset of the last parsed
   frame in that clf file during processing */
#define RESTART_LINE_1_OUT       "%d %s %lu\n"
#define RESTART_LINE_1_IN        "%d %128s %lu\n"

/* clf file of the last frame sent, offset last frame sent and
   nominal time of oldest (LIFO) or newest (FIFO) frame */
#define RESTART_LINE_2_OUT       "%s %lu %f\n"      /* %f and */
#define RESTART_LINE_2_IN        "%128s %lu %lf\n"  /* %lf is correct! */

/* frame set, file name and path, offset in bin file, frame size,
   frame type, receiving time, nominal time, DFF offset or 0, DFF size
   or 0, sent state and send count */
#define RESTART_FRAMELINE_OUT       "%s %s %lu %lu %c %f %f %lu %lu %hu %hu\n"
#define RESTART_FRAMELINE_IN  "%39s %128s %lu %lu %c %lf %lf %lu %lu %hu %hu\n"
#define END_OF_FILE           "__END__"

/* Defines for removing frames from IRL */
#define THAT_FILE_OBJECT (char) 0
#define THAT_DFF         (char) 1

/* Macro for unlink element from sent history list */
#define UNLINK_FROM_SEND_HIST(element) \
   if (NULL != (element)->pHistNext) \
   { \
      DEBUG3(9, "Set for [%s] new prev [%s]", (element)->pHistNext->aFrameset, NULL == (element)->pHistPrev ? "null" : (element)->pHistPrev->aFrameset); \
      (element)->pHistNext->pHistPrev = (element)->pHistPrev; \
   } \
   if (NULL != (element)->pHistPrev) \
   { \
      DEBUG3(9, "Set for [%s] new next [%s]", (element)->pHistPrev->aFrameset, NULL == (element)->pHistNext ? "null" : (element)->pHistNext->aFrameset); \
      (element)->pHistPrev->pHistNext = (element)->pHistNext; \
   } \
   (element)->pHistPrev = NULL; \
   (element)->pHistNext = NULL;

#define S2_DEBUG_HDR()    (void)fprintf(DEBUG_FP, "(SDebug:%.2hu)[%.17s] ", \
  pRefListHead->pDebugLevel == NULL ? getDebugLevel() : \
                                      *(pRefListHead->pDebugLevel), \
                                        pRefListHead->plID)
#define IF_RH_DEBUG(level)        if ((pRefListHead->pDebugLevel != NULL && \
      *(pRefListHead->pDebugLevel) >= level) || \
      (pRefListHead->pDebugLevel == NULL && getDebugLevel() >= level))
#define RH_DEBUG(level,msg)       IF_RH_DEBUG(level)\
        { S2_DEBUG_HDR();\
          (void)fprintf(DEBUG_FP, "%s\n", msg); (void)fflush(DEBUG_FP);}
#define RH_DEBUG2(level,fmt,p1)   IF_RH_DEBUG(level)\
        { S2_DEBUG_HDR();\
          (void)fprintf(DEBUG_FP, fmt,p1); (void)fprintf(DEBUG_FP, "\n");\
          (void)fflush(DEBUG_FP);}
#define RH_DEBUG3(level,fmt,p1,p2)  IF_RH_DEBUG(level)\
        { S2_DEBUG_HDR();\
          (void)fprintf(DEBUG_FP, fmt, p1, p2);\
          (void)fprintf(DEBUG_FP, "\n");\
          (void)fflush(DEBUG_FP);}
#define RH_DEBUG4(level,fmt,p1,p2,p3)  IF_RH_DEBUG(level)\
        { S2_DEBUG_HDR();\
          (void)fprintf(DEBUG_FP, fmt, p1, p2, p3);\
          (void)fprintf(DEBUG_FP, "\n");\
          (void)fflush(DEBUG_FP);}

#define EXTENSION_IRL_BACKUP  "irl_sav" /* Extension for restart backup files */

/* ==== Data structures ===================================================== */

typedef struct _intRefListFile
{
   int refs;                            /* Reference count */
   char fileNameAndPath[LIBPAR_BUFSIZ]; /* Path and Filename without
                                           extension */
   long fid;                            /* File ID used by CD2W */
} intRefListFile_t;

typedef struct _intRefListParams  /* used for internal reference list */
{
#define SZ_FRAMESET   (SZ_FRAME_CREATOR + SZ_FRAME_DESTINATION + 3 + 20)
   char aFrameset[SZ_FRAMESET];   /* Frame number or frame set name */
   /*@dependent@*/
   struct _intRefListFile  *pFileObj;  /* file object for related file name
                                          without extension */
   uint32_t offsetBinFile;        /* offset in bin file */
   uint32_t frameSize;            /* frame size */
   char frameType;                /* frame type: GOOD_DATA, BAD_DATA or all */
   double clfTime;                /* start sending/receiving time in clf file */
   double clfTimeSpan;            /* time spawn for send/receive frame in clf
                                     file */
   double nominalTime;            /* nominal time of frame */
   uint32_t fpc;                  /* frame parsing code */
   uint32_t DFFoffset;            /* offset of DFF in bin file */
   uint32_t DFFsize;              /* size of DFF */
   unsigned short sendState;      /* frame state, see below */
#define         NOT_SENT             ((unsigned short)0)
#define         SENT_NOACK           ((unsigned short)1)
#define         SENT_ACK             ((unsigned short)2)
#define         NOT_SENT_REQUESTED   ((unsigned short)3)
#define         HOLD_BACK            ((unsigned short)4)
#define         SENT_LOST            ((unsigned short)5)
#define         UNLINKED             ((unsigned short)99)
#define         RELEASED             ((unsigned short)100)
   unsigned short sendCount;      /* how many times the frame has been sent */
   uint64_t cm1;                  /* First part of channel mask loaded from */
   uint64_t cm2;                  /* clf file; second part of channel mask */
   uint32_t frameTimeLng;         /* frame time length */
   /*@null@*/
   struct  _intRefListParams *pNextElement; /* pointer to next frame */
   /*@null@*//*@dependent@*/
   struct  _intRefListParams *pPrevElement; /* pointer to previous frame */
   /*@null@*//*@dependent@*/
   struct  _intRefListParams *pHistNext; /* Next element in sending history */
   /*@null@*//*@dependent@*/
   struct  _intRefListParams *pHistPrev; /* Previous element in sending
                                            history */
} intRefListParams_t;


typedef struct _intRefListHead
{
   /*@null@*/
   struct _intRefListParams *pRefListStart;/* pointer to 1. element of list */
   /*@null@*//*@dependent@*/
   struct _intRefListParams *pRefListEnd;  /* pointer to last element of list */
   /*@null@*//*@dependent@*/
   struct _intRefListParams *pLastSent;  /* last element sent. Is needed for
                                            writing re-start info */
   uint32_t nbrOfEntries;                /* number of entries in intRefList */
   /*@dependent@*//*@null@*/
   FILE *currentClfFile;   /* File pointer of current clf file */
   /*@dependent@*//*@null@*/
   intRefListFile_t *pClfFileObj;   /* File object for last clf file
                                       processed */
   uint32_t frameOffset;    /* Offset of last frame seen in clf file */
   /*@dependent@*/
   FILE *intRefListFp;     /* File pointer of internal reference list file */
   uint16_t framesSinceLastSave; /* Number of frames sent since last saving
                                    of intRefList */
   uint32_t unAckedHighMark; /**< threshold for unacknowledged frames */
   uint32_t lastDFFoffset; /* Offset of last DFF in corresponding bin file */
   uint32_t lastDFFsize;   /* size of last DFF in corresponding bin file */
   double restartNominalTime; /* nominal time of oldest (LIFO) or newest
                                 (FIFO/unsorted) data frame sent */
   char plID[LEN_CON_ID+1];       /* name of production line */
   /*@null@*//*@dependent@*/
   unsigned short *pDebugLevel; /* pointer to per station debug level */
   /*@null@*/void *dbdesc;
   uint64_t cm1r;         /* reference channel mask, part 1 */
   uint64_t cm2r;         /* reference channel mask, part 2 */
   double frameEndTime;   /* End time of last frame sent */
} intRefListHead_t;

/* ==== Prototypes ========================================================== */
extern int parseClfFile(const char *pFileName, unsigned char type,
             intRefListHead_t *pRefListHead, double restartNomTime,
             unsigned long offsetRestart, int fileType, int lastMode);
extern void freeRefListElement(intRefListParams_t /*@only@*/*element);
extern int
deleteRefListFrame(intRefListHead_t *pRefListHead,
                   /*@only@*/intRefListParams_t *element);
extern int moveFrame2top(intRefListHead_t *head, intRefListParams_t *element);
extern int insertSavedDFF(intRefListHead_t *pRefListHead, station_t *pStation);
extern void dumpClfList(intRefListHead_t *pRefListHead);
extern void clearIntRefList(intRefListHead_t *pIntRefListHead);
extern void compactIntRefList(intRefListHead_t *pRefListHead);
extern void sortIntRefList(intRefListHead_t *pIntRefListHead,
                           int sortMode);
extern void dumpIRL(intRefListHead_t *pRefListHead);
extern void dumpIRLsendSequence(intRefListHead_t *pRefListHead);
extern int loadIrlEntries(intRefListHead_t *pRefListHead);
extern int addClfLine(const char *buffer,
                      intRefListHead_t *pRefListHead,
                      double restartNomTime, unsigned long offsetRestart,
                      int fileType, int lastMode, time_t startDate,
                      time_t endDate);
extern /*@null@*/intRefListFile_t *createFileObject(const char *fileName);
extern int getUnsentFrames(intRefListHead_t *pRefListHead);
extern /*@null@*/ intRefListParams_t *
getAndUnlinkFirstElement(intRefListHead_t *pRefListHead);
extern void checkClfFileObj(intRefListHead_t *pRefListHead);
extern void saveIntRefList(int sortLevel, uint16_t refListStoreInterval,
                           intRefListHead_t *pRefListHead);
extern int openIntRefList(const char * refListStorePath,
                          intRefListHead_t *pRefListHead, int openBackup);
extern uint16_t acknowledgeFrames(intRefListHead_t *pRefListHead,
                                  CDframeset_t *pFs);
extern /*@null@*/ intRefListParams_t *
deleteWithThatFileObject(intRefListHead_t *pRefListHead,
                         /*@only@*/intRefListParams_t *pStartElement);
extern /*@null@*/ intRefListParams_t *
removeFrameWith(intRefListHead_t *pRefListHead,
                /*@only@*/intRefListParams_t *pStartElement, char mode);
extern /*@null@*/ intRefListParams_t *
removeFrameFromIRL(intRefListHead_t *pRefListHead,
                   /*@only@*/intRefListParams_t *pElement);
#endif /* _PREPARE_H */
