
/*******************************************************/
/*  International Data Centre                          */
/*  Comprehensive Test Ban Treaty Organization         */
/*  Vienna                                             */
/*  Austria                                            */
/*******************************************************/

/**
 * @internal
 * @author    Johannes Schreitl
 *
 * $Log: schedule.h,v $
 *
 * Revision 1.14  2015-03-31 GK
 * Add combined regular expression for sites and channels.
 * Compile regular expression only once.
 *
 * Revision 1.13  2014-11-10 GK
 * Add and update function references for doxygen
 *
 * Revision 1.12  2013-04-03 gerald
 * Support new idcsyslog public header filename (from cbase 1.1.0)
 *
 * Revision 1.11  2013-02-25 gerald
 * Add nextSyncTime
 *
 * Revision 1.10  2011-12-05 17:05:58  ctbto
 * Add frame sets to sender structure
 *
 * Revision 1.9  2009-03-05 17:34:51  gerald
 * Fix splint warnings which occurred after upgrading
 * to new splint version
 *
 * Revision 1.8  2009-01-22 15:14:27  gerald
 * Move getSortLevel() from sender/cd2w code to libCD
 *
 * Revision 1.7  2007-09-13 15:52:19  gerald
 * Fix bug in auxiliary station mode which prevents sending
 * the same requested data twice
 *
 * Revision 1.6  2007/04/04 12:13:03  gerald
 * Remove unused splint annotation
 *
 * Revision 1.5  2007/03/12 13:12:23  gerald
 * Add container for application specific data (reporting tool)
 *
 * Revision 1.4  2007/01/04 12:53:25  gerald
 * Sort index files by date before parsing them. This should fix
 * the sporadic problem that an index file is not parsed when the
 * sender is restarted after a long shutdown period.
 *
 * Change parameter of catIdxDir(). The passed start date is set to
 * the time of the last index file parsed to avoid sorting too many
 * index files when the sender runs for a long time
 *
 * Revision 1.3  2006/10/04 12:08:14  gerald
 * Add doxygen annotations and clf file creation date to
 * clf file list structure
 *
 * Revision 1.2  2006/09/06 12:29:22  gerald
 * Add structure member for worker thread specific configuration value
 *
 * Revision 1.1  2006/03/28 08:58:54  gerald
 * Move functions used previously only by cd sender to libCD. These
 * functions are needed by the new Data Parsing Programming Interface
 *
 * Revision 1.13  2006/03/20 14:09:21  gerald
 * Add internal reference list header to sender structure
 *
 * Revision 1.12  2006/02/27 10:20:10  gerald
 * Move production line and data consumer structure to schedule.h
 * to become more independent from sender code when searching
 * for index files
 *
 * Revision 1.11  2005/10/12 12:38:57  gerald
 * Remove static declaration from appendClfEntry(). This function is needed
 * in db_access.c
 *
 * Revision 1.10  2005/07/14 13:03:08  gerald
 * Move code from main.c to schedule.c
 *
 * Revision 1.9  2005/07/12 12:03:21  gerald
 * Move sender specific variables from station structure into
 * application extension structure. Move variables for data request
 * from production line structure to application extension structure
 * (a production line has no longer a 1:1 relationship to a station object)
 *
 * Revision 1.8  2004/09/27 12:09:39  gerald
 * Enhance sendIrlData() to allow sending when frames not sorted by
 * arrival time (first patch for sending sorted by nominal time)
 *
 * Revision 1.7  2004/03/05 13:52:02  gerald
 * Rename pClfFileParams to pClfFileHead
 *
 * Revision 1.6  2004/02/25 12:30:28  gerald
 * Improved debug and logging messages. Fix missing config.h
 *
 * Revision 1.5  2004/02/13 14:57:19  gerald
 * Replace tbs with clf in function and variable names
 *
 * Revision 1.4  2004/02/03 10:26:17  gerald
 * Remove schedule() function and trailing blanks. Add some
 * log warnings and re-work applyTbsIndexFileList()
 *
 * Revision 1.3  2004/01/19 16:12:31  karol
 * Simple continuous mode.
 *
 * Revision 1.2  2003/04/15 09:21:37  johannes
 * linebreak on pos 80 added
 *
 * Revision 1.1  2003/04/15 08:36:27  johannes
 * first issue
 *
 */

#ifndef _SCHEDULE_H
#define _SCHEDULE_H

#include "cd_common.h"
#include <regex.h>

/* ==== Data structures ===================================================== */
typedef struct _clfFileParams /**< used for clf files */
{
   char *pFile;           /**< Path and Name of clf file without extension */
   unsigned char cdtype;  /**< CD protocol types, see below */
/* INTERNAL_FAILURE is 0 */
#define         UNKNOWN        ((unsigned char)1)
#define         CD1            ((unsigned char)2)
#define         CD1_1          ((unsigned char)3)
   time_t ClfDate;          /**< creation time of clf file */
   /*@only@*/
   struct _clfFileParams *pNextClfFileParams;  /**< pointer to next entry */

} clfFileParams_t;


typedef struct _clfFileList /**< used for per worker thread clf file lists */
{
   int nbrOfClfParams;         /**< number of clf files in list */
   struct _clfFileParams *pClfFileHead; /**< starting point of clf file list */
} clfFileList_t;

typedef struct _cdsend
{
   /*@null@*//*@dependent@*/
   struct _prodLine *pProdLine;   /**< Pointer to production line info */
   /*@null@*/
   struct _clfFileList *pClfFilesList; /**< Pointer to clf file list */
   /*@null@*/
   struct _intRefListHead *pRefListHead; /**< Pointer to head of internal
                                              reference list */
   time_t lastClfSearchTime;        /**< Time of last search for clf files */
   uint32_t requestStartTime;       /**< Nominal start time for requested
                                         data */
   uint16_t requestDuration;        /**< request time duration */
   uint16_t requestProcessMode;     /**< LIFO or FIFO */
   uint32_t sndOpMode;              /**< Sender thread operating mode */
   time_t nextSyncTime;             /**< Time next wfdisc synchronization */
   CDframeSets_t frameSets;         /**< Frame sets */
#define LEN_FLT_REGEXP  256         /**< Max length of regexp */
#define REGEXP_FLT_FIELDS 3
   char *pNewRx[REGEXP_FLT_FIELDS]; /**< Filter strings set by command (tmp) */
   regex_t *pRegExpr[REGEXP_FLT_FIELDS]; /**< Compiled regular expressions */
   /*@null@*/
   void *addAppExtension;        /**< container for additional application
                                       specific data */
} cdsend_t;

typedef struct _dcParams /**< used for data consumers in the config file */
{
   char dcName[LEN_STATION_NAME+1];    /**< name of data consumer */
   char cdIpAddr[16];                  /**< IP address of data consumer */
   uint16_t  portNumber;             /**< well known port of data consumer */
   /*@null@*/
   struct _dcParams *pNextdcParams;  /**< pointer to next data consumer struct*/
} dcParams_t;


#define HAS_CHANNEL_BLACKLISTED(pBlackLine) \
         ('\0' != (pBlackLine)->pRxSite[0] || '\0' != (pBlackLine)->pRxChan[0])

#define NOT_SEND_CH    '\0'
#define SEND_CH        '\1'

typedef struct _prodLine /**< used for production lines in the config file */
{
   /*@dependent@*/
   dcParams_t *pDataConsumer;        /**< link to data consumer */
   char pDpName[LEN_STATION_NAME+1]; /**< name of data provider */
   /*@only@*/
   char *pRxSite;                   /**< regexp of sites */
   /*@only@*/
   char *pRxChan;                   /**< regexp of channels */
   /*@only@*/
   char *pRxSaC;                    /**< regexp of site and channels */
   /*@null@*/
   char *newpRxSite;                 /**< String for pending new filter */
   /*@null@*/
   char *newpRxChan;                 /**< String for pending new filter */
   /*@null@*/
   char *newpRxSaC;                 /**< String for pending new filter */
   /*@null@*/
   struct _prodLine *pNextprodLine;  /**< pointer to next production line */
   int isBlacklisted;                /**< flag if this is a blacklist line */
} prodLine_t;

/* ==== Prototypes ========================================================== */
extern int discardClfList( /*@null@*//*@out@*/clfFileList_t *pClfList);
extern void getDPname(const char *conID, /*@out@*/ char *dpName,
                      size_t lenDPname);
/*@null@*//*@dependent@*/ extern station_t *
initSenderStation(const char *conID, const char *pDcName,
                  /*@dependent@*/prodLine_t *pProdLine);
extern int appendClfEntry(/*@null@*/clfFileList_t *pClfList,
                          const char *pAll, const char *pCdFormat,
                          time_t ClfDate);
extern int catIdxDir(const char *pDirectory, uint16_t portNumber,
                     /*@null@*/prodLine_t *pFirstProdline,
                     uint16_t nbrOfProdlines,
                     time_t *startDate, time_t endDate);
extern int
haveDataToSend(/*@null@*/clfFileParams_t *pCurrentClf, /*@null@*/FILE *fp);
/*@null@*//*@dependent@*/ extern clfFileParams_t *
unlinkClfElement(station_t *pStation, /*@dependent@*/clfFileParams_t *pClfFile,
                 uint16_t *event);
extern int removeTooOld(station_t *pStation, const int olderThan);
extern int getSortLevel(station_t *pStation);
extern int check4FilterProvided(cdsend_t *pCdSend);
extern uint32_t getFilteredChannelMask(cdsend_t *pCdSend,
                uint32_t nbrOfChannels, CDdata_t *DFinfo, char *sendChannel);
extern int checkRegExpr(split_t *result, unsigned int offset,
                        /*@out@*/int *errFld);
#endif /* _SCHEDULE_H */
