#!/usr/bin/python

# *******************************************************
# *  International Data Centre                          *
# *  Comprehensive Test Ban Treaty Organization         *
# *  Vienna                                             *
# *  Austria                                            *
# *******************************************************

##
#  @author    Gerald Klinkl
#
#  $Log: initQual.py,v $
#
#  Revision 1.19  2014-10-02  gerald
#  Add CHAN_STATUS:3, STA_STATUS:3 and STA_STATISTICS:2 reports
#
#  Revision 1.18  2014-01-16  gerald
#  Convert debugLevel to integer at startup. Add
#  experimental digitizerNoiseThreshold configuration parameter
#
#  Revision 1.17  2013-11-29  gerald
#  Fix regexp for splitting account into password and account
#
#  Revision 1.16  2013-11-18  gerald
#  Fix import of SQLite on CentOS 5.9 (Python 2.4.3)
#
#  Revision 1.15  2013-09-19  gerald
#  Try all known sqlite modules (FreeBSD uses a different
#  module name)
#
#  Revision 1.14  2013-08-08  gerald
#  Make cdrecvIndexFilePath optional for all reports except
#  AUTH_STATUS
#
#  Revision 1.13  2013-05-26  ctbto
#  Add combined 'AUTH_STATUS' report and disable AUTH_INTERVAL
#  report.
#
#  Revision 1.12  2013-03-29  ctbto
#  Add AUTH_INTERVAL report
#
#  Revision 1.11  2012-12-18 GK
#  Add Julian start and end date to parameter block
#
#  Revision 1.10  2011-09-29 09:51:17  ctbto
#  Rename CHANNEL_STATISTIC report to CHANNEL_STATISTICS
#
#  Revision 1.9  2011-06-06 13:42:58  ctbto
#  Set empty string as default for configuration parameters
#  not set to avoid stop of cdqual with "None" value error
#
#  Revision 1.8  2011-04-18 14:30:33  gerald
#  Add configuration parameters for flat file mode and load static table
#  in flat file mode
#
#  Revision 1.7  2011-02-10 09:20:47  gerald
#  Add new cdqual report CHAN_STATISTIC
#
#  Revision 1.6  2011-01-18 12:42:13  gerald
#  Add channels to usage
#
#  Revision 1.5  2010-12-13 16:25:14  gerald
#  Add parsing of hours/minutes/seconds to start end end
#  time
#
#  Revision 1.4  2010-11-09 20:55:47  gerald
#  Create report for previous day if no start end end time is
#  provided. Add default for ncdNet configuration parameter
#
#  Revision 1.3  2010-09-28 08:55:55  gerald
#  Add support for non continuous data
#
#  Revision 1.2  2010-09-13 15:21:09  gerald
#  Add support for archived records
#
#  Revision 1.1  2010-09-02 10:02:08  gerald
#  Move squal package files to libsrc/cdqual
#
#  Revision 1.2  2010-07-07 12:27:11  gerald
#  Rename cdsqualParams to cdqualParams
#
#  Revision 1.1  2010-06-14 08:10:39  gerald
#  Add first prototype of cdqual
#
#
#

__version__ = "$Revision: 1.15 $"
version = "1.15"

noOdbc = 0
noSqlLite = 0

from optparse import OptionParser
for actSQLite in ['pysqlite2', 'sqlite3']:
   try:
      _tmp = __import__(actSQLite, globals(), locals(), ['dbapi2', ])
      sqlite3 = _tmp.dbapi2
      noSqlLite = 0
      break
   except:
      noSqlLite = 1

import ConfigParser, os, sys
import re
import time
import calendar
try:
   import pyodbc
except:
   noOdbc = 1
import csv
import os

# -----------------------------------------------------------------------------
# @short Log debug message
#
# @description This function logs a debug message if the requested debug
# @description level is equal or higher than the configured debug level
#
# @param reqDebugLevel   requested debug level
# @param cdqualParams   dictionary containing name value pairs
# @param debugMessage    Debug message
#
# @return nothing
# -----------------------------------------------------------------------------

def debug(reqDebugLevel, cdqualParams, debugMessage):

   if reqDebugLevel <= cdqualParams['debugLevel']:
      print >> sys.stderr, "(Debug%d): %s" % (cdqualParams['debugLevel'],
                                           debugMessage)


# -----------------------------------------------------------------------------
# @short Initialize CDQUAL
#
# @description This function initializes cdqual. It reads the command
# @description line parameters and the configuration file and creates
# @description a dictionary containing name value pairs. In case of an
# @description error it stops the application
#
# @param none
#
# @return dictionary containing name value pairs
# -----------------------------------------------------------------------------

def init():

   # define usage and allowed CLI parameters
   usage = "usage: %prog -c config_file [options] stationOrNetwork[/channel] .."
   parser = OptionParser(usage)
   parser.add_option("-c", "--config", dest="configFile",
                     help="configuration file", metavar="FILE")
   parser.add_option("-v", "--verbose",
                     action="store_true", dest="verbose", default=False,
                     help="print status messages to stdout")
   parser.add_option("-r", "--reporttype", action="append",
#  default=["CHAN_STATUS",], default is always added regardless is seen in CL
                     type="choice",
           choices=["STA_STATUS", "STA_STATUS:2", "STA_STATUS:3",
                    "CHAN_STATUS", "CHAN_STATUS:2", "CHAN_STATUS:3",
                    "OUTAGE", "CHAN_STATISTICS",
                    "STA_STATISTICS", "STA_STATISTICS:2", "AUTH_STATUS"],
                     dest="reportTypes",
        metavar="STA_STATUS|STA_STATUS:2|CHAN_STATUS|CHAN_STATUS:2|\
OUTAGE|CHAN_STATISTICS|STA_STATISTICS|STA_STATISTICS:2|AUTH_STATUS",
                     help="report type for reports, default is channel status")
   parser.add_option("-s", "--starttime", dest="startTime", default = '',
                     help="report start time", metavar="YYYY/MM/DD")
   parser.add_option("-e", "--endtime", dest="endTime", default = '',
                     help="report end time", metavar="YYYY/MM/DD")
   parser.add_option("-o", "--output_format", dest="outFormat",
                     type="choice",
                     choices=["ims", "dsv", "xml", "json"],
                     metavar="ims|dsv|xml|json",
           help="output file format, one of ims/dsv/xml/json, default is xml")
   parser.add_option("-f", "--to_file",
                     action="store_true", dest="fileMode", default=False,
                     help="save report(s) in file(s)")
   parser.add_option("-N", "--no_database",
                     action="store_true", dest="noDB", default=False,
                     help="run in file mode")

   # parse command line parameters
   (options, args) = parser.parse_args()

   # Check if config file is supplied
   if not options.configFile:
      parser.error('configuration file is required')

   if len(args) == 0:
      parser.error('no station/network provided')


   cdqualParams = { 'Version': version }

   if not options.startTime:
      # if no start time is given, start time is begin of previous day
      cdqualParams['startTime'] = int(time.time() / (60*60*24)) * (60*60*24)
      cdqualParams['startTime'] -= (60*60*24)
   else:
      try:
         sTime = time.strptime(options.startTime, "%Y/%m/%d")
      except ValueError:
         sTime = time.strptime(options.startTime, "%Y/%m/%d %H:%M:%S")

      cdqualParams['startTime'] = int(calendar.timegm(sTime))

   cdqualParams['jStartTime'] = int(time.strftime('%Y%j',
                                    time.gmtime(cdqualParams['startTime'])))

   if not options.endTime:
      # if no end time is given, end time start time plus 24 hours
      cdqualParams['endTime'] = cdqualParams['startTime'] + (60*60*24)
   else:
      try:
         sTime = time.strptime(options.endTime, "%Y/%m/%d")
      except ValueError:
         sTime = time.strptime(options.endTime, "%Y/%m/%d  %H:%M:%S")

      cdqualParams['endTime'] = int(calendar.timegm(sTime))

   cdqualParams['jEndTime'] = int(time.strftime('%Y%j',
                                    time.gmtime(cdqualParams['endTime'])))

   # Check start and end time
   if (cdqualParams['endTime'] - cdqualParams['startTime']) <= 0:
      sys.exit("Error: start time <= end time");

   # All times are in UTC times
   if options.verbose:
      print time.asctime(time.gmtime(cdqualParams['startTime']))
      print cdqualParams['jStartTime']
      print time.asctime(time.gmtime(cdqualParams['endTime']))
      print cdqualParams['jEndTime']
      print time.altzone
      print time.daylight

   # Create Directory with all configuration parameters and command line
   # arguments
   cdqualParams['args'] = args
   cdqualParams['reportTypes'] = options.reportTypes
   cdqualParams['outFormat'] = options.outFormat
   cdqualParams['verbose'] = options.verbose
   cdqualParams['fileMode'] = options.fileMode

   # Default arguments
   if not cdqualParams['reportTypes']:
      cdqualParams['reportTypes'] = ('CHAN_STATUS', )
   if not cdqualParams['outFormat']:
      cdqualParams['outFormat'] = 'xml'

   # Read the configuration file and store parameter found in the
   # configuration file in cdqualParams
   readConfigFile(options.configFile, cdqualParams, options)

   # Check if cdrecvIndexFilePath is set if report type is AUTH_STATUS
   for actType in cdqualParams['reportTypes']:
       if actType == 'AUTH_STATUS' and \
          cdqualParams['cdrecvIndexFilePath'] == '':
          sys.exit("Error: no required value provided for '%s'" % \
                    'cdrecvIndexFilePath')

   # Register IMS DSV dialect which must be a subclass of Dialect
   class imsDialect(csv.Dialect):
      delimiter = cdqualParams['dsvDelimiter']
      quotechar = '"'
      doublequote = False
      escapechar = '\\'
      lineterminator = '\n'
      quoting = csv.QUOTE_NONE
      skipinitialspace = False

   csv.register_dialect('ims', imsDialect)

   # Supply values for database connection
   dbConnections = [['dbConnectPassword', 'dbAccount',
                     'dbConnectName', 'dbConOps'],
                    ['dbConnectPasswordArchive', 'dbAccountArchive',
                     'dbConnectNameArchive', 'dbConArchive']]

   if options.noDB:
      if noSqlLite:
         sys.exit("Error: running in file mode without python-sqlite installed")
      # Load static tables from files into sqlLite in memory database
      loadStaticTables(cdqualParams)
      cdqualParams['noDB'] = 1
      return cdqualParams
   else:
      cdqualParams['noDB'] = 0
      if noOdbc:
         sys.exit("Error: running in DB mode without pyodbc installed")

   for actDB in dbConnections:
      # Open database connection
      try:
         if cdqualParams[actDB[0]]:
            passwd = cdqualParams[actDB[0]]
            account = cdqualParams[actDB[1]]
      except KeyError:
         # split account into account and password if password is not set
         matchObj = re.search( r'([^/]+)/(.*)', cdqualParams[actDB[1]])
         if matchObj:
            account = matchObj.group(1)
            passwd = matchObj.group(2)
         else:
            sys.exit("Error: database account not set correct '%s'" %
                  cdqualParams[actDB[1]])

      dbc = pyodbc.connect(dsn=cdqualParams[actDB[2]], uid=account,
                           pwd=passwd)
      cdqualParams[actDB[3]] = dbc

   return cdqualParams

# -----------------------------------------------------------------------------
# @short Load static tables
#
# @description This function loads static tables from files
#
# @param cdqualParams dictionary containing name value pairs
#
# @return nothing
# -----------------------------------------------------------------------------

def loadStaticTables(cdqualParams):

   # Load static tables
   sTableFile = cdqualParams['staticTablesFile']
   ext = sTableFile.rfind('.')
   if -1 == ext:
      sTableDb = sTableFile + '.db';
   else:
      sTableDb = sTableFile[:ext] + '.db';

   sTableFile2 = cdqualParams['nonStaticTablesFile']

   ops = sqlite3.connect(sTableDb)
   if os.stat(sTableDb).st_size == 0 or \
      os.stat(sTableDb).st_mtime < os.stat(sTableFile).st_mtime:
      # Create static tables if they do not exist
      debug(1, cdqualParams, 'Create static tables')
      try:
         fh = open(sTableFile, 'rb')
         lines = fh.read()
         cur = ops.cursor()
         cur.executescript(lines)
         # ops.commit()
         fh.close()
         fh = open(sTableFile2, 'rb')
         lines = fh.read()
         lines = re.sub('sysdate',"date('now')", lines)
         ops.executescript(lines)
         # ops.commit()
         fh.close()

      except IOError, why:
         sys.exit("Error: can't open or read table file: %s" % why)

   cdqualParams['dbConOps'] = ops

   # Create tables used by cd2wng in memory
   arch = sqlite3.connect(':memory:')
   # Open the table definition file
   try:
      fh = open(sTableFile2, 'rb')
      lines = fh.read()
      lines = re.sub('sysdate',"date('now')", lines)
      # Remove the not NULL constraints to avoid blowing up
      # in memory DB unnecessary
      lines = re.sub(' NOT NULL','', lines)
      cur = arch.cursor()
      arch.executescript(lines)
      # ops.commit()
      fh.close()

   except IOError, why:
      sys.exit("Error: can't open or read cd2wng table file: %s" % why)

   cdqualParams['dbConArchive'] = arch


# -----------------------------------------------------------------------------
# @short Read configuration file
#
# @description This function reads and parses a configuration file
#
# @param configFile    name of the configuration file
# @param cdqualParams dictionary containing name value pairs
#
# @return dictionary containing name value pairs
# -----------------------------------------------------------------------------

def readConfigFile(configFile, cdqualParams, options):

   configParamsRequired = ( 'dbConnectName', 'dbAccount',
                            'dbConnectNameArchive', 'dbAccountArchive' )
   configParamsOptional = { 'staticTablesPrefix': '',
                            'ncdNet': 'CUR_AUX',
                            'dsvDelimiter': ';',
                            'schemaPrefix': '',
                            'debugLevel': 0,
                            'cdrecvIndexFilePath' : '',
                            'filePath': '',
                            'staticTablesFile': '',
                            'nonStaticTablesFile': '',
                            'digitizerNoiseThreshold': 5,
                            'userGroupPrivileges': 'READONLY',
                            'worldPrivileges': 'READONLY' }

   # Open and read the configuration file
   try:
      fh = open(configFile, 'rb')
      lines = fh.readlines()
      for line in lines:
         # Skip comment or empty line
         if line[0] == '#' or line[0] == '\n':
            continue
         matchObj = re.search( r'(\w+)=(.*)', line, re.X)
         if matchObj:
            if matchObj.group(2)[:2] == '$(': # check for shell variable
               ev =  matchObj.group(2)[2:-1]
               cdqualParams.update({matchObj.group(1): os.environ.get(ev)})
            else:
               cdqualParams.update({matchObj.group(1): matchObj.group(2)})
         else:
            print "Warning: line with invalid syntax found [%s]" % line[:-1]
      fh.close()
   except IOError, why:
      sys.exit("Error: can't open or read configuration file: %s" % why)

   # Check if database is used
   if options.noDB:
      for file in ['staticTablesFile', 'nonStaticTablesFile', 'filePath']:
         try:
            if not cdqualParams[file]:
               sys.exit("Error: no required value provided for '%s'" % file)
         except KeyError:
            sys.exit("Error: no required value provided for '%s'" % file)
   else:
      for configParam in configParamsRequired:
         try:
            if not cdqualParams[configParam]:
               sys.exit("Error: no required value provided for '%s'" % \
                        configParam)
         except KeyError:
            sys.exit("Error: can't find '%s' in configuration file" % \
                     configParam)


   for configParam in configParamsOptional:
      try:
         if not cdqualParams[configParam]:
            if '' != configParamsOptional[configParam]:
               sys.exit("Error while reading '%s'" % configParam)
            else:
               # Set empty string to avoid None values
               cdqualParams[configParam] = ''
      except KeyError:
         cdqualParams.update({configParam: configParamsOptional[configParam]})
         if options.verbose:
            print "Info: no value for '%s', using default" % configParam

   # if threshold has been set, is treated as a string later if
   # it is not explicitly converted to int
   cdqualParams['digitizerNoiseThreshold'] = \
               int(cdqualParams['digitizerNoiseThreshold'])
   # same for debug level
   cdqualParams['debugLevel'] = int(cdqualParams['debugLevel'])

   if options.verbose:
      if cdqualParams['debugLevel'] == 0:
          cdqualParams['debugLevel'] = 1
      for configParam in cdqualParams:
          print "Info: %s [%s]" % (configParam, cdqualParams[configParam])

# -----------------------------------------------------------------------------
# @short Read configuration file (old variant)
#
# @description This function reads and parses a configuration file
#
# @param configFile    name of the configuration file
# @param cdqualParams dictionary containing name value pairs
#
# @return dictionary containing name value pairs
# -----------------------------------------------------------------------------

def readConfigFileOld(configFile, cdqualParams):

   configParamsRequired = [ 'dbConnectName', 'dbAccount',
                            'dbConnectNameArchive', 'dbAccountArchive' ]
   configParamsOptional = [ 'staticTablesPrefix', 'schemaPrefix',
                            'userGroupPrivileges', 'worldPrivileges',
                            'dbConnectPassword' ]

   config = ConfigParser.ConfigParser()
   try:
      config.readfp(open(options.configFile))

   except IOError, why:
      print why
      sys.exit("Error: can\'t open or read configuration file")
   except:
      # MissingSectionHeaderError exception
      sys.exit("Error: No section found in configuration file" % configParam)

   for configParam in configParamsRequired:
      try:
         cdqualParams[configParam] = config.get('DEFAULT', configParam);
      except ConfigParser.NoOptionError, why:
         sys.exit("Error: can't find '%s' in configuration file" % configParam)


   for configParam in configParamsOptional:
      try:
         cdqualParams[configParam] = config.get('DEFAULT', configParam);
      except ConfigParser.NoOptionError, why:
         if options.verbose:
            print "Info: no value for '%s', using default" % configParam


