#!/usr/bin/python

# *******************************************************
# *  International Data Centre                          *
# *  Comprehensive Test Ban Treaty Organization         *
# *  Vienna                                             *
# *  Austria                                            *
# *******************************************************

__version__ = "$Revision: 1.66 $"
version = "1.66"

##
#
#  Revision 1.66  2016-04-08 GK
#  Adjust debug levels and debug output to
#  avoid unnecessary debug output. Ensure
#  that one samples time (1/sample_rate) is
#  represented as a float value
#
#  Revision 1.65  2015-12-02  GK
#  Adjust surety status bits as requested by SN
#
#  Revision 1.64  2015-11-30  GK
#  Re-work "prepared statements" to avoid performance drop
#  when DB data type does not match python type in where
#  clause. Bind as string variables.
#
#  Revision 1.63  2015-06-23  GK
#  Use "prepared statement" syntax for SQL operations.
#  pyodbc does not provide an API for prepared statements,
#  but according to the pyodbc documentation, it uses
#  prepared statements internally if possible.
#
#  Revision 1.62  2015-03-10  GK
#  Fix incorrect values except mission capable with and
#  without surety for auxiliary stations if channels
#  are in the state 'requested' forever (ISI-311).
#  Fix exception and add warning if cdqual records
#  are found without related wfdiscs.
#
#  Revision 1.61  2015-02-20  GK
#  Fix incorrect minimum values with surety for auxiliary
#  stations (ISI-291)
#
#  Revision 1.60  2014-11-20  gerald
#  Fix division by zero if no minimal channels left (ISI-270).
#  Fix too high channel count warnings (ISI-272)
#
#  Revision 1.59  2014-10-20  gerald
#  Fix mission capability calculation if not all channels
#  of an auxiliary station have been requested
#
#  Revision 1.58  2014-10-17  gerald
#  More  STA_STATISTICS and STA_STATISTICS:2 ims report alignment fixes
#  Replace *timely with timely* in JSON reports
#
#  Revision 1.57  2014-10-02  gerald
#  Add CHAN_STATUS:3, STA_STATUS:3 and STA_STATISTICS:2 reports
#
#  Revision 1.56  2014-10-01 GK
#  Fix missing blanks in STA_STATISTICS and STA_STATUS:2
#  IMS reports. Fix header in STA_STATISTICS dsv report
#
#  Revision 1.55  2014-09-03 GK
#  Fix checked values in AUTH_STATUS report which were
#  authentication passed before
#
#  Revision 1.54  2014-06-23 GK
#  Fix auxiliary data timeliness broken with 2.4.15
#
#  Revision 1.53  2014-04-15 GK
#  Re-work calculation of timeliness for auxiliary data
#  to work without stored procedure as requested by AS
#
#  Revision 1.52  2014-03-24 GK
#  Fix incorrect constant data detection if not at begin of segment
#  Calculate timeliness of auxiliary data based on information in the
#  request table instead of quality flag
#
#  Revision 1.51  2014-02-03 GK
#  Make clear that 'no channels left' stops cdqual.
#  Move MKO, MWD and MWS from Eligible to Geophysical Channel
#  set, move MDA to All Channel set as requested by PJ (see DCR-1952).
#
#  Revision 1.50  2014-01-16 GK
#  Use staticTablesPrefix for affiliation instead of schemaPrefix
#
#  Revision 1.49  2014-01-16 GK
#  Do not longer check for consecutive 'no input from sensor'
#  gaps in getBadSampleNumber(). This check is done now in cd2wng.
#  Recheck for 'no input from sensor' if the segment has constant
#  data detections  with a configurable digitizerNoiseThreshold
#  to minimize number of constant data gaps if channel data is only
#  noise as requested by AS (see ISI-197)
#
#  Revision 1.48  2013-11-22 GK
#  Fix scanning the same index file twice due to incorrect
#  set startTime. Fix seismic channels ending with '1' or
#  '2' do not contribute to mission values (except BH1 and BH2).
#  Fix incorrect values when not all channels of an auxiliary
#  station have been requested for the same time period.
#
#  Revision 1.47  2013-10-04 GK
#  Add 'inner' and 'other' rule set (DCR-1867) as
#  requested by PJ. Improved warning if no clf file is found.
#  Do not check for 'constant data' if 'no input from sensor'
#  flag is set
#
#  Revision 1.46  2013-09-10 GK
#  Fix stop when a CD1.0 clf file is parsed
#  for AUTH_STATUS report and improve debug messages
#
#  Revision 1.45  2013-08-16 GK
#  Fix over optimized infrasound rule set  and improve warnings
#  if an infrasound rule set is missing
#
#  Revision 1.44  2013-07-15 GK
#  Merge authentication gaps for authentication intervals. Fix
#  incorrect value for 'Timely Data Availability' which
#  was 'Timely Data Received' in STA_STATUS. Fix format of
#  IMS STA_STATISTICS report. Fix incorrect values for
#  eligible channel subset where the timely data availability (88.380%)
#  is larger than the data availability (88.000%) caused by
#  unexpected python data type converting of float DB data types to
#  integers when time difference is an integer
#
#  Revision 1.43  2013-07-15 GK
#  Change layout of AUTH_REPORT for ims and dsv format
#  and internal AUTH_INTERVAL report. Use 0% as
#  default for mission values for auxiliary stations if no
#  request has been made within report period (ISI-119)
#
#  Revision 1.42  2013-07-03 GK
#  Revert change how to compute data received authenticated as
#  requested by PJ. Data received authenticated is computed as
#  data received authenticated relative to data expected to be received
#  (ISI-119/ISI-154)
#
#  Revision 1.41  2013-06-27 GK
#  Fix missing outage/constant segments and station metrics for auxiliary
#  stations (ISI-154)
#
#  Revision 1.40  2013-06-24 GK
#  Fix wrong regexp for eligible T-Phase (ISI-154/ISI-119)
#
#  Revision 1.39  2013-06-21 GK
#  Follow changes in clf file layout for
#  'AUTH_STATUS' report
#
#  Revision 1.38  2013-06-06 GK
#  Improve error message when clf file does not contain
#  channel name frames
#
#  Revision 1.37  2013-05-21 GK
#  Fix XML 'AUTH_STATUS' report
#
#  Revision 1.36  2013-05-21 GK
#  Add combined 'AUTH_STATUS' report and disable AUTH_INTERVAL
#  report
#
#  Revision 1.35  2013-05-13 GK
#  Add 'AUTH_STATUS' (not combined with AUTH_INTERVAL) report.
#
#  Revision 1.34  2013-04-22 GK
#  Add 'AUTH_INTERVAL' report. Increase field size in the
#  IMS CHAN_STATISTICS report for i5 fields to i8
#
#  Revision 1.33  2013-04-18 GK
#  Use time based values also for STA_STATUS report
#  and calculate data received authenticated as ratio
#  of data received authenticated relative to data received instead
#  of data received authenticated relative to data expected to be received
#
#  Revision 1.32  2013-04-07 GK
#  Do not count periods with 'no input from sensor' detections
#  as available data as requested by PJ. Fix no input from sensor
#  and constant data detection algorithm (which was off by
#  one sample). Add number of samples detected as 'sensor
#  noise' to constant samples reported. Fix swapped values for
#  data availability with surety and timely availability
#
#  Revision 1.31  2013-01-09 GK
#  Compute station status values based on time
#  instead based on samples as requested by PJ
#
#  Revision 1.30  2012-12-18 GK
#  Rework rule set code and add more comments
#
#  Revision 1.29  2012-11-06 GK
#  Fix typo (LHR coded instead of LRH). Consider only the vertical
#  components for the eligible channel set if T-phase station.
#  Replace 'surety' with 'Surety' in IMS and DSV reports. Fix
#  loading of mission rule sets for infrasound stations.
#
#  Revision 1.28  2012-10-24 GK
#  Consider also SURETY_GAP_T when computing values with surety
#
#  Revision 1.27  2012-09-14 GK
#  Fix sporadic incorrect minimum values when channels are
#  absent (e.g. data available with surety)
#
#  Revision 1.26  2012-09-12 GK
#  Fix missing samples from cdquality records (which caused
#  0.001% Data Availability with Surety e. g.) caused by
#  incorrect optimization of the select
#
#  Revision 1.25  2012-09-10 GK
#  Add different minimum channel calculation. Fix incorrect
#  no data gaps for overlapped data in wfdiscs
#
#  Revision 1.24  2012-07-25 GK
#  Fix minimum channel calculation. Fix percentage calculation
#  in rule-sets
#
#  Revision 1.23  2012-07-04 GK
#  Fix minimum value calculation for seismic and hydro stations
#
#  Revision 1.22  2012-06-01 GK
#  Fix check for vertical seismic channels
#
#  Revision 1.21  2012-04-03 GK
#  Leave channel list unchanged if no channels are provided
#  in a backup group
#
#  Revision 1.20  2012-03-20 GK
#  Check for more authentication states
#
#  Revision 1.19  2012-03-12 GK
#  Add station specific rule sets (inner, outer, essential and
#  other pressure) and fix channel status bit check. Fix
#  incorrect mission capability value calculation for seismic
#  and infrasound stations
#
#  Revision 1.18  2012-03-06 GK
#  Add V2.0 station and channel status reports
#  Add station statistic report
#  Skip station report requests for channels with warning
#  Fix incorrect formatting in json reports when more than
#  one station is name provided
#
#  Revision 1.17  2012-02-07 GK
#  Start work on cdqual Quality Metrics Enhancements
#
#  Revision 1.16  2012-02-07 GK
#  Speed up second wfdisc select
#
#  Revision 1.15  2011-12-21 11:18:29  ctbto
#  Trim percentage to 100 percent if it is more than 100
#  percent and fix sporadic duplicated constant data gaps shown
#  in OUTAGE reports
#
#  Revision 1.14  2011-11-02 16:07:50  ctbto
#  Remove channels named BDA and LDA from known data channels
#  for infrasound stations (requested by GP)
#
#  Revision 1.13  2011-10-13 14:32:19  ctbto
#  Speed up selects for wfdisc and cdquality records as
#  suggested by JG
#
#  Revision 1.12  2011-09-29 09:51:46  ctbto
#  Rename CHANNEL_STATISTIC report to CHANNEL_STATISTICS. Fix
#  author string in json reports. Fix row header for IMS channel
#  statistics report. Fix missing channel statistics records in
#  cdqual
#
#  Revision 1.11  2011-07-07 18:09:09  ctbto
#  Fix wrong XML duration format in outage report.
#  Remove dependency for xml.dom.ext and add own
#  pretty print XML function because xml.dom.ext
#  is not longer maintained. Change format of
#  maximum expected time to seconds in json
#  and csv reports. Fix invalid XML URLs
#  when called with more than one station
#
#  Revision 1.10  2011-04-18 14:29:51  gerald
#  Add flat file mode. Load content of files into temporary in memory
#  SQL database (SQLite). Add JSON output format. Fix swapped day/month
#  in report start and end time. Fix multiple headers in all report formats
#  if more than one station is reported
#
#  Revision 1.9  2011-02-10 09:20:48  gerald
#  Add new cdqual report CHAN_STATISTICS
#
#  Revision 1.8  2011-01-18 12:41:17  gerald
#  Allow channel names to be added to station/network/array names
#  and check all station/network/array/channel names
#  before a report is created. Do not log virtual frames without
#  -v option
#
#  Revision 1.7  2010-12-13 16:27:45  gerald
#  Query cdquality instead of quality table.
#  Fix s3/s4 conversion on 64 bit systems
#
#  Revision 1.6  2010-11-09 20:57:17  gerald
#  Fix s3/s4 to integer conversion. Change extension for dsv
#  format in file mode from '.dsv' to '.csv' to be able to load
#  the file without renaming in oocalc. Fix constant data
#  detection
#
#  Revision 1.5  2010-09-28 08:58:21  gerald
#  Stop cdqual if the output file in file mode already
#  exists. Add support for non continuous data. Fix
#  date format in XML reports and OUTAGE report.
#  Count extreme short period horizontal seismic channels
#  (E*) for mission capability. Omit SOH channels in
#  station status report
#
#  Revision 1.4  2010-09-20 11:42:06  gerald
#  Fix wrong start and end date in xml output. Change default
#  xml encoding to iso8859-1. Compute real sample rate more
#  accurate
#
#  Revision 1.3  2010-09-13 15:21:09  gerald
#  Add support for archived records
#
#  Revision 1.1  2010-09-02 10:02:08  gerald
#  Move squal package files to libsrc/cdqual
#
#  Revision 1.4  2010-08-26 14:37:07  gerald
#  Add file output to cdqual. Add DSV output format. Add
#  XML output format. Fix incorrect mission capability values.
#  Rename txt output format to ims. Improve station technology
#  detection. Improve mission capability computation,3C detection and
#  gap merging algorithm. Make xml the default output format
#
#  Revision 1.3  2010-07-07 12:44:48  gerald
#  Continue work on STA_STATUS
#
#  Revision 1.2  2010-06-30 09:11:57  gerald
#  Consider late data and constant data when computing mission capability,
#  fix percentage authenticated value
#
#  Revision 1.1  2010-06-14 08:10:39  gerald
#  Add first prototype of cdsqual
#
#
#

import sys
import os
import glob
import re
import time
from datetime import datetime
import csv
## import xml.dom.ext
import xml.dom.minidom
from math import sqrt, ceil
from initQual import debug
## from bitarray import bitarray

CONST_DATA_THRESHOLD = 5        #**< min. number of constant data samples
# DIGITIZER_NOISE_THRESHOLD = 5   Now loaded from configuration file


# ------------- Gap types -------------
NO_GAP_T = 0   #**< Virtual gap for requests
NO_DATA_GAP_T = 1   #**< Missing data gaps
LATE_GAP_T = 2   #**< Late data gaps
BAD_DATA_GAP_T = 4   #**< Bad data gaps
AUTH_FAIL_GAP_T = 8   #**< Unauthenticated data gaps
SURETY_GAP_T = 16   #**< Surety data gaps (caused by some channel attributes)
REQUEST_GAP_T = 32   #**< Requested data start/end time
REQUEST_GAP_TD = 64   #**< Requested data start/end time delayed arrival

AVAIL_GAPS_T = NO_DATA_GAP_T + BAD_DATA_GAP_T
AVAIL_SUR_GAPS_T = AVAIL_GAPS_T + AUTH_FAIL_GAP_T + SURETY_GAP_T
TIM_DATA_GAPS_T = NO_DATA_GAP_T + LATE_GAP_T
TIM_AVAIL_GAPS_T = TIM_DATA_GAPS_T + BAD_DATA_GAP_T
TIM_AVAIL_SUR_GAPS_T = TIM_AVAIL_GAPS_T + AUTH_FAIL_GAP_T + SURETY_GAP_T
MISSION_GAPS_T = TIM_AVAIL_GAPS_T
MISSION_SUR_GAPS_T = TIM_AVAIL_SUR_GAPS_T

ALL_GT = MISSION_SUR_GAPS_T + REQUEST_GAP_T

# ------------- Time types -------------
NO_DATA_TIME = 0 #**< Accumulated time for no data gaps
BAD_DATA_TIME = 1 #**< Accumulated time for no data and bad data gaps
INSECURE_OR_BAD_TIME = 2 #**< Accumulated time for no data, bad
                         #    data, unauthenticated data and
                         #    'insecure' data gaps
LATE_DATA_TIME = 3 #**< Accumulated time for no data and late data gaps
BAD_OR_LATE_TIME = 4 #**< Accumulated time for no data, late data gaps
                     #    and bad data gaps
INSECURE_LATE_BAD_TIME = 5 #**< Accumulated time for no data, bad
                           #    data, late data, unauthenticated data and
                           #    'insecure' data gaps
NOT_MISSION_CAP_DATA_TIME = 6 #**< BAD_OR_LATE_TIME gaps NOT simply summed up
NOT_MISSION_CAP_INC_INSECURE_DATA_TIME = 7 #**< INSECURE_LATE_BAD_TIME gaps NOT
                                           # simply summed up

# Values for STS_* defines have to be the same as in merge.h !!
STS_DEAD_SENSOR_CHANNEL =     (1<<0)
STS_ZEROED_DATA =             (1<<1)
STS_CLIPPED =                 (1<<2)
STS_CALIBRATION_UNDERWAY =    (1<<3)
STS_EQUIP_HOUSING_OPEND =     (1<<8)
STS_DIGI_EQUIP_OPEN =         (1<<9)
STS_VAULT_DOOR_OPEN =         (1<<10)
STS_AUTH_SEAL_BROKEN =        (1<<11)
STS_EQUIP_MOVED =             (1<<12)
STS_CLOCK_DIFF_TOO_LARGE =    (1<<16)
STS_GPS_RECEIVER_OFF =        (1<<17)
STS_GPS_RECEIVER_UNLOCKED =   (1<<18)
STS_DIGI_ANALOG_IN_SHORTEND = (1<<19)
STS_DIGI_CAL_LOOP_BACK =      (1<<20)
STS_MAIN_PWR_FAILURE =        (1<<24)
STS_BACKUP_PWR_UNSTABLE =     (1<<25)

# CD11 channel status [0x1F00] is:
#                equipment housing open
#                digitizing equipment open
#                vault door opened
#                authentication seal broken
#                equipment moved

SURETY_CHANNEL_ATTRIBUTES = STS_EQUIP_HOUSING_OPEND + \
                            STS_DIGI_EQUIP_OPEN + \
                            STS_VAULT_DOOR_OPEN + \
                            STS_AUTH_SEAL_BROKEN + \
                            STS_EQUIP_MOVED


# ------------- Gap list indexes -------------
G_BEG = 0     #**< Gap begin */
G_END = 1     #**< Gap end */
G_CNT = 2     #**< Gap channel count */
G_TYP = 3     #**< Gap Type

# ------------- Quality flag  -------------
QUA_CONST_DATA = 1  # This must be in sync with merge.h !!
QUA_NO_INPUT = 2
QUA_NO_DATA_CHECK = 4 # SOH channels. Data is treated as "good" data
QUA_NOT_TIMELY = 8
QUA_AUTH_FAILED = 16
QUA_NOT_AUTHENTICATED = 32
QUA_AUTH_NO_CERT = 64
QUA_AUTH_NOT_SIGNED = 128
QUA_FR_AUTH_FAILED = 256
QUA_FR_NOT_AUTHENTICATED = 512
QUA_FR_AUTH_NO_CERT = 1024
QUA_FR_AUTH_NOT_SIGNED = 2048

QUA_AUTH_FAILURE = QUA_AUTH_FAILED | QUA_NOT_AUTHENTICATED |\
                   QUA_AUTH_NO_CERT | QUA_AUTH_NOT_SIGNED
QUA_FR_AUTH_FAILURE = QUA_FR_AUTH_FAILED | QUA_FR_NOT_AUTHENTICATED |\
                   QUA_FR_AUTH_NO_CERT | QUA_FR_AUTH_NOT_SIGNED

# ------------- Other define  -------------
ONE_DAY = 86400

# -----------------------------------------------------------------------------
# @short Log gap list
#
# This function logs a gap list if the requested debug
# level is equal or higher than the configured debug level
#
# @param reqDebugLevel   requested debug level
# @param cdqualParams    dictionary containing name value pairs
# @param gapList         list containing headline and gap list
#
# @return nothing
# -----------------------------------------------------------------------------

def debugGaps2(reqDebugLevel, cdqualParams, gapList):

   if reqDebugLevel <= int(cdqualParams['debugLevel']):
      print >> sys.stderr, "(Debug%d): %s" % (int(cdqualParams['debugLevel']),
               gapList[0])
      for gap in gapList[1]:
         print >> sys.stderr, "%7.7f %7.7f %d %d" % (gap[G_BEG], gap[G_END],
                                                     gap[G_CNT], gap[G_TYP])

# -----------------------------------------------------------------------------
# @short Add station status record to DOM
#
# This function adds a station status record to the DOM
#
# @param doc          root XML node
# @param XMLrecord    parent XML node
# @param rowCont      values for row
# @param nS           name space values
#
# @return nothing
# -----------------------------------------------------------------------------

def addStationRecordXML(doc, XMLrecord, rowCont, nS):

   rowNodes = ( ('station', 'Sta', 'station code'),
                ('maxexptime', 'Max_Exp_Time', 'maximum data time possible'),
     ('received', 'Data Received', 'percent of data received at the IDC', '%'),
('available', 'Data Availability', 'percent of data available at the IDC', '%'),
('timelyavailable', 'Timely Data Availability',
 'percent of data timely available at the IDC', '%'),
('missioncapable', 'Mission Capability',
 'percent of data mission capable at the IDC', '%'),
('receivedauthenticated', 'Data Received Authenticated',
 'percent of data received authenticated at the IDC', '%'))

   ii = 0

   XMLrecord.setAttributeNS(nS[0], nS[1] + 'station', rowCont[0])
   # Create nodes for row based on rowNodes table
   for rowNode in rowNodes:
       actNode = doc.createElementNS(nS[0], nS[1] + rowNode[0])
       XMLrecord.appendChild(actNode)
       actNode.setAttributeNS(nS[0], nS[1] + 'title', rowNode[1])
       actNode.setAttributeNS(nS[0], nS[1] + 'description', rowNode[2])
       if len(rowNode) > 3:
          actNode.setAttributeNS(nS[0], nS[1] + 'unit', rowNode[3])
       if ii == 1:
          # Reformat duration for XML
          durationStr = 'P%dDT%dH%dM%dS' % (rowCont[1], rowCont[2],
                                            rowCont[3], rowCont[4])
          actText = doc.createTextNode(durationStr)
          ii = 4
       else:
          actText = doc.createTextNode(str(rowCont[ii]))
       actNode.appendChild(actText)
       ii += 1

# -----------------------------------------------------------------------------
# @short Add V2.0 station status record to DOM
#
# This function adds a V2.0 station status record to the DOM
#
# @param doc          root XML node
# @param XMLrecord    parent XML node
# @param rowCont      values for row
# @param nS           name space values
#
# @return nothing
# -----------------------------------------------------------------------------

def addStationRecord2XML(doc, XMLrecord, rowCont, nS):

   rowNodes = ( ('station', 'Sta', 'station code'),
                ('maxexptime', 'Max_Exp_Time', 'maximum data time possible'))

   subRecords = (
 ('minimumchannels', 'Minimum Channels', 'minimum channels subset'),
 ('geophysicalchannels', 'Geophysical Channels', 'geophysical channels subset'))

   subRowNodes = (
     ('received', 'Data Received', 'percent of data received at the IDC', '%'),
('available', 'Data Availability', 'percent of data available at the IDC', '%'),
('availablewithsurety', 'Data Availability with Surety',
 'percent of data available with surety at the IDC', '%'),
('timelyavailablewithsurety', 'Timely Data Availability with Surety',
 'percent of data timely available at the IDC', '%'),
('missioncapablewithsurety', 'Mission Capability with Surety',
 '"percent of data mission capable with surety at the IDC', '%'))

   ii = 0

   XMLrecord.setAttributeNS(nS[0], nS[1] + 'station', rowCont[0])
   # Create nodes for row based on rowNodes table
   for rowNode in rowNodes:
      actNode = doc.createElementNS(nS[0], nS[1] + rowNode[0])
      XMLrecord.appendChild(actNode)
      actNode.setAttributeNS(nS[0], nS[1] + 'title', rowNode[1])
      actNode.setAttributeNS(nS[0], nS[1] + 'description', rowNode[2])
      if len(rowNode) > 3:
         actNode.setAttributeNS(nS[0], nS[1] + 'unit', rowNode[3])
      if ii == 1:
         # Reformat duration for XML
         durationStr = 'P%dDT%dH%dM%dS' % (rowCont[1], rowCont[2],
                                           rowCont[3], rowCont[4])
         actText = doc.createTextNode(durationStr)
         ii = 4
      else:
         actText = doc.createTextNode(str(rowCont[ii]))
      actNode.appendChild(actText)
      ii += 1

   # Create nodes for subrecords based on subRecords table
   for subRecord in subRecords:
      recordNode = doc.createElementNS(nS[0], nS[1] + subRecord[0])
      XMLrecord.appendChild(recordNode)
      recordNode.setAttributeNS(nS[0], nS[1] + 'title', subRecord[1])
      recordNode.setAttributeNS(nS[0], nS[1] + 'description', subRecord[2])
      # Create nodes for subRowNodes based on subRowNodes table
      ij = 0;
      for subRowNode in subRowNodes:
         if subRecord[0] == 'minimumchannels' and ij < 2:
            # Skip received and available
            ij += 1
            continue
         actNode = doc.createElementNS(nS[0], nS[1] + subRowNode[0])
         recordNode.appendChild(actNode)
         actNode.setAttributeNS(nS[0], nS[1] + 'title', subRowNode[1])
         actNode.setAttributeNS(nS[0], nS[1] + 'description', subRowNode[2])
         if len(subRowNode) > 3:
            actNode.setAttributeNS(nS[0], nS[1] + 'unit', subRowNode[3])
         actText = doc.createTextNode(str(rowCont[ii]))
         actNode.appendChild(actText)
         ii += 1
         ij += 1
         if subRecord[0] != 'minimumchannels' and ij == 3:
            # Skip timelyavailablewithsurety and missioncapablewithsurety
            break


# -----------------------------------------------------------------------------
# @short Add V3.0 station status record to DOM
#
# This function adds a V3.0 station status record to the DOM
#
# @param doc          root XML node
# @param XMLrecord    parent XML node
# @param rowCont      values for row
# @param nS           name space values
#
# @return nothing
# -----------------------------------------------------------------------------

def addStationRecord3XML(doc, XMLrecord, rowCont, nS):

   rowNodes = ( ('station', 'Sta', 'station code'),
                ('maxexptime', 'Max_Exp_Time', 'maximum data time possible'))

   subRecords = (
 ('minimumchannels', 'Minimum Channels', 'minimum channels subset'),
 ('geophysicalchannels', 'Geophysical Channels', 'geophysical channels subset'))

   subRowNodes = (
     ('received', 'Data Received', 'percent of data received at the IDC', '%'),
('availableunauthenticated', 'Data Availability Unauthenticated',
 'percent of data available unauthenticated at the IDC', '%'),
('available', 'Data Availability with Surety',
 'percent of data available with surety at the IDC', '%'),
('timelyavailable', 'Timely Data Availability with Surety',
 'percent of data timely available at the IDC', '%'),
('missioncapable', 'Mission Capability with Surety',
 '"percent of data mission capable with surety at the IDC', '%'))

   ii = 0

   XMLrecord.setAttributeNS(nS[0], nS[1] + 'station', rowCont[0])
   # Create nodes for row based on rowNodes table
   for rowNode in rowNodes:
      actNode = doc.createElementNS(nS[0], nS[1] + rowNode[0])
      XMLrecord.appendChild(actNode)
      actNode.setAttributeNS(nS[0], nS[1] + 'title', rowNode[1])
      actNode.setAttributeNS(nS[0], nS[1] + 'description', rowNode[2])
      if len(rowNode) > 3:
         actNode.setAttributeNS(nS[0], nS[1] + 'unit', rowNode[3])
      if ii == 1:
         # Reformat duration for XML
         durationStr = 'P%dDT%dH%dM%dS' % (rowCont[1], rowCont[2],
                                           rowCont[3], rowCont[4])
         actText = doc.createTextNode(durationStr)
         ii = 4
      else:
         actText = doc.createTextNode(str(rowCont[ii]))
      actNode.appendChild(actText)
      ii += 1

   # Create nodes for subrecords based on subRecords table
   for subRecord in subRecords:
      recordNode = doc.createElementNS(nS[0], nS[1] + subRecord[0])
      XMLrecord.appendChild(recordNode)
      recordNode.setAttributeNS(nS[0], nS[1] + 'title', subRecord[1])
      recordNode.setAttributeNS(nS[0], nS[1] + 'description', subRecord[2])
      # Create nodes for subRowNodes based on subRowNodes table
      ij = 0;
      for subRowNode in subRowNodes:
         if subRecord[0] == 'minimumchannels' and ij < 2:
            # Skip received and available
            ij += 1
            continue
         actNode = doc.createElementNS(nS[0], nS[1] + subRowNode[0])
         recordNode.appendChild(actNode)
         actNode.setAttributeNS(nS[0], nS[1] + 'title', subRowNode[1])
         actNode.setAttributeNS(nS[0], nS[1] + 'description', subRowNode[2])
         if len(subRowNode) > 3:
            actNode.setAttributeNS(nS[0], nS[1] + 'unit', subRowNode[3])
         actText = doc.createTextNode(str(rowCont[ii]))
         actNode.appendChild(actText)
         ii += 1
         ij += 1
         if subRecord[0] != 'minimumchannels' and ij == 3:
            # Skip timelyavailable and missioncapable
            break


# -----------------------------------------------------------------------------
# @short Add station statistic record to DOM
#
# This function adds a station statistic record to the DOM
#
# @param doc          root XML node
# @param XMLrecord    parent XML node
# @param rowCont      values for row
# @param nS           name space values
#
# @return nothing
# -----------------------------------------------------------------------------

def addStatisticRecordXML(doc, XMLrecord, rowCont, nS):

   rowNodes = ( ('station', 'Sta', 'station code'),
                ('maxexptime', 'Max_Exp_Time', 'maximum data time possible'))

   subRecords = (
 ('minimumchannels', 'Minimum Channels', 'minimum channels subset'),
 ('eligiblechannels', 'Eligible Channels', 'eligible channels subset'),
 ('geophysicalchannels', 'Geophysical Channels', 'geophysical channels subset'),
 ('allchannels', 'All Channels', 'all channels'))

   subRowNodes = (
     ('received', 'Data Received', 'percent of data received at the IDC', '%'),
('available', 'Data Availability', 'percent of data available at the IDC', '%'),
('availablewithsurety', 'Data Availability with Surety',
 'percent of data available with surety at the IDC', '%'),
('timelyreceived', 'Timely Data Received',
 'percent of data timely received at the IDC', '%'),
('timelyavailable', 'Timely Data Availability',
 'percent of data timely available at the IDC', '%'),
('timelyavailablewithsurety', 'Timely Data Availability with Surety',
 'percent of data timely available at the IDC', '%'),
('missioncapable', 'Mission Capability',
 'percent of data mission capable at the IDC', '%'),
('missioncapablewithsurety', 'Mission Capability with Surety',
 '"percent of data mission capable with surety at the IDC', '%'))

   ii = 0

   XMLrecord.setAttributeNS(nS[0], nS[1] + 'station', rowCont[0])
   # Create nodes for row based on rowNodes table
   for rowNode in rowNodes:
      actNode = doc.createElementNS(nS[0], nS[1] + rowNode[0])
      XMLrecord.appendChild(actNode)
      actNode.setAttributeNS(nS[0], nS[1] + 'title', rowNode[1])
      actNode.setAttributeNS(nS[0], nS[1] + 'description', rowNode[2])
      if len(rowNode) > 3:
         actNode.setAttributeNS(nS[0], nS[1] + 'unit', rowNode[3])
      if ii == 1:
         # Reformat duration for XML
         durationStr = 'P%dDT%dH%dM%dS' % (rowCont[1], rowCont[2],
                                           rowCont[3], rowCont[4])
         actText = doc.createTextNode(durationStr)
         ii = 4
      else:
         actText = doc.createTextNode(str(rowCont[ii]))
      actNode.appendChild(actText)
      ii += 1

   # Create nodes for subrecords based on subRecords table
   for subRecord in subRecords:
      recordNode = doc.createElementNS(nS[0], nS[1] + subRecord[0])
      XMLrecord.appendChild(recordNode)
      recordNode.setAttributeNS(nS[0], nS[1] + 'title', subRecord[1])
      recordNode.setAttributeNS(nS[0], nS[1] + 'description', subRecord[2])
      # Create nodes for subRowNodes based on subRowNodes table
      ij = 0;
      for subRowNode in subRowNodes:
         actNode = doc.createElementNS(nS[0], nS[1] + subRowNode[0])
         recordNode.appendChild(actNode)
         actNode.setAttributeNS(nS[0], nS[1] + 'title', subRowNode[1])
         actNode.setAttributeNS(nS[0], nS[1] + 'description', subRowNode[2])
         if len(subRowNode) > 3:
            actNode.setAttributeNS(nS[0], nS[1] + 'unit', subRowNode[3])
         actText = doc.createTextNode(str(rowCont[ii]))
         actNode.appendChild(actText)
         ii += 1
         ij += 1
         if subRecord[0] != 'minimumchannels' and ij == 6:
            break

# -----------------------------------------------------------------------------
# @short Add station statistic record V2.0 to DOM
#
# This function adds a station statistic record V2.0 to the DOM
#
# @param doc          root XML node
# @param XMLrecord    parent XML node
# @param rowCont      values for row
# @param nS           name space values
#
# @return nothing
# -----------------------------------------------------------------------------

def addStatisticRecord2XML(doc, XMLrecord, rowCont, nS):

   rowNodes = ( ('station', 'Sta', 'station code'),
                ('maxexptime', 'Max_Exp_Time', 'maximum data time possible'))

   subRecords = (
 ('minimumchannels', 'Minimum Channels', 'minimum channels subset'),
 ('eligiblechannels', 'Eligible Channels', 'eligible channels subset'),
 ('geophysicalchannels', 'Geophysical Channels', 'geophysical channels subset'),
 ('allchannels', 'All Channels', 'all channels'))

   subRowNodes = (
     ('received', 'Data Received', 'percent of data received at the IDC', '%'),
('availableunauthenticated', 'Data Availability Unauthenticated',
 'percent of data available at the IDC unauthenticated', '%'),
('available', 'Data Availability with Surety',
 'percent of data available with surety at the IDC', '%'),
('timelyreceived', 'Timely Data Received',
 'percent of data timely received at the IDC', '%'),
('timelyavailableunauthenticated', 'Timely Data Availability Unauthenticated',
 'percent of data timely available unauthenticated at the IDC', '%'),
('timelyavailable', 'Timely Data Availability with Surety',
 'percent of data timely available at the IDC with surety', '%'),
('missioncapableunauthenticated', 'Mission Capability',
 'percent of data mission capable at the IDC', '%'),
('missioncapable', 'Mission Capability with Surety',
 '"percent of data mission capable with surety at the IDC', '%'))

   ii = 0

   XMLrecord.setAttributeNS(nS[0], nS[1] + 'station', rowCont[0])
   # Create nodes for row based on rowNodes table
   for rowNode in rowNodes:
      actNode = doc.createElementNS(nS[0], nS[1] + rowNode[0])
      XMLrecord.appendChild(actNode)
      actNode.setAttributeNS(nS[0], nS[1] + 'title', rowNode[1])
      actNode.setAttributeNS(nS[0], nS[1] + 'description', rowNode[2])
      if len(rowNode) > 3:
         actNode.setAttributeNS(nS[0], nS[1] + 'unit', rowNode[3])
      if ii == 1:
         # Reformat duration for XML
         durationStr = 'P%dDT%dH%dM%dS' % (rowCont[1], rowCont[2],
                                           rowCont[3], rowCont[4])
         actText = doc.createTextNode(durationStr)
         ii = 4
      else:
         actText = doc.createTextNode(str(rowCont[ii]))
      actNode.appendChild(actText)
      ii += 1

   # Create nodes for subrecords based on subRecords table
   for subRecord in subRecords:
      recordNode = doc.createElementNS(nS[0], nS[1] + subRecord[0])
      XMLrecord.appendChild(recordNode)
      recordNode.setAttributeNS(nS[0], nS[1] + 'title', subRecord[1])
      recordNode.setAttributeNS(nS[0], nS[1] + 'description', subRecord[2])
      # Create nodes for subRowNodes based on subRowNodes table
      ij = 0;
      for subRowNode in subRowNodes:
         actNode = doc.createElementNS(nS[0], nS[1] + subRowNode[0])
         recordNode.appendChild(actNode)
         actNode.setAttributeNS(nS[0], nS[1] + 'title', subRowNode[1])
         actNode.setAttributeNS(nS[0], nS[1] + 'description', subRowNode[2])
         if len(subRowNode) > 3:
            actNode.setAttributeNS(nS[0], nS[1] + 'unit', subRowNode[3])
         actText = doc.createTextNode(str(rowCont[ii]))
         actNode.appendChild(actText)
         ii += 1
         ij += 1
         if subRecord[0] != 'minimumchannels' and ij == 6:
            break

# -----------------------------------------------------------------------------
# @short Add channel statistics record to DOM
#
# This function adds a channel statistics record to the DOM
#
# @param doc          root XML node
# @param XMLrecord    parent XML node
# @param rowCont      values for row
# @param nS           name space values
#
# @return nothing
# -----------------------------------------------------------------------------

def addChannelStatisticsRecordXML(doc, XMLrecord, rowCont, nS):

   rowNodes = ( ('network', 'Net', 'network code'),
                ('station', 'Sta', 'station code'),
                ('channel', 'Chan', 'FDSN channel code'),
     ('wfdiscrecords', 'WFDrecs', 'number of wfdisc records'),
     ('cdqualityrecords', 'CDQrecs', 'number of cdquality records'),
                ('gaps', 'Gaps', 'number of data gaps'),
                ('latesegments', 'Lsegs', 'number of late data segments'),
                ('constantsegments', 'Csegs',
                 'number of constant data segments'),
                ('lateconstantsegments', 'LCsegs',
                 'number of late and constant data segments'),
     ('received', '%_Recvd', 'number of percent of data received at the IDC',
      '%'))

   ii = 0

   XMLrecord.setAttributeNS(nS[0], nS[1] + 'network', rowCont[0])
   XMLrecord.setAttributeNS(nS[0], nS[1] + 'station', rowCont[1])
   XMLrecord.setAttributeNS(nS[0], nS[1] + 'channel', rowCont[2])
   # Create nodes for row based on rowNodes table
   for rowNode in rowNodes:
       actNode = doc.createElementNS(nS[0], nS[1] + rowNode[0])
       XMLrecord.appendChild(actNode)
       actNode.setAttributeNS(nS[0], nS[1] + 'title', rowNode[1])
       actNode.setAttributeNS(nS[0], nS[1] + 'description', rowNode[2])
       if len(rowNode) > 3:
          actNode.setAttributeNS(nS[0], nS[1] + 'unit', rowNode[3])
       actText = doc.createTextNode(str(rowCont[ii]))
       actNode.appendChild(actText)
       ii += 1

# -----------------------------------------------------------------------------
# @short Add channel status record to DOM
#
# This function adds a channel status record to the DOM
#
# @param doc          root XML node
# @param XMLrecord    parent XML node
# @param rowCont      values for row
# @param nS           name space values
# @param cStatArray   list of statistics returned by getChannelStatistics()
#
# @return nothing
# -----------------------------------------------------------------------------

def addAuthRecords(doc, XMLrecord, rowCont, nS, cStatArray):

   rowNodes = (('network', 'Net', 'network code'),
                ('station', 'Sta', 'station code'))
   rowNodesF = (('received', 'Received', 'number of received frames'),
                ('checked', 'Checked', 'number of checked frames'),
                ('failed', 'Failed', 'number of failed frames'))
   rowNodesC = (
     ('received', 'Received', 'number of received channel sub-frames'),
     ('checked', 'Checked', 'number of checked channel sub-frames'),
     ('failed', 'Failed', 'number of failed channel sub-frames'))
   rowNodesI = (
     ('channel', 'Chan', 'FDSN channel code'),
     ('start', 'Start Date Time', 'date/time of last sample before ' +
      'failed interval or start date/time of report period'),
     ('end', 'End Date Time', 'date/time of last sample after failed ' +
      'interval or end date/time of report period'),
     ('comment', 'Comment', 'comment'))

   ii = 0

   XMLrecord.setAttributeNS(nS[0], nS[1] + 'network', rowCont[0])
   XMLrecord.setAttributeNS(nS[0], nS[1] + 'station', rowCont[1])
   # Create nodes for row based on rowNodes table
   for rowNode in rowNodes:
       actNode = doc.createElementNS(nS[0], nS[1] + rowNode[0])
       XMLrecord.appendChild(actNode)
       actNode.setAttributeNS(nS[0], nS[1] + 'title', rowNode[1])
       actNode.setAttributeNS(nS[0], nS[1] + 'description', rowNode[2])
       actText = doc.createTextNode(str(rowCont[ii]))
       actNode.appendChild(actText)
       ii += 1

   # Create frame nodes for row based on rowNodesF table
   XMLrecordF = doc.createElementNS(nS[0], nS[1] + 'frames')
   XMLrecord.appendChild(XMLrecordF)
   XMLrecordF.setAttributeNS(nS[0], nS[1] + 'title', 'Frames')
   XMLrecordF.setAttributeNS(nS[0], nS[1] + 'description', 'frames')
   for rowNode in rowNodesF:
       actNode = doc.createElementNS(nS[0], nS[1] + rowNode[0])
       XMLrecordF.appendChild(actNode)
       actNode.setAttributeNS(nS[0], nS[1] + 'title', rowNode[1])
       actNode.setAttributeNS(nS[0], nS[1] + 'description', rowNode[2])
       actText = doc.createTextNode(str(rowCont[ii]))
       actNode.appendChild(actText)
       ii += 1

   # Create channel sub-frame nodes for row based on rowNodesC table
   XMLrecordC = doc.createElementNS(nS[0], nS[1] + 'channelsubframes')
   XMLrecord.appendChild(XMLrecordC)
   XMLrecordC.setAttributeNS(nS[0], nS[1] + 'title', 'Channel Sub-Frames')
   XMLrecordC.setAttributeNS(nS[0], nS[1] + 'description', 'channel sub-frames')
   for rowNode in rowNodesC:
       actNode = doc.createElementNS(nS[0], nS[1] + rowNode[0])
       XMLrecordC.appendChild(actNode)
       actNode.setAttributeNS(nS[0], nS[1] + 'title', rowNode[1])
       actNode.setAttributeNS(nS[0], nS[1] + 'description', rowNode[2])
       actText = doc.createTextNode(str(rowCont[ii]))
       actNode.appendChild(actText)
       ii += 1

   # Create failed interval nodes for row based on rowNodesI table
   XMLrecordI = doc.createElementNS(nS[0], nS[1] + 'failedintervals')
   XMLrecord.appendChild(XMLrecordI)
   XMLrecordI.setAttributeNS(nS[0], nS[1] + 'title', 'Failed Intervals')
   XMLrecordI.setAttributeNS(nS[0], nS[1] + 'description', 'failed intervals')
   for cStat in cStatArray:
     # rowForOut = "%-4s %s" % (cStat[CHANNEL], "%-20s%03d %-20s%03d %10.3f %s")
     rowForOut = "%-4s %s" % (cStat[CHANNEL], "%-20s%03dZ %-20s%03dZ %s")

     gaps = cStat[AUTH_GAP_LIST]
     for gap in gaps:
        XMLrecordI2 = doc.createElementNS(nS[0], nS[1] + 'failedinterval')
        XMLrecordI.appendChild(XMLrecordI2)
        XMLrecordI2.setAttributeNS(nS[0], nS[1] + 'channel', cStat[CHANNEL])
        actLine = rowForOut % getAuthGapLine(gap)
        lst = actLine.split(None, 5)
        if len(lst) == 5:
           lst.append('')
        lst[1] = lst[1] + 'T' + lst[2]
        lst[3] = lst[3] + 'T' + lst[4]
        # Replace 2013/05/04 with 2013-05-04
        lst[1] = lst[1].replace('/','-')
        lst[3] = lst[3].replace('/','-')
        del lst[4]
        del lst[2]
        ij = 0
        for rowNode in rowNodesI:
           actNode = doc.createElementNS(nS[0], nS[1] + rowNode[0])
           XMLrecordI2.appendChild(actNode)
           actNode.setAttributeNS(nS[0], nS[1] + 'title', rowNode[1])
           actNode.setAttributeNS(nS[0], nS[1] + 'description', rowNode[2])
           actText = doc.createTextNode(str(lst[ij]))
           actNode.appendChild(actText)
           ij += 1


# -----------------------------------------------------------------------------
# @short Add frame status record to DOM
#
# This function adds a frame status record to the DOM
#
# @param doc          root XML node
# @param XMLrecord    parent XML node
# @param rowCont      values for row
# @param nS           name space values
#
# @return nothing
# -----------------------------------------------------------------------------

def addChannelRecordXML(doc, XMLrecord, rowCont, nS):

   rowNodes = ( ('network', 'Net', 'network code'),
                ('station', 'Sta', 'station code'),
                ('channel', 'Chan', 'FDSN channel code'),
     ('received', '%_Recvd', 'percent of data received at the IDC', '%'),
     ('available', '%_Avail', 'percent of data available at the IDC', '%'),
                ('gaps', 'Gaps', 'number of data gaps'),
                ('samples', 'Samples', 'number of samples'),
                ('constant', 'Constant', 'number of constant values'),
                ('mean', 'Mean', 'mean'),
                ('rms', 'RMS', 'root mean square'))

   ii = 0

   XMLrecord.setAttributeNS(nS[0], nS[1] + 'network', rowCont[0])
   XMLrecord.setAttributeNS(nS[0], nS[1] + 'station', rowCont[1])
   XMLrecord.setAttributeNS(nS[0], nS[1] + 'channel', rowCont[2])
   # Create nodes for row based on rowNodes table
   for rowNode in rowNodes:
       actNode = doc.createElementNS(nS[0], nS[1] + rowNode[0])
       XMLrecord.appendChild(actNode)
       actNode.setAttributeNS(nS[0], nS[1] + 'title', rowNode[1])
       actNode.setAttributeNS(nS[0], nS[1] + 'description', rowNode[2])
       if len(rowNode) > 3:
          actNode.setAttributeNS(nS[0], nS[1] + 'unit', rowNode[3])
       actText = doc.createTextNode(str(rowCont[ii]))
       actNode.appendChild(actText)
       ii += 1

# -----------------------------------------------------------------------------
# @short Add channel status V2.0 record to DOM
#
# This function adds a channel status V2.0 record to the DOM
#
# @param doc          root XML node
# @param XMLrecord    parent XML node
# @param rowCont      values for row
# @param nS           name space values
#
# @return nothing
# -----------------------------------------------------------------------------

def addChannelRecord2XML(doc, XMLrecord, rowCont, nS):

   rowNodes = ( ('network', 'Net', 'network code'),
                ('station', 'Sta', 'station code'),
                ('channel', 'Chan', 'FDSN channel code'),
     ('received', '%_Recvd', 'percent of data received at the IDC', '%'),
     ('available', '%_Avail', 'percent of data available at the IDC', '%'),
     ('availablewithsurety', '%__AvaSu',
      'percent of data available with surety at the IDC', '%'),
                ('gaps', 'Gaps', 'number of data gaps'),
                ('samples', 'Samples', 'number of samples'),
                ('constant', 'Constant', 'number of constant values'),
                ('mean', 'Mean', 'mean'),
                ('rms', 'RMS', 'root mean square'))

   ii = 0

   XMLrecord.setAttributeNS(nS[0], nS[1] + 'network', rowCont[0])
   XMLrecord.setAttributeNS(nS[0], nS[1] + 'station', rowCont[1])
   XMLrecord.setAttributeNS(nS[0], nS[1] + 'channel', rowCont[2])
   # Create nodes for row based on rowNodes table
   for rowNode in rowNodes:
       actNode = doc.createElementNS(nS[0], nS[1] + rowNode[0])
       XMLrecord.appendChild(actNode)
       actNode.setAttributeNS(nS[0], nS[1] + 'title', rowNode[1])
       actNode.setAttributeNS(nS[0], nS[1] + 'description', rowNode[2])
       if len(rowNode) > 3:
          actNode.setAttributeNS(nS[0], nS[1] + 'unit', rowNode[3])
       actText = doc.createTextNode(str(rowCont[ii]))
       actNode.appendChild(actText)
       ii += 1

# -----------------------------------------------------------------------------
# @short Add channel status V3.0 record to DOM
#
# This function adds a channel status V3.0 record to the DOM
#
# @param doc          root XML node
# @param XMLrecord    parent XML node
# @param rowCont      values for row
# @param nS           name space values
#
# @return nothing
# -----------------------------------------------------------------------------

def addChannelRecord3XML(doc, XMLrecord, rowCont, nS):

   rowNodes = ( ('network', 'Net', 'network code'),
                ('station', 'Sta', 'station code'),
                ('channel', 'Chan', 'FDSN channel code'),
     ('received', '%_Recvd', 'percent of data received at the IDC', '%'),
     ('availableunauthenticated', '%_AvaUn',
      'percent of data available at the IDC unauthenticated', '%'),
     ('available', '%_Avail',
      'percent of data available with surety at the IDC', '%'),
                ('gaps', 'Gaps', 'number of data gaps'),
                ('samples', 'Samples', 'number of samples'),
                ('constant', 'Constant', 'number of constant values'),
                ('mean', 'Mean', 'mean'),
                ('rms', 'RMS', 'root mean square'))

   ii = 0

   XMLrecord.setAttributeNS(nS[0], nS[1] + 'network', rowCont[0])
   XMLrecord.setAttributeNS(nS[0], nS[1] + 'station', rowCont[1])
   XMLrecord.setAttributeNS(nS[0], nS[1] + 'channel', rowCont[2])
   # Create nodes for row based on rowNodes table
   for rowNode in rowNodes:
       actNode = doc.createElementNS(nS[0], nS[1] + rowNode[0])
       XMLrecord.appendChild(actNode)
       actNode.setAttributeNS(nS[0], nS[1] + 'title', rowNode[1])
       actNode.setAttributeNS(nS[0], nS[1] + 'description', rowNode[2])
       if len(rowNode) > 3:
          actNode.setAttributeNS(nS[0], nS[1] + 'unit', rowNode[3])
       actText = doc.createTextNode(str(rowCont[ii]))
       actNode.appendChild(actText)
       ii += 1

# -----------------------------------------------------------------------------
# @short Add authentication record to DOM
#
# This function adds an authentication status record to the DOM
#
# @param doc          root XML node
# @param XMLrecord    parent XML node
# @param rowCont      values for row
# @param nS           name space values
#
# @return nothing
# -----------------------------------------------------------------------------

def addAuthRecordXML(doc, XMLrecord, rowCont, nS):

   rowNodes = ( ('network', 'Net', 'network code'),
                ('station', 'Sta', 'station code'),
                ('channel', 'Chan', 'FDSN channel code'),
     ('start', 'Start Date Time', 'date/time of last sample \
before outage interval or start date/time of report period'),
     ('end', 'End Date Time', 'date/time of first sample after outage \
interval or end date/time of report period'),
#                ('duration', 'Duration', 'duration of the interval'),
                ('comment', 'Comment', 'comment'))

   ii = 0

   XMLrecord.setAttributeNS(nS[0], nS[1] + 'network', rowCont[0])
   XMLrecord.setAttributeNS(nS[0], nS[1] + 'station', rowCont[1])
   XMLrecord.setAttributeNS(nS[0], nS[1] + 'channel', rowCont[2])
   # Create nodes for row based on rowNodes table
   for rowNode in rowNodes:
       actNode = doc.createElementNS(nS[0], nS[1] + rowNode[0])
       XMLrecord.appendChild(actNode)
       actNode.setAttributeNS(nS[0], nS[1] + 'title', rowNode[1])
       actNode.setAttributeNS(nS[0], nS[1] + 'description', rowNode[2])
       if len(rowNode) > 3:
          actNode.setAttributeNS(nS[0], nS[1] + 'unit', rowNode[3])
       if ii == 3 or ii == 5:
          # Reformat start and end date for XML
          dateS = rowCont[ii].replace('/','-')
          ii += 1
          dateS = dateS + 'T' + rowCont[ii] + 'Z'
          actText = doc.createTextNode(dateS)
#       elif ii == 7:
          # Python 2.4 does not support %f
#          duration = round(float(rowCont[ii]),3)
#          days = int(duration / (60*60*24))
#          rest = duration - (days * 60*60*24)
#          hours = int(rest / (60*60))
#          rest -= hours * 60*60
#          minutes = int(rest/60)
#          rest -= minutes * 60
#          durS = 'P%dDT%dH%dM%sS' % (days, hours, minutes, str(rest))
#          actText = doc.createTextNode(durS)
#       elif ii == 8 and len(rowCont) == 8:
       elif ii == 7 and len(rowCont) == 7:
          # record with empty comment will raise an exception otherwise
          return
       else:
          actText = doc.createTextNode(str(rowCont[ii]))

       actNode.appendChild(actText)
       ii += 1

# -----------------------------------------------------------------------------
# @short Add outage record to DOM
#
# This function adds an outage status record to the DOM
#
# @param doc          root XML node
# @param XMLrecord    parent XML node
# @param rowCont      values for row
# @param nS           name space values
#
# @return nothing
# -----------------------------------------------------------------------------

def addOutageRecordXML(doc, XMLrecord, rowCont, nS):

   rowNodes = ( ('network', 'Net', 'network code'),
                ('station', 'Sta', 'station code'),
                ('channel', 'Chan', 'FDSN channel code'),
     ('start', 'Start Date Time', 'date/time of last sample \
before outage interval or start date/time of report period'),
     ('end', 'End Date Time', 'date/time of first sample after outage \
interval or end date/time of report period'),
                ('duration', 'Duration', 'duration of the interval'),
                ('comment', 'Comment', 'comment'))

   ii = 0

   XMLrecord.setAttributeNS(nS[0], nS[1] + 'network', rowCont[0])
   XMLrecord.setAttributeNS(nS[0], nS[1] + 'station', rowCont[1])
   XMLrecord.setAttributeNS(nS[0], nS[1] + 'channel', rowCont[2])
   # Create nodes for row based on rowNodes table
   for rowNode in rowNodes:
       actNode = doc.createElementNS(nS[0], nS[1] + rowNode[0])
       XMLrecord.appendChild(actNode)
       actNode.setAttributeNS(nS[0], nS[1] + 'title', rowNode[1])
       actNode.setAttributeNS(nS[0], nS[1] + 'description', rowNode[2])
       if len(rowNode) > 3:
          actNode.setAttributeNS(nS[0], nS[1] + 'unit', rowNode[3])
       if ii == 3 or ii == 5:
          # Reformat start and end date for XML
          dateS = rowCont[ii].replace('/','-')
          ii += 1
          dateS = dateS + 'T' + rowCont[ii] + 'Z'
          actText = doc.createTextNode(dateS)
       elif ii == 7:
          # Python 2.4 does not support %f
          duration = round(float(rowCont[ii]),3)
          days = int(duration / (60*60*24))
          rest = duration - (days * 60*60*24)
          hours = int(rest / (60*60))
          rest -= hours * 60*60
          minutes = int(rest/60)
          rest -= minutes * 60
          durS = 'P%dDT%dH%dM%sS' % (days, hours, minutes, str(rest))
          actText = doc.createTextNode(durS)
       elif ii == 8 and len(rowCont) == 8:
          # record with empty comment will raise an exception otherwise
          return
       else:
          actText = doc.createTextNode(str(rowCont[ii]))

       actNode.appendChild(actText)
       ii += 1

# -----------------------------------------------------------------------------
# @short Add Report Time for XML
#
# This function adds the report time period as XML nodes
#
# @param cdqualParams dictionary containing name value pairs
# @param doc          root XML node
# @param XMLreport    parent XML node
# @param nS           name space values
#
# @return nothing
# -----------------------------------------------------------------------------

def addReportTimeXML(cdqualParams, doc, XMLreport, nS):

   # Create to and from nodes
   from_t = doc.createElementNS(nS[0], nS[1] + 'from')
   XMLreport.appendChild(from_t)
   to_t = doc.createElementNS(nS[0], nS[1] + 'to')
   XMLreport.appendChild(to_t)
   # Create to and from text nodes
   from_s = doc.createTextNode(toDateTimeXML(cdqualParams['startTime']))
   from_t.appendChild(from_s)
   to_s = doc.createTextNode(toDateTimeXML(cdqualParams['endTime']))
   to_t.appendChild(to_s)

# -----------------------------------------------------------------------------
# @short Add Report header for XML
#
# This function adds the report header as XML nodes
#
# @param doc          root XML node
# @param cdqualParams dictionary containing name value pairs
# @param nS           name space values
#
# @return nothing
# -----------------------------------------------------------------------------

def addXMLheader(doc, cdqualParams, nS):

   # Initialize name space for XML documents
   nS[0] = 'http://www.ctbto.org/xmlns/idc/product/report/' + nS[2]
   XMLreport = doc.createElementNS(nS[0], nS[1] + 'report')
   doc.appendChild(XMLreport)
   XMLreport.setAttributeNS(nS[0], 'xmlns:' + nS[1][0], nS[0])
   XMLreport.setAttributeNS(nS[0], nS[1] + 'version', nS[4])
   XMLreport.setAttributeNS(nS[0], nS[1] + 'created',
                            toDateTimeXML(time.time()))
   XMLreport.setAttributeNS(nS[0], nS[1] + 'author', 'cdqual ' + version)
   XMLreport.setAttributeNS(nS[0], nS[1] + 'type', nS[2])
   XMLreport.setAttributeNS(nS[0], nS[1] + 'title', nS[3])
   # Add report duration
   addReportTimeXML(cdqualParams, doc, XMLreport, nS)
   XMLrecords = doc.createElementNS(nS[0], nS[1] + 'records')
   XMLreport.appendChild(XMLrecords)
   return(XMLrecords)


# -----------------------------------------------------------------------------
# @short Get time in XML dateTime format
#
# This function returns a given time in XML dateTime format
#
# @param tis     time to convert
#
# @return tis  converted to XML dateTime string
# -----------------------------------------------------------------------------

def toDateTimeXML(tis):

   return time.strftime("%Y-%m-%dT%TZ", time.gmtime(tis))

# -----------------------------------------------------------------------------
# @short Get time in Julian format
#
# This function returns a given time in Julian format
#
# @param tis     time to convert
#
# @return tis  converted to Julian time (integer e.g. 2013033)
# -----------------------------------------------------------------------------

def toJulian(tis):

   return int(time.strftime("%Y%j", time.gmtime(tis)))

# -----------------------------------------------------------------------------
# @short Print Report Time
#
# This function prints the report time period to stdout
#
# @param cdqualParams dictionary containing name value pairs
#
# @return nothing
# -----------------------------------------------------------------------------

def printReportTime(cdqualParams):

   # Report start and end time
   start = time.strftime("%Y/%m/%d %T", time.gmtime(cdqualParams['startTime']))
   end = time.strftime("%Y/%m/%d %T", time.gmtime(cdqualParams['endTime']))
   print "Report period from %s.000 to %s.000" % (start, end)

# -----------------------------------------------------------------------------
# @short Print JSON header
#
# This function prints the JSON specific header
#
# @param cdqualParams dictionary containing name value pairs
# @param reportType   report type
#
# @return nothing
# -----------------------------------------------------------------------------

def printJsonHeader(cdqualParams, reportType):

   vStr = "1.0"
   str3 = "records"
   # Report start and end time
   start = time.strftime("%Y-%m-%d %T", time.gmtime(cdqualParams['startTime']))
   end = time.strftime("%Y-%m-%d %T", time.gmtime(cdqualParams['endTime']))
   now = time.strftime("%Y-%m-%d %T", time.gmtime())
   if reportType == 'CHAN_STATUS':
      str1 = "channelstatus"
      str2 = "Channel Status"
   elif reportType == 'CHAN_STATUS:2':
      str1 = "channelstatus"
      str2 = "Channel Status"
      vStr = "2.0"
   elif reportType == 'CHAN_STATUS:3':
      str1 = "channelstatus"
      str2 = "Channel Status"
      vStr = "3.0"
   elif reportType == 'CHAN_STATISTICS':
      str1 = "channelstatistics"
      str2 = "Channel Statistics"
      vStr = "1.1"
   elif reportType == 'STA_STATUS':
      str1 = "stationstatus"
      str2 = "Station Status"
   elif reportType == 'STA_STATUS:2':
      str1 = "stationstatus"
      str2 = "Station Status"
      vStr = "2.0"
   elif reportType == 'STA_STATUS:3':
      str1 = "stationstatus"
      str2 = "Station Status"
      vStr = "3.0"
   elif reportType == 'STA_STATISTICS':
      str1 = "stationstatistics"
      str2 = "Station Statistics"
   elif reportType == 'STA_STATISTICS:2':
      str1 = "stationstatistics"
      str2 = "Station Statistics"
      vStr = "2.0"
   elif reportType == 'AUTH_INTERVAL':
      str1 = "authenticationinterval"
      str2 = "Authentication Interval"
      str3 = "failedintervals"
   elif reportType == 'AUTH_STATUS':
      str1 = "authenticationstatus"
      str2 = "Authentication Status"
   else:
      str1 = "outage"
      str2 = "Outage"

   print '{\n "version": %s,\n "type": "%s",' % (vStr, str1, )
   print ' "created": "%s.000Z",\n "author": "cdqual %s",' % (now, version)
   print ' "title": "%s",' % (str2, )
   print ' "from": "%s.000Z",\n "to": "%s.000Z",' % (start, end)
   print ' "%s": [' % (str3, )



# -----------------------------------------------------------------------------
# @short Merge gap list with request list
#
# This function modifies the channels gap list according
# to the entries in the request table
#
# @param cdqualParams dictionary containing name value pairs
# @param cStat        channel statistics record
#
# @return nothing
# -----------------------------------------------------------------------------

def modifyGapList (cdqualParams, cStat):

   # Set up a 'virtual site' with two channels, assign the
   # requests (as gaps) to the first channel and the gaps
   # (also as gaps) to the second and call combineGaps2()
   vSite = []
   vSiteChannel1 = createStatisticRec('_VIR_', '_R_', 1.0)
   vSiteChannel1[GAP_LIST] = cStat[REQ_LIST]
   vSite.append(vSiteChannel1)
   vSiteChannel2 = createStatisticRec('_VIR_', '_G_', 1.0)
   vSiteChannel2[GAP_LIST] = cStat[GAP_LIST]
   vSite.append(vSiteChannel2)
   gapResult = combineGaps2(cdqualParams, vSite, ALL_GT)

   # The intersection between the "request gaps" list and the gaps list
   # are gaps in the requests. Remove all other gaps
   for cGap in gapResult[:]:
      if 1 == cGap[G_CNT]:
         gapResult.remove(cGap)

   debugGaps2(2, cdqualParams, ('Gap list: ', gapResult))

   # Next step is to remove the 'request gap type' from the gap type
   for cGap in gapResult:
      cGap[G_TYP] = cGap[G_TYP] & ~REQUEST_GAP_T

   # Last step is to replace the gap list with the combination of request
   # and gap list
   cStat[GAP_LIST] = gapResult

# -----------------------------------------------------------------------------
# @short Check if station sends non-continuous data
#
# This function checks if station sends non-continuous data
#
# @param cdqualParams  dictionary containing name value pairs
# @param arrayStation  array or station name
#
# @return 1 if non-continuous station, 0 otherwise
# -----------------------------------------------------------------------------

def checkAuxTable(cdqualParams, arrayStation):

   auxNet = cdqualParams.get('ncdNet', '')
   if not auxNet:
      return 0

   schema = cdqualParams.get('staticTablesPrefix', '')
   curCursor = cdqualParams['dbConOps'].cursor()

   # Set up database query. Search the given table for array or station name.
   # For use of prepared statement, build static part of SQL string first
   sqlStr = "SELECT sta FROM %saffiliation WHERE net = '%s' AND \
 (sta=? OR sta=?)" % (schema, auxNet)

   debug(3, cdqualParams, (sqlStr, arrayStation[0], arrayStation[1]))
   curCursor.execute(sqlStr, (arrayStation[0], arrayStation[1]))
   for row in curCursor:
       debug(2, cdqualParams, row)
       return 1

   return 0

# -----------------------------------------------------------------------------
# @short Calculate expected time
#
# This function returns the expected time interval for
# the given site/channel combination. If there is an request
# in the request table for the given site/channel combination
# and the requested time period, a request list with start and
# end time of the request(s) is added to the channel statistics
# record
#
# @param cdqualParams  dictionary containing name value pairs
# @param siteChan      site and channel name
# @param reqList       list with requested data time periods (out)
#
# @return maximal expected time
# -----------------------------------------------------------------------------

def getMaxExpTime (cdqualParams, siteChan, reqList):

   maxExpTime = 0

   # Query the request table for requests
   schema = cdqualParams.get('schemaPrefix', '')
   curCursor = cdqualParams['dbConOps'].cursor()

   # Oracle: (lddate-to_date('01/01/1970',
   #                         'MM/DD/YYYY'))*86400,'99999999999') as diff"

   # Set up database query. Search the request table for requests which
   # start or end time is within the requested period.

   ## sqlStr = "select start_time, end_time, modtime - d2epoch(lddate) \
   # For use of prepared statement, build static part of SQL string first
   sqlStr = "SELECT start_time, end_time, \
 modtime, to_char(lddate, 'YYYY/MM/DD HH24:MI:SS'), state \
 FROM %srequest \
 WHERE sta=? AND chan=? AND \
 start_time >= %f AND end_time <= %f \
 ORDER BY start_time" % (schema,
               cdqualParams['startTime'], cdqualParams['endTime'])

   ## currTime = time.time()
   ## skipAll = 0

   debug(3, cdqualParams, (sqlStr, siteChan[0], siteChan[1]))
   curCursor.execute(sqlStr, (siteChan[0], siteChan[1]))
   for row in curCursor:
       debug(2, cdqualParams, row)
       # Append request to request list and check if requested data
       # has been arrived delayed
       lddate = int(time.mktime(time.strptime(row[3], '%Y/%m/%d %H:%M:%S')))
       modtime = int(row[2])
       debug(3, cdqualParams, (siteChan[0], siteChan[1], row[0], row[1],
                               modtime - lddate, row[4]))

       ## Check when the request has been made
       ## if currTime - lddate < 300:
       ##    skipAll = 1
       ##    break

       if modtime - lddate > 300:
          reqList.append([row[0], row[1], 0, REQUEST_GAP_TD])
       else:
          reqList.append([row[0], row[1], 0, REQUEST_GAP_T])
       maxExpTime += row[1] - row[0]

   if 0 == len(reqList):
      debug(2, cdqualParams, 'No request found in request table for %s/%s' % \
            (siteChan[0], siteChan[1]))
      maxExpTime = 0
      reqList.append((0, 0, 0, NO_GAP_T))

   ## Ignore too young requests will require grouping requests
   ## by date/time and remove requests with the same time from
   ## other channels of the same stations. This can't be done here!
   # Ignore too "young" requests for whole auxiliary
   # stations, independent of current state of request.
   # Otherwise some channels may have arrived and some not,
   # which looks for cdqual like no data have been requested for
   # channels which have not been arrived yet. If the threshold
   # for too young requests is reached, open requests are treated
   # as missing data
   ## if skipAll:
   ##    debug(2, cdqualParams, 'Too young request, skip %s/%s' % \
   ##          (siteChan[0], siteChan[1]))
   ##    del reqList[:]  # clear all requests
   ##    maxExpTime = 0
   ##    reqList.append((0, 0, 0, NO_GAP_T))

   #TODO: return empty request list to avoid strange select, and check if
   #      code expect at least one record in reqList
   debug(2, cdqualParams, ('maxExpTime ', maxExpTime))
   return(maxExpTime)

# -----------------------------------------------------------------------------
# @short Apply mission rule sets for seismic stations
#
# This function computes the mission availability based on rule sets
# (if any) for seismic stations
#
# @param cdqualParams         dictionary containing name value pairs
# @param filteredStatistics   list of channel statistics
# @param rulesets             list of rule sets
#
# @return flag if rule set found and virtual channel with gap list (if any)
# -----------------------------------------------------------------------------

def applySeismicRuleSet(cdqualParams, filteredStatistics, rulesets):

   # Create virtual channel to be able to reprocess the gaps later
   combinedChannels = createStatisticRec('_VIR_', '_S_', 1.0)
   actRules = []

   # -----------------------------------------------------------------
   # Set default values for rule sets
   for ruleset in rulesets:
      debug(5, cdqualParams, ruleset)
      if ruleset[0] == 'verticals seismic':
         actSet = ruleset
         if actSet[2] == '':
            actSet[2] = '80%'
      elif ruleset[0] == 'reserved':
         actSet = ruleset
         if actSet[2] == '':
            actSet[2] = '50%'

      actRules.append(actSet)

   # check if any rule set
   if len(actRules) == 0:
      return(None)

   # Process rule set now
   newFilteredStatistics = []
   for actRule in actRules:
      if actRule[2] != '0%':
         channelCnt = 0

         # sort out all channels not used in rule set
         for cStat in filteredStatistics:
            siteChan = cStat[SITE] + '/' + cStat[CHANNEL]
            # Empty channel list leaves channel list unchanged to
            # provide an easy way to change only the  minimum number
            # of channels
            if actRule[1] == '' or re.match(actRule[1], siteChan, 0):
               # NOTE: Not sure if a channel may appear in more than one rule.
               # If not, the rules should checked when created with a DB
               # trigger before
               newFilteredStatistics.append(cStat)
               channelCnt += 1

         # Calculate minimum number of required channels
         if actRule[2][-1] == '%':
            perCentage = actRule[2][:-1]
            perCentage = int(perCentage)
            perCentage = 0.01 * perCentage
            minChannels = ceil(channelCnt * perCentage)
         elif actRule[2][-1] == 'B':
            backupChannels = actRule[2][:-1]
            backupChannels = int(backupChannels)
            minChannels = channelCnt - backupChannels
         else:
            minChannels = int(actRule[2])

         # Second pass: consolidate gaps and add channel count to gaps for
         #              rule set group
         gapResult = combineGaps3(cdqualParams, newFilteredStatistics,
                                  minChannels, channelCnt)

         combinedChannels[GAP_LIST] = gapResult
         newFilteredStatistics = [ combinedChannels, ]

   return(combinedChannels)

# -----------------------------------------------------------------------------
# @short Load mission rule sets for all station types
#
# This function loads the mission rules sets
# (if any) for all types of stations. So far only hydroacustic rule sets
# are supported
#
# @param stationName          name of the station
# @param cdqualParams         dictionary containing name value pairs
# @param filteredStatistics   list of channel statistics
# @param ruleset              type of ruleset ('infrasound', )
#
# @return None or virtual channel with gap list (if any)
# -----------------------------------------------------------------------------

def loadMissionRuleSet(stationName, cdqualParams, filteredStatistics, ruleset):

   # Create virtual channel to be able to reprocess the gaps later
   combinedChannels = createStatisticRec('_VIR_', '_U_', 1.0)

   debug(2, cdqualParams, 'Load mission ruleset')
   # Rule set is only load in DB mode
   if cdqualParams['noDB'] == 1:
      return(None)

   staticTab = cdqualParams.get('staticTablesPrefix', '')
   ops = cdqualParams['dbConOps']
   opsCursor = ops.cursor()

   # Set up database query. Search for station in rule set
   # and on and offdate within the requested period.
   sqlStr = "SELECT DISTINCT name, regexp, minchan FROM %sruleset \
WHERE sta=? AND (offdate >= %d OR offdate = -1) \
And ondate <= %d" % (staticTab, toJulian(cdqualParams['startTime']),
                   toJulian(cdqualParams['endTime']))

   debug(5, cdqualParams, sqlStr)
   try:
      opsCursor.execute(sqlStr, (stationName, ))
   except:
      print >> sys.stderr, \
               "Warning: can't read ruleset table, did you create it?"
      return(None)

   rulesets = opsCursor.fetchall()

   # check if any rule set
   if len(rulesets) == 0:
      debug(1, cdqualParams, ('No rule set found for', stationName))
      return(None)

   if (ruleset == 'infrasound'):
      return(applyInfraRuleSet(cdqualParams, filteredStatistics, rulesets))
   elif (ruleset == 'seismic'):
      return(applySeismicRuleSet(cdqualParams, filteredStatistics, rulesets))
   else:
      print >> sys.stderr, "Warning: can't determine ruleset type"
      return(None)


# -----------------------------------------------------------------------------
# @short Log gap times in percent
#
# This function logs gap times in percent
#
# @param cdqualParams         dictionary containing name value pairs
# @param cStat                (virtual) channel
#
# @return nothing
# -----------------------------------------------------------------------------

def logGapTimesChannel(cdqualParams, cStat):

   gT = getGapTimes(cdqualParams, cStat[GAP_LIST])

   if 0 != cStat[MAX_EXP_TIME]:
      mEt = cStat[MAX_EXP_TIME]
   else:
      mEt = ONE_DAY

   for ii in range(8):
      gT[ii] = getPercent(mEt, mEt - gT[ii])

   print 'gapTime [%.6s/%.3s] is %7.3f %7.3f %7.3f %7.3f %7.3f %7.3f' % \
         (cStat[SITE], cStat[CHANNEL], gT[0], gT[1], gT[2], gT[3], gT[4], gT[5])


# -----------------------------------------------------------------------------
# @short Apply mission rule sets for hydroacustic stations
#
# This function computes the mission availability based on rule sets
# (if any) for hydroacustic stations
#
# @param cdqualParams         dictionary containing name value pairs
# @param filteredStatistics   list of channel statistics
# @param rulesets             list of rule sets
#
# @return flag if rule set found and virtual channel with gap list (if any)
# -----------------------------------------------------------------------------

def applyInfraRuleSet(cdqualParams, filteredStatistics, rulesets):

   actRules = []

   # -----------------------------------------------------------------
   # Set default values for rule sets
   essentialSet = innerSet = outerSet = otherSet = []
   for ruleset in rulesets:
      debug(2, cdqualParams, ruleset)
      if ruleset[0] == 'essential pressure':
         essentialSet = ruleset
         if essentialSet[2] == '':
            essentialSet[2] = '100%'
      elif  ruleset[0] == 'inner pressure':
         innerSet = ruleset
         if innerSet[2] == '':
            innerSet[2] = '3'
      elif  ruleset[0] == 'outer pressure':
         outerSet = ruleset
         if outerSet[2] == '':
            outerSet[2] = '3'
      elif  ruleset[0] == 'other pressure':
         otherSet = ruleset
         if otherSet[2] == '':
            otherSet[2] = '50%'

   if len(essentialSet) and len(innerSet) and len(otherSet):
      # If essential pressure channels are defined the following
      # rule is applied: Minimum 3 inner pressure channels AND all
      # essential pressure channels AND 50% of other pressure channels.
      actRules.append(essentialSet)
      actRules.append(innerSet)
      actRules.append(otherSet)
   elif len(innerSet) and len(outerSet):
      # If no essential pressure channels are defined but inner and
      # outer pressure channels are defined the following rule is applied:
      # Minimum 3 inner pressure channels AND minimum 3 outer pressure
      # channels.
      actRules.append(innerSet)
      actRules.append(outerSet)
   elif len(innerSet) and len(otherSet):
      # If no essential pressure channels are defined but inner and
      # other pressure channels are defined the following rule is applied:
      # Minimum 3 inner pressure channels AND minimum 50% of other pressure
      # channels.
      actRules.append(innerSet)
      actRules.append(otherSet)
   elif len(essentialSet) and len(innerSet):
      print >> sys.stderr, \
          "Warning! No other infrasound rule found for essential! " + \
          "Skip rule set"
   elif len(essentialSet) and len(otherSet):
      print >> sys.stderr, \
          "Warning! No inner infrasound rule found for essential! " + \
          "Skip rule set"
   elif len(essentialSet):
      print >> sys.stderr, \
        "Warning! No inner and other infrasound rule found for essential! " + \
        "Skip rule set"
   elif len(innerSet):
      print >> sys.stderr, \
          "Warning! No outer infrasound rule found for inner! " + \
          "Skip rule set"
   elif len(outerSet):
      print >> sys.stderr, \
          "Warning! No inner infrasound rule found for outer! " + \
          "Skip rule set"

   # If a required rule set is missing, no rule sets are applied. See
   # comment above
   if len(actRules) == 0:
      return(None)

   debug(2, cdqualParams, ('Infrasound rulesets:', actRules))

   # Process rule set now
   newFilteredStatistics = []
   allRuleSetsStatistics = []
   ruleCnt = 0

   for actRule in actRules:
      if actRule[2] != '0%':
         channelCnt = 0

         # sort out all channels not used in rule set
         for cStat in filteredStatistics:
            siteChan = cStat[SITE] + '/' + cStat[CHANNEL]
            if re.match(actRule[1], siteChan, 0):
               # NOTE: Not sure if a channel may appear in more than one rule.
               # If not, the rules should checked when created with a DB
               # trigger before
               newFilteredStatistics.append(cStat)
               channelCnt += 1
               debug(2, cdqualParams, ('Infrasound rule match:', siteChan))

         # Calculate minimum number of required channels
         if actRule[2][-1] == '%':
            perCentage = actRule[2][:-1]
            perCentage = int(perCentage)
            perCentage = 0.01 * perCentage
            minChannels = ceil(channelCnt * perCentage)
         else:
            minChannels = int(actRule[2])

         debug(2, cdqualParams, ('Infrasound rule minChannels:',
                                 minChannels, channelCnt))
         # Second pass: consolidate gaps and add channel count to gaps for
         #              rule set group
         gapResult = combineGaps3(cdqualParams, newFilteredStatistics,
                                  minChannels, channelCnt)

         # Create virtual channel to be able to reprocess the gaps later
         cN = "_I%d" % (ruleCnt, )
         combinedChannels = createStatisticRec('_VIR_', cN, 1.0)
         combinedChannels[GAP_LIST] = gapResult

         ## logGapTimesChannel(cdqualParams, combinedChannels)

         newFilteredStatistics = []
         allRuleSetsStatistics.append(combinedChannels)
         ruleCnt += 1

   # Last pass: combine all rule sets. ANY of the rule sets MUST match!!
   gapResult = combineGaps3(cdqualParams, allRuleSetsStatistics,
                            ruleCnt, ruleCnt)
   combinedChannels = createStatisticRec('_VIR_', '_I_', 1.0)
   combinedChannels[GAP_LIST] = gapResult

   ## logGapTimesChannel(cdqualParams, combinedChannels)

   return(combinedChannels)


# -----------------------------------------------------------------------------
# @short Check if station is an array
#
# This function returns the station name if the station
# is an array
#
# @param stationName  name of the station
# @param cdqualParams dictionary containing name value pairs
#
# @return station name if station is an array
# -----------------------------------------------------------------------------

def isArray (stationName, cdqualParams):

   staticTab = cdqualParams.get('staticTablesPrefix', '')
   ops = cdqualParams['dbConOps']
   opsCursor = ops.cursor()

   # Set up default
   arrayName = ''

   # Set up database query. Search for station in site with station type
   # array and on and off date within the requested period.
   # For use of prepared statement, build static part of SQL string first
   sqlStr = "SELECT DISTINCT sta FROM %ssite WHERE sta=? AND \
             statype = 'ar' AND (offdate >= %d OR offdate = -1) \
             AND ondate <= %d" % (staticTab,
             toJulian(cdqualParams['startTime']),
             toJulian(cdqualParams['endTime']))

   debug(3, cdqualParams, (sqlStr,stationName))
   opsCursor.execute(sqlStr, (stationName,))
   for row in opsCursor:
       debug(5, cdqualParams, row)
       arrayName = row[0]

   opsCursor.close()
   return(arrayName)

# -----------------------------------------------------------------------------
# @short Get Bad Sample Number
#
# This function looks into the waveform file to get
# the number of constant data or 'no input from sensor' samples
# It also creates a "constant data gap list"
#
# @param db           database (OPS or ARCHIVE)
# @param siteChan     list containing site and channel
# @param cdqualParams dictionary containing name value pairs
# @param startTime    data start time
# @param endTime      data end time
#
# @return number of constant data and 'no input from sensor' samples and
#         constant data gap list
# -----------------------------------------------------------------------------

def getBadSampleNumber (db, siteChan, cdqualParams, startTime, endTime, nsamp):

   # Attention! This algorithm requires that quality and wfdisc records
   # are archived without breaking the relations temporary (cp wfdiscs
   # to archive, move quality to archive, remove wfdiscs
   schema = cdqualParams.get('schemaPrefix', '')

   curCursor = cdqualParams[db].cursor()

   # Set up default record
   constAndNoiseRec = [ 0, 0 ]
   constGaps = []
   isFirstRecord = 1
   startMinMax = 1
   minc = maxc = -1
   constSamples = 0
   samples2process = nsamp
   sampRate = 1 # needed for recheck of sensor noise

   # Set up database query. Search all wfdiscs which start or end time
   # is within the requested period.
   # NOTE: sample rate and data type is not stored in cdquality table!
   #       This is the reason why the wfdisc table has to be queried.
   #       Data type could be coded in the file extension
   # For use of prepared statement, build static part of SQL string first
   sqlStr = "SELECT nsamp, dir, dfile, foff, datatype, time, endtime, samprate \
 FROM %swfdisc WHERE sta=? AND chan=? AND \
 time BETWEEN ? AND ? AND endtime > ? ORDER BY \
 time" % (schema)

   # Bind parameters as strings to avoid performance drop when DB
   # data type does not match python type in where clause
   startTimeSm = str(startTime - 14500)
   endTimeS = str(endTime)
   startTimeS = str(startTime)

   # NOTE: This select does not consider on and off dates for channels!
   debug(4, cdqualParams, (sqlStr, siteChan[0], siteChan[1],
            startTimeSm, endTimeS, startTimeS))
   curCursor.execute(sqlStr, (siteChan[0], siteChan[1],
                     startTimeSm, endTimeS, startTimeS))
   for row in curCursor:
       debug(4, cdqualParams, row)
       # Now check if the wfdiscs are complete within the requested
       # time period and calculate requested values. A quality record
       # can spawn several wfdisc records in the worst case!
       samplesInWfdisc = row[0]
       filename = row[1] + "/" + row[2]
       foffset = row[3]
       sampleLen = int(row[4][1])
       wfdiscStartTime = row[5]
       wfdiscEndTime = row[6]
       sampleRate = row[7]
       tdiff = 0
       # ........W1++++++w1W2++++++++w2W3++++w3..
       # ...........Q++++++++++++++++++++++q.....
       # In the first iteration reading has to begin at position Q
       # Reading will stop automatically at position w1
       # The rest of the quality record is processed in the next wfdisc(s)
       if isFirstRecord:
          tdiff = startTime - wfdiscStartTime
          skipSamples = int(round((tdiff * sampleRate), 0))
          samplesInWfdisc -= skipSamples
          foffset += (skipSamples * sampleLen)
          isFirstRecord = 0

       # If there are fewer samples left in the quality record, only
       # the remaining samples are processed (e.g. W3 to q)
       if nsamp < samplesInWfdisc:
           samplesInWfdisc = nsamp

       nsamp -= samplesInWfdisc

       debug(5, cdqualParams, filename)
       try:
           fp = open(filename, "rb")

       except:
           sys.exit("Error: can't open '%s' for reading" % (filename,))
           # Next two lines are only for testing with no mwf files!!
           # print >> sys.stderr, \
           #         "Warning: can't open '%s' for reading" % (filename,)
           # return (nsamp, nsamp, [(startTime, endTime, 1, 'M')])
       try:
           # Set the file pointer to the computed offset
           fp.seek(long(foffset))
           lastSample = 0
           suspSamples = 0
           firstSample = 1

           # Set up default values for start end end time of segment
           tEnd = -1
           tStart = wfdiscStartTime + tdiff
           debug(4, cdqualParams, ('samplesInWfdisc ', samplesInWfdisc))

           # Inspect segment to find constant values and 'no input from sensor'
           for ii in range(0, samplesInWfdisc):
               # Read one sample
               sa1 = fp.read(sampleLen)
               #### ADD retry here
               # Convert s3/s4 to native integer
               if sampleLen == 3:
                  sample = (ord(sa1[0])<<16)|(ord(sa1[1])<<8)|ord(sa1[2])
                  if sample >= 0x800000: sample -= 0x1000000
               else:
                  sample = (ord(sa1[0])<<24)|(ord(sa1[1])<<16)|\
                           (ord(sa1[2])<<8)|ord(sa1[3])
                  if sample >= 0x80000000: sample -= 0x100000000

               # Set minimal and maximal sample value seen so far
               if startMinMax:
                  maxc = minc = sample
                  startMinMax = 0
               elif sample > maxc:
                  maxc = sample
               elif sample < minc:
                   minc = sample

               if firstSample:
                  lastSample = sample
                  firstSample = 0

               # Check for constant data
               # Count only suspicious constant data samples first and
               # create a 'constant data' gap if constant data threshold
               # is reached
               #
               if lastSample == sample:
                  # Could be constant data
                  suspSamples += 1
               else:
                  # calling the debug function only takes surprisingly much
                  # time
                  # debug(2, cdqualParams, ('s/c: s l i a u ', siteChan[0],
                  #       siteChan[1], sample, lastSample, min, max,
                  #       suspSamples))

                  lastSample = sample

                  if suspSamples >= CONST_DATA_THRESHOLD:
                     # No constant data, but threshold for constant data
                     # is reached. Count number of constant samples
                     ## constAndNoiseRec[0] += suspSamples
                     constSamples += suspSamples

                     # Add bad data gap
                     constGapStart = tStart + ((ii + 1 - suspSamples) *\
                                     1./sampleRate)
                     constGapEnd = tStart + (ii * 1./sampleRate)
                     constGaps.append((constGapStart, constGapEnd, 'C'))
                     debug(4, cdqualParams, ('1CGS', constGapStart, 'CGE',
                           constGapEnd, 'SCS', suspSamples))
                     # Start next detection cycle. Start with one
                     # suspicious sample if not at start of segment
                     suspSamples = 1

                  else:
                     # No constant data and threshold for constant data
                     # is not reached. Start next detection cycle. Start
                     # with one suspicious sample if not at start of segment
                     suspSamples = 1

           # End processing of current wfdisc segment
           # -----------------------------------------------------------
           # Check for remaining constant data detected
           # This check here causes that bad data segments do not spawn
           # more than one wfdisc record!
           if suspSamples >= CONST_DATA_THRESHOLD:
               ## constAndNoiseRec[0] += suspSamples
               constSamples += suspSamples

               # Add bad data gap
               constGapStart = tStart + ((ii + 1 - suspSamples) *\
                               1./sampleRate)
               constGapEnd = tStart + (ii * 1./sampleRate)
               constGaps.append((constGapStart, constGapEnd, 'C'))
               debug(4, cdqualParams, ('2CGS', constGapStart, 'CGE',
                     constGapEnd, 'SCS', suspSamples))
               suspSamples = 0

           fp.close()
       except EOFError, why:
           print why
           sys.exit("Error: EOF when read from '%s', offset %d" % (filename,
                    row[3]))
       except IOError, why:
           print why
           sys.exit("Error: can't read from '%s', offset %d" % (filename,
                    row[3]))

   # No more wfdiscs to process
   #-----------------------------------------------------------------
   if startMinMax:
      print >> sys.stderr, \
        "Warning! Quality record without related wfdisc found %s/%s %f-%f" % \
        (siteChan[0], siteChan[1], startTime, endTime)

   cCnt = len(constGaps)
   gPS = cCnt/(endTime - startTime)
   dbgTxt = 'Gap statistics'
   ## RMS = round(sqrt(m2/samples),4)
   # 'No input from sensor' is detected in cd2wng, but the
   # DIGITIZER_NOISE_THRESHOLD value does not catch all known
   # cases. So some experimental checks are run here again.
   # Keep in mind that gaps do not spawn wfdisc records
   if not startMinMax and cdqualParams['digitizerNoiseThreshold'] > 0 and \
      abs(int(maxc-minc)) < cdqualParams['digitizerNoiseThreshold']:
      # The whole segment is is treated as 'no input from sensor'
      # if the maximum difference of all channel counts are lower
      # or equal than 'digitizerNoiseThreshold'
      # Looks like whole segment does not contain real data
      # Overwrite constant samples and gap list with one gap
      # which covers the whole segment
      # TODO: constant data gaps endtime is off by one sample, so
      #       here the same endtime is used to minimize deviations
      #       in the tests cases.
      ## for gap in constGaps:
      ##   print >> sys.stderr, "%7.7f %7.7f %s" % (gap[G_BEG], gap[G_END],
      ##                                            gap[G_CNT])
      debug(4, cdqualParams, ('3CGS', startTime, 'CGE',
               endTime - 1./sampleRate, 'SCS', constSamples))
      constSamples = samples2process
      constGaps = []
      constGaps.append((startTime, endTime - 1./sampleRate, 'C'))
      dbgTxt = 'Override gap list due to too low differences'

   debug(4, cdqualParams, ("%s %s %s %d %d %d %d %.2f" % ( dbgTxt,
                           siteChan[0], siteChan[1], maxc, minc,
                           cCnt, endTime - startTime,
                           cCnt/(endTime - startTime))))

   # Add the constant data gap list
   constAndNoiseRec[0] += constSamples
   constAndNoiseRec.append(constGaps)
   return constAndNoiseRec



# -----------------------------------------------------------------------------
# @short Compute percentage value
#
# This function computes percentage value of nSamples
# when samplesExpected is 100 percent
#
# @param samplesExpected number of samples expected
# @param nSamples        number of samples available
#
# @return percentage of samples available
# -----------------------------------------------------------------------------

def getPercent(samplesExpected, nSamples):

   if not samplesExpected:
      return 0.000

   # Compute percentage. Make sure that at least one
   # of the operands is a float or the result might be
   # truncated to an integer
   per = round(float(nSamples) * 100 / samplesExpected, 3)

   # Auxiliary stations might send more than the requested data or
   # inaccurate knowledge of the sample rate of primary stations might
   # cause a result of more than hundred percent. In these cases
   # the result is trimmed to 100 percent (request by GP and JC)
   if per > 100.0:
      per = 100.000

   if per <= 0.0: # To avoid -0.000
      per = 0.000

   return per

# -----------------------------------------------------------------------------
# @short Filter sitechan channels
#
# This function removes all channels in siteChannels
# which are not present in uncheckedChannels
#
# @param siteChannels         list of site/channel pairs (in/out)
# @param uncheckedChannels    list of requested channel names
#
# @return nothing
# -----------------------------------------------------------------------------

def filterSiteChan(siteChannels, uncheckedChannels):

   # For each site/channel combination
   for siteChan in siteChannels[:]:
      if siteChan[1] not in uncheckedChannels:
         siteChannels.remove(siteChan)

# -----------------------------------------------------------------------------
# @short Format gap line
#
# This function formats a gap for printing
#
# @param gap         list containing gap start and end time
#
# @return list containing gap start, gap end and gap duration and comment
# -----------------------------------------------------------------------------

def getGapLine(gap):

    # Python 2.4 does not support %f
    diff = gap[0] - int(gap[0])
    diff1 = round(diff,3) * 1000
    if 1000 == diff1:
       diff1 = 999 # only 3 digits after the comma
    start = time.strftime("%Y/%m/%d %H:%M:%S.", time.gmtime(gap[0]))
    diff = gap[1] - int(gap[1])
    diff2 = round(diff,3) * 1000
    if 1000 == diff2:
       diff2 = 999 # only 3 digits after the comma
    end = time.strftime("%Y/%m/%d %H:%M:%S.", time.gmtime(gap[1]))
    duration = gap[1] - gap[0]
    gapType = ''
    if (gap[G_TYP] & BAD_DATA_GAP_T) > 0:
       gapType += 'C'
    if (gap[G_TYP] & LATE_GAP_T) > 0:
       gapType += 'L'
    if (gap[G_TYP] & AUTH_FAIL_GAP_T) > 0:
       # Keep in mind that there are more than one reason for authentication
       # failure which could cause segmentation of quality records
       gapType += 'U'
    return (start, diff1, end, diff2, duration, gapType)

# -----------------------------------------------------------------------------
# @short Format authentication gap line
#
# This function formats a gap for printing
#
# @param gap         list containing gap start and end time
#
# @return list containing gap start, gap end and gap duration and comment
# -----------------------------------------------------------------------------

def getAuthGapLine(gap):

    # Python 2.4 does not support %f
    diff = gap[0] - int(gap[0])
    diff1 = round(diff,3) * 1000
    if 1000 == diff1:
       diff1 = 999 # only 3 digits after the comma
    start = time.strftime("%Y/%m/%d %H:%M:%S.", time.gmtime(gap[0]))
    diff = gap[1] - int(gap[1])
    diff2 = round(diff,3) * 1000
    if 1000 == diff2:
       diff2 = 999 # only 3 digits after the comma
    end = time.strftime("%Y/%m/%d %H:%M:%S.", time.gmtime(gap[1]))
    duration = gap[1] - gap[0]
    gapType = ''

    if (gap[G_TYP] & QUA_FR_AUTH_FAILED) > 0:
       gapType += 'Frame authentication failed, '
    if (gap[G_TYP] & QUA_FR_NOT_AUTHENTICATED) > 0:
       gapType += 'Frame not authenticated, '
    if (gap[G_TYP] & QUA_FR_AUTH_NO_CERT) > 0:
       gapType += 'No certificate for frame found, '
    if (gap[G_TYP] & QUA_FR_AUTH_NOT_SIGNED) > 0:
       gapType += 'Frame not signed, '
    if (gap[G_TYP] & QUA_AUTH_FAILED) > 0:
       gapType += 'Channel sub-frame authentication failed, '
    if (gap[G_TYP] & QUA_NOT_AUTHENTICATED) > 0:
       gapType += 'Channel sub-frame not authenticated, '
    if (gap[G_TYP] & QUA_AUTH_NO_CERT) > 0:
       gapType += 'No certificate for channel sub-frame found, '
    if (gap[G_TYP] & QUA_AUTH_NOT_SIGNED) > 0:
       gapType += 'Channel sub-frame not signed, '

    gapType = gapType.rstrip(' ,')
    # No duration in the interval
    # return (start, diff1, end, diff2, duration, gapType)
    return (start, diff1, end, diff2, gapType)


# -----------------------------------------------------------------------------
# @short Load frame values
#
# This function load information related to received frames from clf files.
# This function requires that CD Receiver index and clf files have been
# created by CD Receiver (and have not been archived without changing the
# information in the index files!). 'frames' contains a list of received
# data frames, virtual channel name frames and virtual connect frames after
# this function call.
#
# @param station      station
# @param cdqualParams dictionary containing name value pairs
# @param frames       see above (out)
#
# @return 0 if successful and 1 if no channel name frames are found
# -----------------------------------------------------------------------------

F_TYPE = 0
F_RECVD_TIME = 1
F_NOM_TIME = 2
F_FPC = 3
F_MASK = 4
F_ENDTIME = 5
F_INDEX = 6
# Next not read from clf file
F_AUTH_STATE = 6

def loadFrameValues(station, cdqualParams, frames):

   startTime = cdqualParams['startTime']
   endTime = cdqualParams['endTime']

   path = cdqualParams['cdrecvIndexFilePath']
   debug(2, cdqualParams, path)

   hasChannelDescFrame = 0
   stationFound = 0

   # While index file has been created in the requested time period
   # plus one day. One day is added to catch late arrived files
   while startTime < (endTime + ONE_DAY):
      debug(2, cdqualParams, ('station=', station, 'starttime=', startTime))
      startTimeD = datetime.utcfromtimestamp(startTime)
      indexFiles = startTimeD.strftime('_*.%Y%m%d.idx')
      debug(2, cdqualParams, indexFiles)
      try:
         errFileName = path
         # Open index files directory
         for infile in glob.glob(os.path.join(path, indexFiles)):
            debug(2, cdqualParams, ('current index file is', infile))
            # Read index file
            errFileName = infile
            ifile = open(infile)
            lines = ifile.readlines()
            curLine = 0
            for line in lines:
               curLine += 1
               debug(5, cdqualParams, ('Index file line is', line, curLine))
               # Search for 'station' in index file
               if re.search(station, line, 0):
                  debug(5, cdqualParams, ('station found in index file',
                        line, curLine))
                  stationFound = 1
                  clffile = re.sub(r'=cd1.*', '.clf', line).strip()
                  debug(2, cdqualParams, ('Clf file is', clffile, curLine))
                  cln = 0
                  # Read clf file
                  errFileName = clffile
                  cfile = open(clffile)
                  clines = cfile.readlines()
                  for cline in clines:
                     # Load data frame
                     if re.search(r'>[dD]', cline, 0):
                        cF = cline.split()
                        mask = cF[9]
                        ## mask = re.sub(r'[^01]', r'0', cf[9][1:-1])
                        ## ba = bitarray(mask)

                        # DF type; received; nominal time; fpc; mask; endtime;
                        #    outstate

                        try:
                           frameEndTime = float(cF[6]) + float(cF[10]) / 1000
                           if frameEndTime < startTime \
                              or float(cF[6]) > endTime:
                              # Skip frames with nominal time not within
                              # report time. Keep in mind that, to catch also
                              # late arrived frames, clf files which are one
                              # day more recent than the report period, are
                              # scanned
                              continue

                           frames.append([3, float(cF[5]), float(cF[6]),
                                  int(cF[7], 16), mask, frameEndTime, 0])

                        except IndexError:
                           # Very old file without duration??
                           frames.append([3, float(cF[5]), float(cF[6]),
                                         int(cF[7], 16), mask,
                                         float(cF[6]) + 10.0, 0])
                        cln += 1;
                     # Reset channel names
                     elif re.search(r'\*[k]', cline, 0):
                        # Reset Channel name type; ; ; ;
                        frames.append([1, 0.0, 0.0, 0, '', 0, 0])
                     # Virtual channel mask frame
                     elif re.search(r'\*[h]', cline, 0):
                        hasChannelDescFrame = 1
                        cF = cline.split()
                        # Add channel to channel mask type; ; ; ;
                        frames.append([2, 0.0, 0.0, int(cF[7], 16), cF[9],
                                       0, int(cF[10])])
                  debug(5, cdqualParams, ("Read %d lines" % (cln,)))
                  cfile.close()
                  # Add virtual connection request frame to make sure
                  # that the previous channel mask frames are reset
                  # when parsing the frames. Otherwise incorrect
                  # 'Channel index too high' warnings are raised!
                  # See ISI-272
                  frames.append([1, 0.0, 0.0, 0, '', 0, 0])
            ifile.close()
      except IOError:
         sys.exit("Error: can't open '%s' for reading" % (errFileName,))

      startTime += ONE_DAY

   # Check if a clf file has been found
   if not stationFound:
      print >> sys.stderr, \
          "Warning! Can't find clf file for station " + station + '.'
      return 1

   # Don't remove duplicates here. This will not work, because the
   # channel index might change if the receiver has been re-started
   # during the requested period and there is no safe way to sort
   # by nominal frame time if virtual channel info frames MUST be
   # sorted in receive time sequence
   if not hasChannelDescFrame:
      print >> sys.stderr, \
          "Warning! Can't find channel name frames " + \
          'in clf file. Skiping ' + station + '. Too old CD Receiver version?'
      return 1

   return 0

# -----------------------------------------------------------------------------
# @short Find site in frames
#
# This function provides site values based on frames received (clf lines). It
# scans the clf lines for channels which belongs to a site.
#
# @param cdqualParams dictionary containing name value pairs
# @param frames       list of frames received
# @param siteValues   site values (net, site, frames received, ok, failed, ...)
# @param reqList      list of requests
#
# @return nothing
# -----------------------------------------------------------------------------

def findSite(cdqualParams, frames, siteValues, reqList):

   # Note: A CSF may, but must not be included in all frames received.
   #       This means, the frame list has be be scanned for any channel
   #       which is part of the requested site.
   # Note: If it is allowed to have different request periods for channels
   #       of the same site, channel names and request periods have to be
   #       known in this function too, and frames have to be excluded if they
   #       contain only channels which have not been requested at the frames
   #       nominal time

   # Copy frames which cover the request time to temporary list
   channelIndex = []
   tmpFrames = []
   REQ_START = 0
   REQ_END = 1
   channelSearchStr = '\|' + siteValues[1] + '/'
   for frame in frames:
      if frame[F_TYPE] == 3: # Data frame
         for request in reqList:
            start = request[REQ_START]
            end = request[REQ_END]
            if frame[F_NOM_TIME] < end and frame[F_ENDTIME] > start:
               try:
                  for index in channelIndex:
                     if (frame[F_MASK][index] == '1'):
                        tmpFrames.append(frame)
                        break;
               except IndexError:
                  print >> sys.stderr, \
                  "Warning! Channel index too high " + frame[F_MASK] + \
                  ", index = " + str(channelIndex) + " [" + siteValues[0] + "]"

      elif frame [F_TYPE] == 2: # Virtual Channel Name frame
         if re.search(channelSearchStr, frame[F_MASK], 0):
            # Site might have more than one channel
            channelIndex.append(frame[F_INDEX])
      else: # Virtual Connect frame
         channelIndex = []

   # Sort frames by nominal time to avoid counting duplicates
   # (frames with same nominal time). Works only if duplicate
   # frames have same duration
   frameList = []
   tmpFrames.sort(key=lambda tmpFrame: tmpFrame[F_NOM_TIME])
   isFirst = 1
   # Remove duplicate frames. First frame received wins
   for frame in tmpFrames:
      if isFirst:
         isFirst = 0
         lowestRecTime = frame[F_RECVD_TIME]
         lastNomTime = frame[F_NOM_TIME]
         lastFrame = frame

      if lastNomTime < frame[F_NOM_TIME]:
         frameList.append(lastFrame)
         lowestRecTime = frame[F_RECVD_TIME]
         lastNomTime = frame[F_NOM_TIME]
         lastFrame = frame
      else:
         if lowestRecTime > frame[F_RECVD_TIME]: # older frame found
            lowestRecTime = frame[F_RECVD_TIME]
            lastFrame = frame

   # 'frameList' now contains only frames which contribute to
   # channel data

   # There is no information in the clf line whether or not authentication
   # has been switched on. It may also happen, that the receiver has different
   # configuration. That means, the authentication state of the frame
   # has to be determined using the information in the quality record,
   # but in this case all channels related to the site has to be considered,
   # because one channel might not have been received all the time.
   # To avoid problems, frame authentication state found in the cdquality
   # table has been added to the frame line when the channel authentication
   # has been checked.

   for frame in frameList:
      siteValues[2] += 1
      siteValues[3] += 1 # Checked
      if frame[F_AUTH_STATE] == QUA_FR_NOT_AUTHENTICATED:
         siteValues[3] -= 1 # frame not checked
      elif frame[F_AUTH_STATE] == QUA_FR_AUTH_FAILURE:
         siteValues[4] += 1 # Failed

# -----------------------------------------------------------------------------
# @short Populate temporary in memory database
#
# This function populates the temporary in memory database
# with quality and wfdisc records for the queried time period
#
# @param station       station
# @param cdqualParams dictionary containing name value pairs
#
# @return nothing
# -----------------------------------------------------------------------------

def populateTempDB(station, cdqualParams):

   startTime = cdqualParams['startTime']
   endTime = cdqualParams['endTime']

   arch = cdqualParams['dbConArchive']
   cur = arch.cursor()

   # Truncate wfdisc and cdquality table first before loading
   # another stations records to save memory
   sqlStr = 'DELETE FROM wfdisc\n'
   debug(5, cdqualParams, sqlStr)
   cur.execute(sqlStr)
   sqlStr = 'DELETE FROM cdquality\n'
   debug(5, cdqualParams, sqlStr)
   cur.execute(sqlStr)

   # Align to start of day
   startTime = startTime - (startTime % ONE_DAY)
   while startTime < endTime:
      timeT = time.gmtime(startTime)
      filename = time.strftime(cdqualParams['filePath'], timeT)
      filename += '/' + station + '_ALL.'
      filename += time.strftime('%Y%m%d.0000', timeT)

      for ext in ('.wfdisc', '.qlr'):
         try:
             fp = open(filename + ext, "rb")

         except:
            # sys.exit("Error: can't open '%s' for reading" % (filename + ext,))
            debug(5, cdqualParams,
                  "Warning: can't open '%s' for reading" % (filename,))
            continue

         for line in fp:
            #line = re.sub(' +',',', line)
            fld = line.split()
            if ext == '.wfdisc':
               sqlStr = 'INSERT INTO wfdisc ' + \
                        '(sta, chan, time, endtime, nsamp, samprate, ' + \
                          'datatype, dir, dfile, foff) ' + \
                        "VALUES (?,?,?,?,?,?,?,?,?,?)" + '\n'
               debug(5, cdqualParams, (sqlStr, fld[0], fld[1], fld[2],
                                       fld[6], fld[7], fld[8], fld[13],
                                       fld[15], fld[16], fld[17]))
               cur.execute(sqlStr, (fld[0], fld[1], fld[2], fld[6], fld[7],
                                    fld[8], fld[13], fld[15], fld[16],
                                    fld[17]))
            else:
               sqlStr = 'INSERT INTO cdquality ' + \
                        '(sta, chan, time, endtime, nsamp, ' + \
                          'qualityflags, meancounts, m2, chanstatus) ' + \
                        "VALUES (?,?,?,?,?,?,?,?,?)" + '\n'
               debug(5, cdqualParams, (sqlStr, fld[0], fld[1], fld[2], fld[5],
                                       fld[6], fld[8], fld[15], fld[17],
                                       fld[9]))
               cur.execute(sqlStr, (fld[0], fld[1], fld[2], fld[5], fld[6],
                                    fld[8], fld[15], fld[17], fld[9]))

         fp.close()
      startTime += ONE_DAY

# -----------------------------------------------------------------------------
# @short Print or add report headers
#
# This function prints or adds report headers. This is done only once
# in non file mode to avoid multiple headers
#
# @param cdqualParams dictionary containing name value pairs
# @param reportType   type of report: CHAN_STATUS, STA_STATUS, OUTAGE
#                                     CHAN_STATISTICS, STA_STATISTICS,
#                                     CHAN_STATUS:2, STA_STATUS:2,
#                                     AUTH_INTERVAL, AUTH_STATUS
# @param cfd          cvs writer object (if DSV document)
# @param doc          document tree (if XML document)
#
# @return nothing
# -----------------------------------------------------------------------------

def addHeader(cdqualParams, reportType, cfd, nS):

   # Print or add report headers. This is done only once
   # in non file mode to avoid multiple headers
   # ---------------------------------------------------------------
   if reportType == 'CHAN_STATUS':
      # ------------------------------------------------------------
      # Channel status V1.0
      # ------------------------------------------------------------
      rowCont = ('Net', 'Sta', 'Chan', '%_Recvd', '%_Avail', 'Gaps',
                 'Samples', 'Constant', 'Mean', 'RMS' )

      if cdqualParams['outFormat'] == 'dsv':
         cfd.writerow(rowCont)
      elif cdqualParams['outFormat'] == 'xml':
         nS.append('c:')
         nS.append('channelstatus')
         nS.append('Channel Status')
         nS.append('1.0')
      elif cdqualParams['outFormat'] == 'json':
         printJsonHeader(cdqualParams, reportType)
      else: # IMS format
         print "Channel Status"
         print "%-9s %-5s %-4s %-7s %-7s %5s %9s %9s %11s %11s" % rowCont

   elif reportType == 'CHAN_STATUS:2' or reportType == 'CHAN_STATUS:3':
      # ------------------------------------------------------------
      # Channel status V2.0
      # ------------------------------------------------------------
      if reportType == 'CHAN_STATUS:2':
         rowCont = ('Net', 'Sta', 'Chan', '%_Recvd', '%_Avail', '%_AvaSu',
                    'Gaps', 'Samples', 'Constant', 'Mean', 'RMS' )
      else:
         rowCont = ('Net', 'Sta', 'Chan', '%_Recvd', '%_AvaUA', '%_Avail',
                    'Gaps', 'Samples', 'Constant', 'Mean', 'RMS' )

      if cdqualParams['outFormat'] == 'dsv':
         cfd.writerow(rowCont)
      elif cdqualParams['outFormat'] == 'xml':
         nS.append('c:')
         nS.append('channelstatus')
         nS.append('Channel Status')
         if reportType == 'CHAN_STATUS:2':
            nS.append('2.0')
         else:
            nS.append('3.0')
      elif cdqualParams['outFormat'] == 'json':
         printJsonHeader(cdqualParams, reportType)
      else: # IMS format
         if reportType == 'CHAN_STATUS:2':
            print "Channel Status"
         else:
            print "Channel Status"
         print "%-9s %-5s %-4s %-7s %-7s %-7s %5s %9s %9s %11s %11s" \
                % rowCont

   elif reportType == 'CHAN_STATISTICS':
      # ------------------------------------------------------------
      # Channel statistics
      # ------------------------------------------------------------
      rowCont = ('Net', 'Sta', 'Chan', 'WFDrecs', 'CDQrecs', 'Gaps',
                 'Lsegs', 'Csegs', 'LCsegs', '%_Recvd')

      if cdqualParams['outFormat'] == 'dsv':
         cfd.writerow(rowCont)
      elif cdqualParams['outFormat'] == 'xml':
         nS.append('c:')
         nS.append('channelstatistics')
         nS.append('Channel Statistics')
         nS.append('1.0')
      elif cdqualParams['outFormat'] == 'json':
         printJsonHeader(cdqualParams, reportType)
      else: # IMS format
         print "Channel Statistics"
         print "%-9s %-5s %-4s %7s %7s %8s %8s %8s %8s %7s" % \
                rowCont

   elif reportType == 'STA_STATISTICS':
      # ------------------------------------------------------------
      # Station statistics
      # ------------------------------------------------------------
      # Minimum channels (set) means channels processed!
      if cdqualParams['outFormat'] == 'dsv':
         cfd.writerow(('Sta', 'Max_Exp_Time',
                     'Minimum Channels Data Received (%)',
                     'Minimum Channels Data Availability (%)',
         'Minimum Channels Data Availability with Surety (%)',
                     'Minimum Channels Timely Data Received (%)',
                     'Minimum Channels Timely Data Availability (%)',
         'Minimum Channels Timely Data Availability with Surety (%)',
                     'Minimum Channels Mission Capability (%)',
                     'Minimum Channels Mission Capability with Surety (%)',
                     'Eligible Channels Data Received (%)',
                     'Eligible Channels Data Availability (%)',
         'Eligible Channels Data Availability with Surety (%)',
                     'Eligible Channels Timely Data Received (%)',
                     'Eligible Channels Timely Data Availability (%)',
         'Eligible Channels Timely Data Availability with Surety (%)',
                     'Geophysical Channels Data Received (%)',
                     'Geophysical Channels Data Availability (%)',
         'Geophysical Channels Data Availability with Surety (%)',
                     'Geophysical Channels Timely Data Received (%)',
                     'Geophysical Channels Timely Data Availability (%)',
         'Geophysical Channels Timely Data Availability with Surety (%)',
                     'All Channels Data Received (%)',
                     'All Channels Data Availability (%)',
         'All Channels Data Availability with Surety (%)',
                     'All Channels Timely Data Received (%)',
                     'All Channels Timely Data Availability (%)',
         'All Channels Timely Data Availability with Surety (%)'))
      elif cdqualParams['outFormat'] == 'xml':
         nS.append('s:')
         nS.append('stationstatistics')
         nS.append('Station Statistics')
         nS.append('1.0')
      elif cdqualParams['outFormat'] == 'json':
         printJsonHeader(cdqualParams, reportType)
      else: # IMS format
         # HINT: Set noai and nocindent and copy text lines from example file,
         # re-format lines and set ai and cindent
         print "Station Statistics"
         print 'Sta   Max_Exp_Time   \
______________________________________________________ Minimum Channels \
______________________________________________________  \
_____________________________________ Eligible Channels \
______________________________________  ____________________________________ \
Geophysical Channels ____________________________________  \
_______________________________________ All Channels \
_________________________________________'
         print '                         Data            Data            \
Data         Timely Data     Timely Data     Timely Data       Mission\
         Mission          Data            Data            Data         \
Timely Data     Timely Data     Timely Data        Data            Data\
            Data         Timely Data     Timely Data     Timely Data\
        Data            Data            Data         Timely Data     \
Timely Data     Timely Data'
         print '                     Received (%)    Availability    \
Availability    Received (%)    Availability    Availability   \
Capability (%)    Capability     Received (%)    Availability    \
Availability    Received (%)    Availability    Availability    \
Received (%)    Availability    Availability    Received (%)    \
Availability    Availability    Received (%)    Availability    \
Availability    Received (%)    Availability    Availability'
         print '                                          (%)       \
with Surety (%)                       (%)       with Surety (%)\
                 with Surety (%)                       (%)       \
with Surety (%)                       (%)       with Surety (%)\
                       (%)       with Surety (%)                       (%)\
       with Surety (%)                       (%)       with Surety (%)\
                       (%)       with Surety (%)'

   elif reportType == 'STA_STATISTICS:2':
      # ------------------------------------------------------------
      # Station statistics V2.0
      # ------------------------------------------------------------
      # Minimum channels (set) means channels processed!
      if cdqualParams['outFormat'] == 'dsv':
         cfd.writerow(('Sta', 'Max_Exp_Time',
                     'Minimum Channels Data Received (%)',
         'Minimum Channels Data Availability Unauthenticated (%)',
                     'Minimum Channels Data Availability (%)',
                     'Minimum Channels Timely Data Received (%)',
         'Minimum Channels Timely Data Availability Unauthenticated (%)',
                     'Minimum Channels Timely Data Availability (%)',
                     'Minimum Channels Mission Capability Unauthenticated (%)',
                     'Minimum Channels Mission Capability (%)',
                     'Eligible Channels Data Received (%)',
                     'Eligible Channels Data Availability Unauthenticated (%)',
                     'Eligible Channels Data Availability (%)',
                     'Eligible Channels Timely Data Received (%)',
         'Eligible Channels Timely Data Availability Unauthenticated (%)',
         'Eligible Channels Timely Data Availability (%)',
                     'Geophysical Channels Data Received (%)',
         'Geophysical Channels Data Availability Unauthenticated (%)',
                     'Geophysical Channels Data Availability (%)',
                     'Geophysical Channels Timely Data Received (%)',
         'Geophysical Channels Timely Data Availability Unauthenticated (%)',
                     'Geophysical Channels Timely Data Availability (%)',
                     'All Channels Data Received (%)',
                     'All Channels Data Availability Unauthenticated (%)',
                     'All Channels Data Availability (%)',
                     'All Channels Timely Data Received (%)',
         'All Channels Timely Data Availability Unauthenticated (%)',
                     'All Channels Timely Data Availability (%)'))
      elif cdqualParams['outFormat'] == 'xml':
         nS.append('s:')
         nS.append('stationstatistics')
         nS.append('Station Statistics')
         nS.append('2.0')
      elif cdqualParams['outFormat'] == 'json':
         printJsonHeader(cdqualParams, reportType)
      else: # IMS format
         # HINT: Set noai and nocindent and copy text lines from example file,
         # re-format lines and set ai and cindent
         print "Station Statistics"
         print 'Sta   Max_Exp_Time   \
______________________________________________________ Minimum Channels \
______________________________________________________  \
_____________________________________ Eligible Channels \
______________________________________  ____________________________________ \
Geophysical Channels ____________________________________  \
_______________________________________ All Channels \
_________________________________________'
         print '                         Data            Data            \
Data         Timely Data     Timely Data     Timely Data       Mission\
         Mission          Data            Data            Data         \
Timely Data     Timely Data     Timely Data        Data            Data\
            Data         Timely Data     Timely Data     Timely Data\
        Data            Data            Data         Timely Data     \
Timely Data     Timely Data'
         print '                     Received (%)    Availability\
    Availability    Received (%)    Availability    Availability\
     Capability    Capability (%)   Received (%)    Availability\
    Availability    Received (%)    Availability    Availability\
    Received (%)    Availability    Availability    Received (%)\
    Availability    Availability    Received (%)    Availability\
    Availability    Received (%)    Availability    Availability'
         print '                                    Unauthenticated\
       (%)                       Unauthenticated       (%)\
       Unauthenticated                                 Unauthenticated\
       (%)                       Unauthenticated                       \
          Unauthenticated       (%)                       Unauthenticated\
       (%)                       Unauthenticated       (%)\
                       Unauthenticated       (%)'
         print '                                          (%)      \
                                       (%)                             (%)\
                                             (%)                         \
                    (%)                                             (%) \
                                            (%)                         \
                    (%)                                             (%)'

   elif reportType == 'STA_STATUS':
      # ------------------------------------------------------------
      # Station Status
      # ------------------------------------------------------------
      if cdqualParams['outFormat'] == 'dsv':
         cfd.writerow(('Sta', 'Max_Exp_Time', 'Data Received (%)',
                     'Data Availability (%)',
                     'Timely Data Availability (%)',
                     'Mission Capability (%)',
                     'Data Received Authenticated (%)'))
      elif cdqualParams['outFormat'] == 'xml':
         nS.append('s:')
         nS.append('stationstatus')
         nS.append('Station Status')
         nS.append('1.0')
      elif cdqualParams['outFormat'] == 'json':
         printJsonHeader(cdqualParams, reportType)
      else: # IMS format
         print "Station Status"
         print "%-5s %-17s %-14s %-12s %-17s %-13s %s" % ('Sta',
                'Max_Exp_Time',
                'Data', 'Data', 'Timely Data', 'Mission', 'Data Received')
         print "%20s%-12s %-16s %-16s %-14s %-17s" % ('', 'Received (%)',
                'Availability (%)', 'Availability (%)', 'Capability (%)',
                'Authenticated (%)')

   elif reportType == 'STA_STATUS:2':
      # ------------------------------------------------------------
      # Station Status V2.0
      # ------------------------------------------------------------
      if cdqualParams['outFormat'] == 'dsv':
         cfd.writerow(('Sta', 'Max_Exp_Time',
         'Minimum Channels Data Availability with Surety (%)',
         'Minimum Channels Timely Data Availability with Surety (%)',
         'Minimum Channels Mission Capability with Surety (%)',
                     'Geophysical Channels Data Received (%)',
                     'Geophysical Channels Data Availability (%)',
         'Geophysical Channels Data Availability with Surety (%)'))
      elif cdqualParams['outFormat'] == 'xml':
         nS.append('s:')
         nS.append('stationstatus')
         nS.append('Station Status')
         nS.append('2.0')
      elif cdqualParams['outFormat'] == 'json':
         printJsonHeader(cdqualParams, reportType)
      else: # IMS format
         print "Station Status"
         print 'Sta   Max_Exp_Time  ______________ Minimum Channels\
 _______________ ____________ Geophysical Channels _____________'
         print '                         Data         Timely Data\
       Mission          Data            Data            Data'
         print '                     Availability    Availability\
     Capability     Received (%)    Availability    Availability'
         print '                    with Surety (%) with Surety (%)\
 with Surety (%)                       (%)       with Surety (%)'

   elif reportType == 'STA_STATUS:3':
      # ------------------------------------------------------------
      # Station Status V3.0
      # ------------------------------------------------------------
      if cdqualParams['outFormat'] == 'dsv':
         cfd.writerow(('Sta', 'Max_Exp_Time',
         'Minimum Channels Data Availability (%)',
         'Minimum Channels Timely Data Availability (%)',
         'Minimum Channels Mission Capability (%)',
         'Geophysical Channels Data Received (%)',
         'Geophysical Channels Data Availability Unauthenticated (%)',
         'Geophysical Channels Data Availability (%)'))
      elif cdqualParams['outFormat'] == 'xml':
         nS.append('s:')
         nS.append('stationstatus')
         nS.append('Station Status')
         nS.append('3.0')
      elif cdqualParams['outFormat'] == 'json':
         printJsonHeader(cdqualParams, reportType)
      else: # IMS format
         print "Station Status"
         print 'Sta   Max_Exp_Time  ______________ Minimum Channels\
 _______________ ____________ Geophysical Channels _____________'
         print '                         Data         Timely Data\
       Mission          Data            Data            Data'
         print '                     Availability    Availability\
   Capability (%)   Received (%)    Availability    Availability'
         print '                          (%)             (%)\
                                       Unauthenticated       (%)'
         print '                                               \
                                           (%)'
   elif reportType == 'AUTH_INTERVAL':
      # ------------------------------------------------------------
      # Authentication status interval
      # ------------------------------------------------------------
      rowCont = ('Net', 'Sta', 'Chan', 'Start Date Time',
                 'End Date Time', 'Comment' )

      if cdqualParams['outFormat'] == 'dsv':
         cfd.writerow(rowCont)
      elif cdqualParams['outFormat'] == 'xml':
         nS.append('o:')
         nS.append('authenticationinterval')
         nS.append('Authentication Interval')
         nS.append('1.0')
      elif cdqualParams['outFormat'] == 'json':
         printJsonHeader(cdqualParams, reportType)
      else: # IMS format
         print "Failed Intervals"
         print "%-9s %-6s%-9s%-25s%-19s%-7s" % rowCont
   elif reportType == 'AUTH_STATUS':
      # ------------------------------------------------------------
      # Authentication status
      # ------------------------------------------------------------
      if cdqualParams['outFormat'] == 'dsv':
         cfd.writerow(('Net', 'Sta',
                     'Frames Received',
                     'Frames Checked',
                     'Frames Failed',
                     'Channel Sub-Frames Received',
                     'Channel Sub-Frames Checked',
                     'Channel Sub-Frames Failed'))
      elif cdqualParams['outFormat'] == 'xml':
         nS.append('o:')
         nS.append('authenticationstatus')
         nS.append('Authentication Status')
         nS.append('1.0')
      elif cdqualParams['outFormat'] == 'json':
         printJsonHeader(cdqualParams, reportType)
      else: # IMS format
         print "Authentication Status"
         print "%-10s%-6s%s %s" % ('Net', 'Sta',
               '____________ Frames ____________',
               '______ Channel Sub-Frames ______')

         print "%18s%-12s%-12s%-9s%-12s%-12s%s" % ('',
                'Received', 'Checked', 'Failed',
                'Received', 'Checked', 'Failed')
   else:
      # ------------------------------------------------------------
      # Outage
      # ------------------------------------------------------------
      rowCont = ('Net', 'Sta', 'Chan', 'Start Date Time',
                 'End Date Time', 'Duration', 'Comment' )

      if cdqualParams['outFormat'] == 'dsv':
         cfd.writerow(rowCont)
      elif cdqualParams['outFormat'] == 'xml':
         nS.append('o:')
         nS.append('outage')
         nS.append('Outage')
         nS.append('1.0')
      elif cdqualParams['outFormat'] == 'json':
         printJsonHeader(cdqualParams, reportType)
      else: # IMS format
         print "Outage"
         print "%-9s %-6s%-9s%-25s%-21s%-9s%-7s" % rowCont


# -----------------------------------------------------------------------------
# @short Print or add report headers
#
# This function prints or adds report headers. This is done only once
# in non file mode to avoid multiple headers
#
# @param cdqualParams dictionary containing name value pairs
# @param reportType   type of report: CHAN_STATUS, STA_STATUS, OUTAGE
#                                     CHAN_STATISTICS, STA_STATISTICS
#                                     CHAN_STATUS:2, STA_STATUS:2,
#                                     AUTH_INTERVAL, AUTH_STATUS
#
# @return row Format
# -----------------------------------------------------------------------------

def setRowFormat(cdqualParams, reportType):

   # Set row format. DSV and XML do not need a row format
   # ---------------------------------------------------------------
   if reportType == 'CHAN_STATUS':
      # ------------------------------------------------------------
      # Channel status
      # ------------------------------------------------------------
      if cdqualParams['outFormat'] == 'json':
         return '''   "network": "%s",\n   "station": "%s",
   "channel": "%s",\n   "received": %.3f,\n   "available": %.3f,\n   "gaps": %d,
   "samples": %d,\n   "constant": %d,\n   "mean": %.1f,\n   "rms": %.1f'''
      else: # IMS format
         return  "%-9s %-5s %-4s %7.3f %7.3f %5d %9d% 10d% 12.1f %11.1f"

   elif reportType == 'CHAN_STATUS:2':
      # ------------------------------------------------------------
      # Channel status V2.0
      # ------------------------------------------------------------
      if cdqualParams['outFormat'] == 'json':
         return '''   "network": "%s",\n   "station": "%s",
   "channel": "%s",\n   "received": %.3f,\n   "available": %.3f,
   "availablewithsurety": %.3f,\n   "gaps": %d,
   "samples": %d,\n   "constant": %d,\n   "mean": %.1f,\n   "rms": %.1f'''
      else: # IMS format
         return "%-9s %-5s %-4s %7.3f %7.3f %7.3f %5d %9d% 10d%\
12.1f %11.1f"
   elif reportType == 'CHAN_STATUS:3':
      # ------------------------------------------------------------
      # Channel status V3.0
      # ------------------------------------------------------------
      if cdqualParams['outFormat'] == 'json':
         return '''   "network": "%s",\n   "station": "%s",
   "channel": "%s",\n   "received": %.3f,\n   "availableunauthenticated": %.3f,
   "available": %.3f,\n   "gaps": %d,
   "samples": %d,\n   "constant": %d,\n   "mean": %.1f,\n   "rms": %.1f'''
      else: # IMS format
         return "%-9s %-5s %-4s %7.3f %7.3f %7.3f %5d %9d% 10d%\
12.1f %11.1f"

   elif reportType == 'CHAN_STATISTICS':
      # ------------------------------------------------------------
      # Channel statistics
      # ------------------------------------------------------------
      if cdqualParams['outFormat'] == 'json':
         return '''   "network": "%s",\n   "station": "%s",
   "channel": "%s",\n   "wfdiscrecords": %d,\n   "cdqualityrecords": %d,
   "gaps": %d,\n   "latesegments": %d,\n   "constantsegments": %d,
   "lateconstantsegments": %d,\n   "received": %.3f'''
      else: # IMS format
         return "%-9s %-5s %-4s %7d %7d %8d %8d %8d %8d %7.3f"

   elif reportType == 'STA_STATISTICS':
      # ------------------------------------------------------------
      # Station statistics
      # ------------------------------------------------------------
      if cdqualParams['outFormat'] == 'json':
         return '''   "station": "%s",
   "maxexptime": "%d",
   "minimumchannels": {
      "received": %.3f,
      "available": %.3f,
      "availablewithsurety": %.3f,
      "timelyreceived": %.3f,
      "timelyavailable": %.3f,
      "timelyavailablewithsurety": %.3f,
      "missioncapable": %.3f,
      "missioncapablewithsurety": %.3f
   },
   "eligiblechannels": {
      "received": %.3f,
      "available": %.3f,
      "availablewithsurety": %.3f,
      "timelyreceived": %.3f,
      "timelyavailable": %.3f,
      "timelyavailablewithsurety": %.3f
   },
   "geophysicalchannels": {
      "received": %.3f,
      "available": %.3f,
      "availablewithsurety": %.3f,
      "timelyreceived": %.3f,
      "timelyavailable": %.3f,
      "timelyavailablewithsurety": %.3f
   },
   "allchannels": {
      "received": %.3f,
      "available": %.3f,
      "availablewithsurety": %.3f,
      "timelyreceived": %.3f,
      "timelyavailable": %.3f,
      "timelyavailablewithsurety": %.3f
   }'''
      else: # IMS format
         rowFormat = "%%-5s %%04d %%02d:%%02d:%%02d%5s\
%%7.3f%9s%%7.3f%9s%%7.3f%9s%%7.3f%9s%%7.3f%9s%%7.3f%9s\
%%7.3f%9s%%7.3f%9s\
%%7.3f%9s%%7.3f%9s%%7.3f%9s%%7.3f%9s%%7.3f%9s%%7.3f%9s\
%%7.3f%9s%%7.3f%9s%%7.3f%9s%%7.3f%9s%%7.3f%9s%%7.3f%9s\
%%7.3f%9s%%7.3f%9s%%7.3f%9s%%7.3f%9s%%7.3f%9s%%7.3f"
         return rowFormat % ( '', '', '', '', '', '', '', '',
               '','','','','','','','','','','','', '', '', '', '', '', '')

   elif reportType == 'STA_STATISTICS:2':
      # ------------------------------------------------------------
      # Station statistics V2.0
      # ------------------------------------------------------------
      if cdqualParams['outFormat'] == 'json':
         return '''   "station": "%s",
   "maxexptime": "%d",
   "minimumchannels": {
      "received": %.3f,
      "availableunauthenticated": %.3f,
      "available": %.3f,
      "timelyreceived": %.3f,
      "timelyavailableunauthenticated": %.3f,
      "timelyavailable": %.3f,
      "missioncapableunauthenticated": %.3f,
      "missioncapable": %.3f
   },
   "eligiblechannels": {
      "received": %.3f,
      "availableunauthenticated": %.3f,
      "available": %.3f,
      "timelyreceived": %.3f,
      "timelyavailableunauthenticated": %.3f,
      "timelyavailable": %.3f
   },
   "geophysicalchannels": {
      "received": %.3f,
      "availableunauthenticated": %.3f,
      "available": %.3f,
      "timelyreceived": %.3f,
      "timelyavailableunauthenticated": %.3f,
      "timelyavailable": %.3f
   },
   "allchannels": {
      "received": %.3f,
      "availableunauthenticated": %.3f,
      "available": %.3f,
      "timelyreceived": %.3f,
      "timelyavailableunauthenticated": %.3f,
      "timelyavailable": %.3f
   }'''
      else: # IMS format
         rowFormat = "%%-5s %%04d %%02d:%%02d:%%02d%5s\
%%7.3f%9s%%7.3f%9s%%7.3f%9s%%7.3f%9s%%7.3f%9s%%7.3f%9s\
%%7.3f%9s%%7.3f%9s\
%%7.3f%9s%%7.3f%9s%%7.3f%9s%%7.3f%9s%%7.3f%9s%%7.3f%9s\
%%7.3f%9s%%7.3f%9s%%7.3f%9s%%7.3f%9s%%7.3f%9s%%7.3f%9s\
%%7.3f%9s%%7.3f%9s%%7.3f%9s%%7.3f%9s%%7.3f%9s%%7.3f"
         return rowFormat % ( '', '', '', '', '', '', '', '',
               '','','','','','','','','','','','', '', '', '', '', '', '')

   elif reportType == 'STA_STATUS':
      # ------------------------------------------------------------
      # Station status
      # ------------------------------------------------------------
      if cdqualParams['outFormat'] == 'json':
         return '''   "station": "%s",
   "maxexptime": "%d",
   "received": %.3f,\n   "available": %.3f,\n   "timelyavailable": %.3f,
   "missioncapable": %.3f,\n   "receivedauthenticated": %.3f'''
      else: #IMS format
         rowFormat = "%%-5s %%04d %%02d:%%02d:%%02d%3s" + \
                     "%%7.3f%8s%%7.3f%10s%%7.3f%9s%%7.3f%10s%%7.3f"
         return rowFormat % ( ' ', ' ', ' ', ' ', ' ')

   elif reportType == 'STA_STATUS:2':
      # ------------------------------------------------------------
      # Station status extended
      # ------------------------------------------------------------
      rowFormat = "%%-5s %%04d %%02d:%%02d:%%02d%5s" + \
                  "%%7.3f%9s%%7.3f%9s%%7.3f%9s%%7.3f%9s%%7.3f%9s%%7.3f"

      if cdqualParams['outFormat'] == 'json':
         return '''   "station": "%s",
   "maxexptime": "%d",
   "minimumchannels": {
      "availablewithsurety": %.3f,
      "timelyavailablewithsurety": %.3f,
      "missioncapablewithsurety": %.3f
   },
   "geophysicalchannels": {
      "received": %.3f,
      "available": %.3f,
      "availablewithsurety": %.3f
   }'''
      else:
         return rowFormat % ( ' ', ' ', ' ', ' ', ' ', ' ' )

   elif reportType == 'STA_STATUS:3':
      # ------------------------------------------------------------
      # Station status extended, availability considers now surety
      # ------------------------------------------------------------
      rowFormat = "%%-5s %%04d %%02d:%%02d:%%02d%5s" + \
                  "%%7.3f%9s%%7.3f%9s%%7.3f%9s%%7.3f%9s%%7.3f%9s%%7.3f"

      if cdqualParams['outFormat'] == 'json':
         return '''   "station": "%s",
   "maxexptime": "%d",
   "minimumchannels": {
      "available": %.3f,
      "timelyavailable": %.3f,
      "missioncapable": %.3f
   },
   "geophysicalchannels": {
      "received": %.3f,
      "availableunauthenticated": %.3f,
      "available": %.3f
   }'''
      else:
         return rowFormat % ( ' ', ' ', ' ', ' ', ' ', ' ' )

   elif reportType == 'AUTH_STATUS':
      # ------------------------------------------------------------
      # Authentication status
      # ------------------------------------------------------------
      if cdqualParams['outFormat'] == 'json':
         return '''   "network": "%s",\n   "station": "%s",
   "frames": {
      "received": "%lu",\n      "checked": %lu,\n      "failed": %lu
    },
    "channelsubframes": {
      "received": %lu,\n      "checked": %lu,\n      "failed": %lu
    },'''
      else: # IMS format
         return "%-10s%-6s%10lu %10lu %10lu %10lu %10lu %10lu"

   elif reportType == 'AUTH_INTERVAL':
      # ------------------------------------------------------------
      # authentication interval
      # ------------------------------------------------------------
      if cdqualParams['outFormat'] == 'json':
            return '''     "network": "%s",\n     "station": "%s",
     "channel": "%s",\n     "start": "%s %s",\n     "end": "%s %s",
     "duration": "%s",\n     "comment": "%s"'''
      # No IMS is OK here
   else:
      # ------------------------------------------------------------
      # outage
      # ------------------------------------------------------------
      if cdqualParams['outFormat'] == 'json':
            return '''   "network": "%s",\n   "station": "%s",
   "channel": "%s",\n   "start": "%s %s",\n   "end": "%s %s",
   "duration": "%s",\n   "comment": "%s"'''
      # No IMS is OK here

   return ''


# -----------------------------------------------------------------------------
# @short Merge authentication gaps
#
# This function merges consecutive authentication gaps
#
# @param cdqualParams dictionary containing name value pairs
# @param cStat        Channel description and statistics
#
# @return nothing
# -----------------------------------------------------------------------------

def mergeAuthGaps(cdqualParams, cStat):

   oneSample = 1 / cStat[SAMP_RATE]
   authGaps = cStat[AUTH_GAP_LIST]
   lenAuthGapList = len(authGaps)
   for ii in range(1, lenAuthGapList):
      debug(5, cdqualParams, ("merge ", authGaps[ii - 1][G_END],
            "with", authGaps[ii][G_BEG],
            authGaps[ii][G_TYP], authGaps[ii - 1][G_TYP],
            authGaps[ii][G_BEG] - authGaps[ii - 1][G_END]))
      if (authGaps[ii][G_BEG] - authGaps[ii - 1][G_END]) < oneSample and \
          authGaps[ii][G_TYP] == authGaps[ii - 1][G_TYP]:
        debug(5, cdqualParams, ("merge ", authGaps[ii - 1][G_END],
              "with", authGaps[ii][G_BEG],
              authGaps[ii][G_BEG] - authGaps[ii - 1][G_END]))
        authGaps[ii][G_BEG] = authGaps[ii - 1][G_BEG]
        authGaps[ii - 1][G_BEG] = 0
        # Don't optimize here like del authGaps[ii - 1]

   # No remove all unnecessary gaps
   for gap in authGaps[:]:
      if gap[G_BEG] == 0:
         authGaps.remove(gap)



# -----------------------------------------------------------------------------
# @short Print json failed authentication interval
#
# This function prints json failed authentication intervals
#
# @param cdqualParams dictionary containing name value pairs
# @param cStatArray   Array of channel descriptions
#
# @return nothing
# -----------------------------------------------------------------------------

def printAuthIntervalJSON(cdqualParams, cStatArray):

   jrs = '    "failedintervals": [\n    {'''
   rowFormat = '''     "channel": "%s",
     "start": "%sT%sZ",
     "end": "%sT%sZ",
     "comment": "%s"'''
#     "duration": "%s",

   for cStat in cStatArray:
     # rowForOut = "%-4s %s" % (cStat[CHANNEL], "%-20s%03d %-20s%03d %10.3f %s")
     rowForOut = "%-4s %s" % (cStat[CHANNEL], "%-20s%03d %-20s%03d %s")

     gaps = cStat[AUTH_GAP_LIST]
     for gap in gaps:
         actLine = rowForOut % getAuthGapLine(gap)
         lst = actLine.split(None, 5)
         print jrs
         if len(lst) == 5:
            lst.append('')
         # Replace 2013/05/04 with 2013-05-04
         lst[1] = lst[1].replace('/','-')
         lst[3] = lst[3].replace('/','-')
         debug(2, cdqualParams, lst);
         print rowFormat % tuple(lst)
         # Close the record
         jrs = '    },\n    {'

   print '    }\n    ]'


# -----------------------------------------------------------------------------
# @short Split report
#
# This function splits a record in its sub-reports. E.g. AUTH_STATUS is
# split in AUTH_STATUS and AUTH_INTERVAL
#
# @param reportType   type of report: CHAN_STATUS, STA_STATUS, OUTAGE
#                                     CHAN_STATISTICS, STA_STATISTICS
#                                     CHAN_STATUS:2, STA_STATUS:2,
#                                     AUTH_INTERVAL, AUTH_STATUS
# @param outFormat    format of report: ims, dsv, json or xml
#
# @return split report type
# -----------------------------------------------------------------------------

def splitReport( reportType, outFormat):

   if reportType == 'AUTH_STATUS' and outFormat in ['ims','dsv']:
      return [ 'AUTH_STATUS', 'AUTH_INTERVAL' ]
   else:
      return [ reportType, ]

# -----------------------------------------------------------------------------
# @short Process CLI arguments
#
# This function processes the CLI arguments one by one.
# An argument might be a site/channel combination, a network
# or an array name
#
# @param cdqualParams dictionary containing name value pairs
# @param reportType   type of report: CHAN_STATUS, STA_STATUS, OUTAGE
#                                     CHAN_STATISTICS, STA_STATISTICS
#                                     CHAN_STATUS:2, STA_STATUS:2,
#                                     AUTH_INTERVAL, AUTH_STATUS
#
# @return nothing
# -----------------------------------------------------------------------------

def processArgs(cdqualParams, reportType):

   isFirst = 1
   jrs = '  {'

   for arg in cdqualParams['args']:
      # Check for file mode. For each of the stations/networks
      # in args an own file is created in file mode
      # ---------------------------------------------------------------
      try:
         if cdqualParams['fileMode']:
            fileName = '%s.%d-%d.%s.' % (arg.replace('/','_'),
                                         cdqualParams['startTime'],
                                         cdqualParams['endTime'], reportType )
            ext = cdqualParams['outFormat']
            jrs = '  {'
            if ext == 'dsv':
               # make life easier for users
               ext = 'csv'

            fileName = fileName + ext
            # Check if the output file exists
            try:
                info = os.stat(fileName)
            except OSError, why:
                debug(5, cdqualParams, ('create file ', fileName))
            else:
                sys.exit("Output file %s exists, remove it first" % fileName)
            saveStdout = sys.stdout
            fd = open(fileName, 'wb', buffering = 0)
            sys.stdout = fd
      except IOError, why:
         sys.exit("Error: can't open output file: %s" % why)

      # For ims output format sys.stdout has been redirected before
      # For dsv open csv-writer and for XML open document.
      if cdqualParams['fileMode'] or isFirst:
         # Create name space
         nS = [ '' ]
         cfd = 0

         if cdqualParams['outFormat'] == 'dsv':
            # print csv.list_dialects()
            cfd = csv.writer(sys.stdout, dialect='ims')
         # For XML use minidom-writer
         elif cdqualParams['outFormat'] == 'xml':
            doc = xml.dom.minidom.Document()
         elif cdqualParams['outFormat'] == 'ims':
            printReportTime(cdqualParams)

         # Print or add headers
         addHeader(cdqualParams, reportType, cfd, nS)

         if cdqualParams['outFormat'] == 'xml':
            XMLrecords = addXMLheader(doc, cdqualParams, nS)

      isFirst = 0

      # ---------------------------------------------------------------
      # End file mode or first CLI argument
      # reset later to avoid fist duplicated header isFirst = 0
      repNbr = 0
      for report in splitReport(reportType, cdqualParams['outFormat']):

         if repNbr > 0 and (cdqualParams['outFormat'] == 'ims' or \
            cdqualParams['outFormat'] == 'dsv'):
            addHeader(cdqualParams, report, cfd, nS)
         repNbr = 1

         # Set row format. DSV and XML do not need a row format
         rowFormat = setRowFormat(cdqualParams, report)

         # Check if station name contains also a channel name
         # Syntax: station/channel
         uncheckedChannels = ''
         mustCheckChannels = 1

         try:
            (station, uncheckedChannels) = arg.split('/', 1)
         except ValueError, why:
            mustCheckChannels = 0
            station = arg

         # Station could be a single station, an array or a "network"
         # Single stations and arrays are covered by stationOrArrayToSites(),
         # but the check for the "network" has to be done here!
         stations = networkToStation(station, cdqualParams)

         for station in stations:
            # Get Array name (if any)
            arrayName = isArray(station, cdqualParams)
            cStatArray = []
            channelCnt = 0
            maxExpTimeMax = 0

            stationStats = [ 0, 0, 0, 0, 0, 0, 0,
                             0, 0, 0, 0, 0, 0, 0,
                             0, 0, 0, 0, 0, 0, 0,
                             0, 0, 0, 0, 0, 0, 0 ]
            stationStatsT = [ 0, 0, 0, 0, 0, 0, 0, 0,
                              0, 0, 0, 0, 0, 0, 0, 0,
                              0, 0, 0, 0, 0, 0, 0, 0,
                              0, 0, 0, 0, 0, 0, 0, 0 ]

            # Check if this station sends non-continuous data
            NCDStation = checkAuxTable(cdqualParams, (arrayName, station))

            # Check if cdqual runs in file mode
            if cdqualParams['noDB']:
               populateTempDB(station, cdqualParams)

            # Get sites and channels of station
            siteChannels = stationOrArrayToSites(station, cdqualParams)

            # If the user has provided channels, check them
            # siteChannels is modified in filterSiteChan()
            debug(2, cdqualParams, siteChannels)
            if mustCheckChannels:
               filterSiteChan(siteChannels, uncheckedChannels)

            debug(2, cdqualParams, siteChannels)

            # Load frame values if report type is AUTH_STATUS
            frames = [];
            if report == 'AUTH_STATUS':
               # NOTE: This will include all frames received during the
               # report period, NOT frames received during the request
               # period if is an auxiliary station!
               if loadFrameValues(station, cdqualParams, frames):
                  # No channel names found, skip station or exit
                  ## if cdqualParams['outFormat'] == 'json':
                     # Need close record and close report
                     ## print " ]\n}\n"
                  ## sys.exit("Error: can't find channel name frames " + \
                  ##          'in clf file. Too old CD Receiver version?')
                  continue

               siteValues = []

            # For each site/channel combination
            for siteChan in siteChannels:
               reqList = []
               if not NCDStation:
                  maxExpTime = cdqualParams['endTime'] - \
                               cdqualParams['startTime']
                  reqList.append((cdqualParams['startTime'],
                                  cdqualParams['endTime'], 0, NO_GAP_T))
               else:
                  debug(2, cdqualParams, 'is AUX station')
                  # Compute maximal expected time by checking the request table
                  # if a request is pending. If no request is pending
                  # take 0 as start and 0 as end time!
                  maxExpTime = getMaxExpTime(cdqualParams, siteChan, reqList)

               # UGLY: If different requests have been made for the channels
               # of an auxiliary station, report longest request in station
               # statistic/status reports
               if maxExpTime > maxExpTimeMax:
                  maxExpTimeMax = maxExpTime

               # Retrieve statistics
               # See header comment of getChannelStatistics()
               # No optimization is done so far if more than one report is
               # being requested
               cStat = getChannelStatistics(siteChan, cdqualParams,
                                            reqList, frames, NCDStation)
               cStat[NET] = arrayName
               debugChannel(3, cdqualParams, cStat)

               cStat[REQ_LIST] = reqList
               cStat[MAX_EXP_TIME] = maxExpTime

               # If the current channel belongs to an auxiliary station,
               # the gap list is modified to remove all gaps which are
               # not in the requested time interval
               if cStat[REQ_LIST][0][G_TYP] == REQUEST_GAP_T:
                  modifyGapList(cdqualParams, cStat)

               # Compute and check number of expected samples using real
               # sample rate
               samplesExpected = round(maxExpTime * cStat[SAMP_RATE])
               debug(3, cdqualParams, ('samplesExpected:%d, %d %d' % (samplesExpected,
                     cStat[NBR_SAMPLES],cStat[NBR_SAMPLES_QUAL])))
               # Null is OK for non-continuous data with no request
               if samplesExpected < 0:
                  # Stop application with an error
                  sys.exit("Error: %-5s %-4s *** invalid sample rate ***" % \
                           (cStat[SITE], cStat[CHANNEL]))

               ## cStat[NBR_SAMPLES] = cStat[NBR_SAMPLES_QUAL]
               # -------------------------------------------------------------
               if report == 'CHAN_STATUS':
                  p_Received = getPercent(samplesExpected, cStat[NBR_SAMPLES])
                  # Compute percentage available as samples received minus
                  # bad samples
                  p_Avail = getPercent(samplesExpected,
                                       cStat[NBR_SAMPLES] - \
                                       cStat[NBR_BAD_SAMPLES])
                  # Print a channel row now
                  rowCont = (cStat[NET], cStat[SITE], cStat[CHANNEL],
                             p_Received, p_Avail, cStat[NBR_GAPS],
                             cStat[NBR_SAMPLES], cStat[NBR_BAD_SAMPLES],
                             cStat[MEAN], cStat[RMS])

                  if cdqualParams['outFormat'] == 'dsv':
                     cfd.writerow(rowCont)
                  elif cdqualParams['outFormat'] == 'xml':
                     XMLrecord = doc.createElementNS(nS[0], nS[1] + 'record')
                     XMLrecords.appendChild(XMLrecord)
                     addChannelRecordXML(doc, XMLrecord, rowCont, nS)
                  elif cdqualParams['outFormat'] == 'json':
                     print jrs
                     print rowFormat % rowCont
                     # Close the previous record
                     jrs = '  },\n  {'
                  else:
                     print rowFormat % rowCont

               # -------------------------------------------------------------
               elif report == 'CHAN_STATUS:2' or report == 'CHAN_STATUS:3':
                  debug(2, cdqualParams, (samplesExpected, cStat[NBR_SAMPLES],
                                          cStat[NBR_BAD_SAMPLES],
                                          cStat[NBR_LATE_SAMPLES],
                                          cStat[NBR_UNTRUSTED_SAMPLES],
                                          cStat[NBR_LATE_SAMPLES_WO_BAD],
                                          cStat[NBR_UNTRUSTED_SAMPLES_WO_BAD],
                                     cStat[NBR_UNTRUSTED_LATE_SAMPLES_WO_BAD]))

                  p_Received = getPercent(samplesExpected, cStat[NBR_SAMPLES])
                  # Compute percentage available as samples received minus
                  # bad samples
                  p_Avail = getPercent(samplesExpected,
                                       cStat[NBR_SAMPLES] - \
                                       cStat[NBR_BAD_SAMPLES])
                  # Compute percentage available with surety as
                  # samples received minus bad samples
                  # and minus untrusted, but not bad data
                  p_AvSur = getPercent(samplesExpected,
                                       cStat[NBR_SAMPLES] - \
                                       cStat[NBR_UNTRUSTED_SAMPLES_WO_BAD] - \
                                       cStat[NBR_BAD_SAMPLES])
                  # Compute percentage available late as
                  # samples received minus bad samples
                  # and minus late, but not bad data
                  p_AvLat = getPercent(samplesExpected,
                                       cStat[NBR_SAMPLES] - \
                                       cStat[NBR_LATE_SAMPLES_WO_BAD] - \
                                       cStat[NBR_BAD_SAMPLES])
                  # Print a channel row now
                  rowCont = (cStat[NET], cStat[SITE], cStat[CHANNEL],
                             p_Received, p_Avail, p_AvSur, cStat[NBR_GAPS],
                             cStat[NBR_SAMPLES], cStat[NBR_BAD_SAMPLES],
                             cStat[MEAN], cStat[RMS])
   #              rowContDebug = (cStat[NET], cStat[SITE], cStat[CHANNEL],
   #                              p_Received, p_Avail, p_AvSur, p_AvLat,
   #                              cStat[NBR_GAPS],
   #                              cStat[NBR_SAMPLES], cStat[NBR_BAD_SAMPLES],
   #                              cStat[MEAN], cStat[RMS])
   #              rowFormatDebug = "%-9s %-5s %-4s %7.3f %7.3f %7.3f %7.3f %5d\
   # %9d% 10d% 12.1f %11.1f"
   #                print rowFormatDebug % rowContDebug

                  if cdqualParams['outFormat'] == 'dsv':
                     cfd.writerow(rowCont)
                  elif cdqualParams['outFormat'] == 'xml':
                     XMLrecord = doc.createElementNS(nS[0], nS[1] + 'record')
                     XMLrecords.appendChild(XMLrecord)
                     if report == 'CHAN_STATUS:2':
                        addChannelRecord2XML(doc, XMLrecord, rowCont, nS)
                     else:
                        addChannelRecord3XML(doc, XMLrecord, rowCont, nS)
                  elif cdqualParams['outFormat'] == 'json':
                     print jrs
                     print rowFormat % rowCont
                     # Close the record
                     jrs = '  },\n  {'
                  else:
                     print rowFormat % rowCont

               # -------------------------------------------------------------
               elif report == 'CHAN_STATISTICS':
                  cGaps = lGaps = clGaps = 0
                  # Count virtual gaps by type
                  gaps = cStat[GAP_LIST]
                  for gap in gaps:
                     agt = gap[G_TYP] & (~AUTH_FAIL_GAP_T)
                     # skip virtual gaps
                     if agt == NO_GAP_T:
                        continue
                     elif agt == BAD_DATA_GAP_T:
                        cGaps += 1
                     elif agt == LATE_GAP_T:
                        lGaps += 1
                     elif agt == (LATE_GAP_T | BAD_DATA_GAP_T):
                        clGaps += 1

                  # Print a channel row now
                  eSamples = round(cStat[MAX_EXP_TIME] * cStat[SAMP_RATE])
                  rowCont = (cStat[NET], cStat[SITE], cStat[CHANNEL],
                             cStat[WFDISCS], cStat[QUAL_RECS], cStat[NBR_GAPS],
                             lGaps, cGaps, clGaps,
                             getPercent(eSamples, cStat[NBR_SAMPLES]))

                  if cdqualParams['outFormat'] == 'dsv':
                     cfd.writerow(rowCont)
                  elif cdqualParams['outFormat'] == 'xml':
                     XMLrecord = doc.createElementNS(nS[0], nS[1] + 'record')
                     XMLrecords.appendChild(XMLrecord)
                     addChannelStatisticsRecordXML(doc, XMLrecord, rowCont, nS)
                  elif cdqualParams['outFormat'] == 'json':
                     print jrs
                     print rowFormat % rowCont
                     # Close the record
                     jrs = '  },\n  {'
                  else:
                     print rowFormat % rowCont

               # -------------------------------------------------------------
               elif report == 'STA_STATISTICS' or report == 'STA_STATUS:3'\
                    or report == 'STA_STATUS:2'\
                    or report == 'STA_STATISTICS:2' or report == 'STA_STATUS':
                  # Collect channel statistics for next step
                  cStatArray.append(cStat)
                  channelCnt += 1

                  # State of Health Channels
                  # [MBL]DA is SOH for historical reasons and
                  # is defined so in the Operating Manual
                  SOH = r'LE[AVIMP]|[MBL]DA|LG[SD]|LKI|LIO|LRH'
                  # Meteorological Channels
                  MET = r'|[MBL]KO|[MBL]W[DS]|l[nez]'

                  channels2skip = [ r'^no_channels_to_skip$', # all data
                                r'^(' + SOH + r')$', # skip SOH channels,
                                                     # leaves Geophysical Data
                         r'^(' + SOH + MET + r')$' ] # skip SOH + MET,
                                                     # leaves Eligible Data

                  if 'H' == station[0] and station[1].isdigit():
                     # Hydro (T-Phase or Hydrophone) has different eligible
                     # channel set (only the verticals are used)
                     channels2skip[2] = r'^(' + SOH + MET + r'|[EH]H[EN])$'

                  # Sum up data values according to the rules for
                  # the current channel for all, geophysical and eligible
                  # data.
                  c2di = 0
                  for ii in range(0, 15, 7):
                     # Check if current channel is used in station values
                     if re.match( channels2skip[c2di], cStat[CHANNEL], 0):
                        break

                     c2di += 1
                     # NOTE: this is the deprecated way to calculate station
                     #       statistic values. This way has been replaced in
                     #       favor of calculation of station values based on
                     #       time. The values computed below are not longer
                     #       used starting with version 1.31
                     stationStats[P_RECVD_ALL + ii] += cStat[NBR_SAMPLES]
                     stationStats[P_BAD_SAMPLES_ALL + ii] += \
                                 cStat[NBR_BAD_SAMPLES]
                     stationStats[P_TOO_LATE_ALL + ii] += \
                                  cStat[NBR_LATE_SAMPLES]
                     stationStats[P_TOO_LATE_WO_BAD_ALL + ii] += \
                                  cStat[NBR_LATE_SAMPLES_WO_BAD]
                     stationStats[P_UNTRUSTED_WO_BAD_ALL + ii] += \
                                  cStat[NBR_UNTRUSTED_SAMPLES_WO_BAD]
                     stationStats[P_UNTRUSTED_LATE_WO_BAD_ALL + ii] += \
                                 cStat[NBR_UNTRUSTED_LATE_SAMPLES_WO_BAD]
                     stationStats[P_EXPECTED_ALL + ii] += \
                                  round(cStat[MAX_EXP_TIME] * cStat[SAMP_RATE])

                  c2di = 0
                  for ii in range(0, 17, 8):
                     # Check if current channel is used in station values
                     if re.match( channels2skip[c2di], cStat[CHANNEL], 0):
                        break

                     c2di += 1
                     # NOTE: this is the proposed way to calculate station
                     #       statistic values. This way has been introduced in
                     #       favor of calculation of station values based on
                     #       samples. The values computed below are used
                     #       starting with version 1.31
                     gapTime = getGapTimes(cdqualParams, cStat[GAP_LIST])
                     for ij in range(6):
                        # NO_DATA_TIME, BAD_DATA_TIME, INSECURE_OR_BAD_TIME,
                        # LATE_DATA_TIME, BAD_OR_LATE_TIME,
                        # INSECURE_LATE_BAD_TIME
                        stationStatsT[T_RECVD_ALL + ii + ij] += gapTime[ij]
                        debug(4, cdqualParams, ('gap times 1: ',
                                 c2di, cStat[SITE], cStat[CHANNEL],
                                 ii, ij,
                                 stationStatsT[T_RECVD_ALL + ii + ij],
                                 maxExpTime,
                                 maxExpTime - gapTime[ij], gapTime[ij]))

                     stationStatsT[T_RECVD_ALL + ii + U__] += gapTime[8];
                     # Sum up maximum expected times
                     stationStatsT[T_RECVD_ALL + ii + CC_] += maxExpTime

               # -------------------------------------------------------------
               elif report == 'AUTH_STATUS':

                  rowCont = []
                  # Authentication status report is something between a
                  # channel and a station report. Instead of stations, it
                  # reports for sites, which means channels values have to
                  # combined into sites.
                  if len(siteValues) == 0:
                     # start a new site
                     siteValues = [ cStat[NET], cStat[SITE], 0, 0, 0, 0, 0, 0 ]

                  if siteValues[1] != cStat[SITE]:
                     # Site has changed. Print site values now
                     findSite(cdqualParams, frames, siteValues, cStat[REQ_LIST])
                     rowCont = tuple(siteValues)
                     siteValues = [ cStat[NET], cStat[SITE], 0, 0, 0, 0, 0, 0 ]

                     if cdqualParams['outFormat'] == 'dsv':
                        cfd.writerow(rowCont)
                     elif cdqualParams['outFormat'] == 'xml':
                        XMLrecord = doc.createElementNS(nS[0], nS[1] + 'record')
                        XMLrecords.appendChild(XMLrecord)
                        addAuthRecords(doc, XMLrecord, rowCont, nS, cStatArray)
                     elif cdqualParams['outFormat'] == 'json':
                        print jrs
                        print rowFormat % rowCont
                        printAuthIntervalJSON(cdqualParams, cStatArray)
                        cStatArray = []
                        # Close the previous record
                        jrs = '  },\n  {'
                     else:
                        debug(3, cdqualParams, rowFormat);
                        debug(3, cdqualParams, rowCont);
   # rowFormat= '%-10s%-6s%10lu %10lu %10lu %10lu %10lu %10lu'
                        print rowFormat % rowCont

                  # Sum up site values
                  siteValues[5] += cStat[CSF_RCVD]
                  siteValues[6] += cStat[CSF_CHECKED]
                  siteValues[7] += cStat[CSF_FAILED]
                  # Collect channel statistics for appended authentication
                  # intervals
                  cStatArray.append(cStat)

               # -------------------------------------------------------------
               else:
                  # OUTAGE / AUTH_INTERVAL: Set up row format
                  if report == 'OUTAGE':
                     gaps = cStat[GAP_LIST]
                     rowForOut = "%-9s %-5s %-4s %s" % (cStat[NET],
                                                        cStat[SITE],
                                                        cStat[CHANNEL],
                                           "%-20s%03d %-20s%03d %10.3f %s")
                  else:
                     gaps = cStat[AUTH_GAP_LIST]
                     rowForOut = "%-9s %-5s %-4s %s" % (cStat[NET],
                                                        cStat[SITE],
                                                        cStat[CHANNEL],
                                              "%-20s%03d %-20s%03d %s")

                  # print a line for each gap
                  for gap in gaps:
                     if not cdqualParams['verbose'] and report == 'OUTAGE':
                        # skip virtual gaps
                        if NO_DATA_GAP_T != gap[G_TYP]:
                           continue

                     if report == 'OUTAGE':
                        actLine = rowForOut % getGapLine(gap)
                     else:
                        actLine = rowForOut % getAuthGapLine(gap)
                     lst = actLine.split(None, 8)

                     if cdqualParams['outFormat'] == 'dsv':
                        # Concatenate date and time
                        lst[3] = lst[3] + ' ' + lst[4]
                        lst[5] = lst[5] + ' ' + lst[6]
                        lst.remove(lst[6])
                        lst.remove(lst[4])
                        cfd.writerow(lst)
                     elif cdqualParams['outFormat'] == 'xml':
                        XMLrecord = doc.createElementNS(nS[0], nS[1] + 'record')
                        XMLrecords.appendChild(XMLrecord)
                        if report == 'OUTAGE':
                           addOutageRecordXML(doc, XMLrecord,
                                              actLine.split(), nS)
                        else:
                           addAuthRecordXML(doc, XMLrecord,
                                            actLine.split(), nS)
                     elif cdqualParams['outFormat'] == 'json':
                        print jrs
                        if len(lst) == 8:
                           lst.append('')
                        debug(3, cdqualParams, lst);
                        print rowFormat % tuple(lst)
                        # Close the record
                        jrs = '  },\n  {'
                     else:
                        print actLine

            # End of channel processing -------------------------------------

            # Authentication status must print remaining site values
            if report == 'AUTH_STATUS':
               # Site has changed. Print site values now
               findSite(cdqualParams, frames, siteValues, cStat[REQ_LIST])
               rowCont = tuple(siteValues)
               siteValues = []

               if cdqualParams['outFormat'] == 'dsv':
                  cfd.writerow(rowCont)
               elif cdqualParams['outFormat'] == 'xml':
                  XMLrecord = doc.createElementNS(nS[0], nS[1] + 'record')
                  XMLrecords.appendChild(XMLrecord)
                  addAuthRecords(doc, XMLrecord, rowCont, nS, cStatArray)
               elif cdqualParams['outFormat'] == 'json':
                  print jrs
                  print rowFormat % rowCont
                  printAuthIntervalJSON(cdqualParams, cStatArray)
                  # Close the previous record
                  jrs = '  }'
               else:
                  print rowFormat % rowCont

               # End site values ------------------------------------

            # Station report needs one more pass to compute mission capability
            elif report == 'STA_STATUS' or report == 'STA_STATISTICS' \
               or report == 'STA_STATUS:3' or report == 'STA_STATUS:2' \
               or report == 'STA_STATISTICS:2':
               # Check for no channels left
               if len(cStatArray) == 0:
                  print >> sys.stderr, \
                           "Warning! No channels found for station " + station
                  continue


               if not maxExpTimeMax:
                  # If the station is a non-continues data source and no
                  # request has been made it has 0% mission capability
                  # as requested by PJ (see ISI-119)
                  stationStats[P_MISSION] = 0
                  minValue = [ 0, 0, 0, 0, 0, 0, 0, 0 ]
               else:

                  ## if NCDStation:
                     # If no minimum rules should be applied for auxiliary
                     # stations, copy eligible values to minimum
                     # minValue = [ stationStatsT[T_RECVD_ELI],
                     ##              stationStatsT[T_RECVD_ELI + A__],
                     ##              stationStatsT[T_RECVD_ELI + AS_],
                     ##              stationStatsT[T_RECVD_ELI + T__],
                     ##              stationStatsT[T_RECVD_ELI + AT_],
                     ##              stationStatsT[T_RECVD_ELI + ATS],
                     ##              stationStatsT[T_RECVD_ELI + AT_],
                     ##              stationStatsT[T_RECVD_ELI + ATS]]
                  ## elif 'I' == station[0] and \

                  # Remove non requested channels from channel list
                  # for minimum value calculation.
                  # Only applicable for auxiliary stations.
                  # Another approach would be to skip non requested
                  # channels when creating the list of channels, but
                  # then non requested channels would not be listed
                  # in the channel status reports, too, which makes
                  # debugging harder
                  if NCDStation:
                     channels2remove = []
                     for cStat in cStatArray:
                        if cStat[MAX_EXP_TIME] == 0:
                           # Should happen only when auxiliary station
                           debug(2, cdqualParams, ('drop unrequested channel ',
                                    cStat[SITE], cStat[CHANNEL]))
                           channels2remove.append(cStat)
                     # Remove non requested channels from station
                     for channel in channels2remove[:]:
                         cStatArray.remove(channel)
                     channels2remove = []

                  # Treat non requested channels as missing data for
                  # minimum calculation.
                  # Only applicable for auxiliary stations.
                  # Another approach would be to create data gaps for
                  # non requested channels when creating the list of
                  # channels, but then non requested channels would
                  # decrease all other channel set values, too.
                  ## if NCDStation:
                  ##    for cStat in cStatArray:
                  ##       if cStat[MAX_EXP_TIME] == 0:
                  ##          # Should happen only when auxiliary station
                  ##          debug(2, cdqualParams,
                  ##                   ('drop unrequested channel ',
                  ##                   cStat[SITE], cStat[CHANNEL]))
                  ##          cStat[GAP_LIST].append((cdqualParams['startTime'],
                  ##                  cdqualParams['endTime'], 0,
                  ##                  NO_DATA_GAP_T))

                  # Mission capable is computed as percentage between
                  # maximum expected time and combined gap time.
                  # Keep in mind that this value might become higher than the
                  # received, available and timely time because depending on
                  # the station technology receiving at least one channel
                  # (alternating) might be sufficient. Seismic, Infrasound
                  # and Hydroacoustic stations are supported.
                  # The first two characters
                  # of the station name is used to determine station technology
                  # Don't use the channel name to distinguish between station
                  # technologies, because seismic and T-Phase stations could
                  # use the same channel names
                  if 'I' == station[0] and \
                        (station[1].isdigit() \
                         or ('S' == station[1] and station[2].isdigit())):
                     # Infrasound
                     debug(2, cdqualParams,
                           'Station ' + station + ' is Infrasound')
                     minValue = getInfrasoundStatistics(cdqualParams,
                                                        cStatArray,
                                                        stationStats, station)

                  elif 'H' == station[0] and station[1].isdigit():
                     # Hydro (T-Phase or Hydrophone)
                     debug(2, cdqualParams, 'Station ' + station + ' is Hydro')
                     minValue = getHydroStatistics(cdqualParams, cStatArray,
                                                   stationStats)
                  else:
                     # Seismic
                     debug(2, cdqualParams, 'Station ' + station + \
                           ' is Seismic')
                     minValue = getSeismicStatistics(cdqualParams, cStatArray,
                                                     stationStats, station)

                  # Check for minimal channels left (ISI-270)
                  if len(minValue) == 0:
                     print >> sys.stderr, \
                     "Warning! No eligible channels left, station " + station
                     for ii in range(8):
                        minValue.append(maxExpTimeMax)

                  stationStats[P_MISSION] = minValue[6]

                  # Compute minimum station statistics percentage as
                  # ration of expected time and (virtual) gap time
                  # and NOT based on received samples to avoid problems
                  # with different sample rates. Note: there is no way
                  # to calculate correct mission values, if different time
                  # periods are requested for auxiliary station channels
                  # of the same station
                  for ii in range(8):
                     minValue[ii] = getPercent(maxExpTimeMax,
                                               maxExpTimeMax - minValue[ii])
                  # Compute station values as
                  # ration of expected time (of all channels
                  # contributing (tmpMaxTime)) and (virtual) gap time
                  # and NOT based on received samples to avoid problems
                  # with different sample rates
                  for ii in range(0, 17, 8):
                     # Compute the other values as percentage of time data
                     # received with category in maximum time expected
                     # for all channels (tmpMaxTime); CC_ contains
                     # maximum expected time for this channel
                     tmpMaxTime = stationStatsT[T_RECVD_ALL + ii + CC_]
                     # Compute received authenticated as percentage of time
                     # data received authenticated in time data received
                     # --------------  Revert change as requested by PJ
                     # Compute received authenticated as percentage of time
                     # data received authenticated in time data expected;
                     # see ISI-154
                     # tmpRcvTime = tmpMaxTime - stationStatsT[T_RECVD_ALL + ii]
                     # ik = stationStatsT[T_RECVD_ALL + ii + U__]
                     tmpRcvTime = tmpMaxTime
                     ik = stationStatsT[T_RECVD_ALL + ii + U__] + \
                                       stationStatsT[T_RECVD_ALL + ii + R__]
                     # --------------  End change as requested by PJ
                     ik = getPercent(tmpRcvTime, tmpRcvTime - ik)
                     stationStatsT[T_RECVD_ALL + ii + U__] = ik

                     for ij in range(6):
                        ik = stationStatsT[T_RECVD_ALL + ii + ij]
                        ik = getPercent(tmpMaxTime, tmpMaxTime - ik)
                        stationStatsT[T_RECVD_ALL + ii + ij] = ik

               # Compute percentage values for station status
               # Samples has been summed up before for
               # channel values (except mission capability)
               samples = stationStats[P_RECVD]
               badSamples =  stationStats[P_BAD_SAMPLES]
               stationStats[P_RECVD] = \
                   getPercent(stationStats[P_EXPECTED], samples)
               # Data received authenticated are samples received minus
               # untrusted data
               stationStats[P_UNTRUSTED] = \
                   getPercent(samples, samples - stationStats[P_UNTRUSTED])
               # Samples available are samples received minus constant data
               stationStats[P_BAD_SAMPLES] = \
                    getPercent(stationStats[P_EXPECTED], samples - badSamples)
               # Samples timely: samples received minus constant data minus
               # too late data. Late samples do not include constant data
               # samples because they are excluded in getChannelStatistics()
               # ----RRRRRRCCCCCCRRRRRRRRRRRRRCCCCC---- received/constant
               # --------LLNNNNNNLLLL------------------ late/late not counted
               # ----AAAA------------AAAAAAAAA--------- available data
               stationStats[P_TOO_LATE] = getPercent(stationStats[P_EXPECTED],
                                           samples - badSamples - \
                                           stationStats[P_TOO_LATE])
               # Compute minimum station statistics percentage as
               # ration of expected time and (virtual) gap time
               # Note: there is no way
               # to calculate correct mission values, if different time
               # periods are requested for auxiliary station channels
               # of the same station
               # stationStats[P_MISSION] = getPercent(maxExpTimeMax,
               #                        maxExpTimeMax - stationStats[P_MISSION])

               if report == 'STA_STATISTICS' or report == 'STA_STATISTICS:2' \
                  or report == 'STA_STATUS:2' or report == 'STA_STATUS:3':
                  # Compute percentage values for station statistics and
                  # station status V2. Samples has been summed up before
                  # (except mission capability)
                  for ii in range(0, 15, 7):
                     samples = stationStats[P_RECVD_ALL + ii]
                     badSamples =  stationStats[P_BAD_SAMPLES_ALL + ii]
                     lateSamples = stationStats[P_TOO_LATE_ALL + ii]
                     lateNotBadSamples = \
                         stationStats[P_TOO_LATE_WO_BAD_ALL + ii]
                     untrustedNotBad = \
                         stationStats[P_UNTRUSTED_WO_BAD_ALL + ii]
                     untrustedLate = \
                         stationStats[P_UNTRUSTED_LATE_WO_BAD_ALL + ii]
                     expectedSamples = stationStats[P_EXPECTED_ALL + ii]

                     stationStats[V_RECVD_ALL + ii] = \
                           getPercent(expectedSamples, samples)
                     # Samples available are samples received minus bad data
                     stationStats[V_AVAILABLE_ALL + ii] = \
                           getPercent(expectedSamples, samples - badSamples)
                     # Samples available with surety are samples received
                     # minus bad and minus untrusted, but not bad data
                     stationStats[V_AVAIL_SURE_ALL + ii] = \
                           getPercent(expectedSamples, samples - badSamples - \
                                      untrustedNotBad)
                     # Samples received timely are samples received minus
                     # too late data
                     stationStats[V_TIMELY_ALL + ii] = \
                           getPercent(expectedSamples, samples - lateSamples)
                     # Timely available are samples received minus bad data
                     # minus too late, but not bad data
                     # Late samples do not include constant data samples
                     # because they are excluded in getChannelStatistics()
                     # --RRRRRRCCCCCCRRRRRRRRRRRRRCCCCC-- received/constant
                     # ------LLNNNNNNLLLL---------------- late/late not counted
                     # --AAAA------------AAAAAAAAA------- available data
                     stationStats[V_TIMELY_AVAIL_ALL + ii] = \
                           getPercent(expectedSamples, samples - badSamples - \
                                      lateNotBadSamples)
                     # Timely available with surety are samples received
                     # minus bad data minus too late and, but not bad data
                     stationStats[V_TIMELY_AVAIL_SURE_ALL + ii] = \
                           getPercent(expectedSamples, samples - badSamples - \
                                      lateNotBadSamples - untrustedNotBad +
                                      untrustedLate)
               # End loop ============================================

               if not cdqualParams['outFormat'] == 'json':
                  # Split maxExpTimeMax in days, hours, minutes and seconds for
                  # IMS and XML output
                  days = int(maxExpTimeMax / (60*60*24))
                  rest = maxExpTimeMax - (days * 60*60*24)
                  hours = int(rest / (60*60))
                  rest -= hours * 60*60
                  minutes = int(rest/60)
                  rest -= minutes * 60

                  if report == 'STA_STATISTICS' \
                     or report == 'STA_STATISTICS:2':
                     # Station statistics V1.0 and V2.0. Differ only in header
                     rowCont = (station, days, hours, minutes, rest,
                                # Minimum channels ------------------
                                minValue[0], minValue[1], minValue[2],
                                minValue[3], minValue[4], minValue[5],
                                minValue[6], minValue[7],
   #                            stationStats[P_RECVD], 0.0, 0.0,
   #                            stationStats[P_TOO_LATE],
   #                            stationStats[P_BAD_SAMPLES], 0.0,
   #                            stationStats[P_MISSION],
   #                            0.0,
                                # Eligible channels ------------------
                                stationStatsT[T_RECVD_ELI],
                                stationStatsT[T_RECVD_ELI + A__],
                                stationStatsT[T_RECVD_ELI + AS_],
                                stationStatsT[T_RECVD_ELI + T__],
                                stationStatsT[T_RECVD_ELI + AT_],
                                stationStatsT[T_RECVD_ELI + ATS],
                                # Geophysical channels ------------------
                                stationStatsT[T_RECVD_GEO],
                                stationStatsT[T_RECVD_GEO + A__],
                                stationStatsT[T_RECVD_GEO + AS_],
                                stationStatsT[T_RECVD_GEO + T__],
                                stationStatsT[T_RECVD_GEO + AT_],
                                stationStatsT[T_RECVD_GEO + ATS],
                                # All channels ------------------
                                stationStatsT[T_RECVD_ALL],
                                stationStatsT[T_RECVD_ALL + A__],
                                stationStatsT[T_RECVD_ALL + AS_],
                                stationStatsT[T_RECVD_ALL + T__],
                                stationStatsT[T_RECVD_ALL + AT_],
                                stationStatsT[T_RECVD_ALL + ATS])
                     debug(3, cdqualParams, rowCont)
                     debug(3, cdqualParams, rowFormat)
                  elif report == 'STA_STATUS:2' or report == 'STA_STATUS:3':
                     # Station status V2.0 and V3.0. Differ only in header
                     rowCont = (station, days, hours, minutes, rest,
                                # Minimum channels ------------------
                                minValue[2], minValue[5], minValue[7],
                                # Geophysical channels ------------------
                                stationStatsT[T_RECVD_GEO],
                                stationStatsT[T_RECVD_GEO + A__],
                                stationStatsT[T_RECVD_GEO + AS_])
                  else:
                     # Station status. If minimum channel values should
                     # be used minValues instead of stationStatsT here
                     rowCont = (station, days, hours, minutes, rest,
                                stationStatsT[T_RECVD_ELI],
                                stationStatsT[T_RECVD_ELI + A__],
                                stationStatsT[T_RECVD_ELI + AT_],
                                minValue[6],
                                stationStatsT[T_RECVD_ELI + U__])

                  actLine = rowFormat % rowCont

               if cdqualParams['outFormat'] == 'dsv':
                  lst = actLine.split()
                  # Replace days, hours, minutes and seconds with maxExpTimeMax
                  lst[1] = maxExpTimeMax
                  lst.remove(lst[2])
                  cfd.writerow(lst)
               elif cdqualParams['outFormat'] == 'xml':
                  XMLrecord = doc.createElementNS(nS[0], nS[1] + 'record')
                  XMLrecords.appendChild(XMLrecord)
                  if report == 'STA_STATISTICS':
                     addStatisticRecordXML(doc, XMLrecord, rowCont, nS)
                  elif report == 'STA_STATISTICS:2':
                     addStatisticRecord2XML(doc, XMLrecord, rowCont, nS)
                  elif report == 'STA_STATUS:2':
                     addStationRecord2XML(doc, XMLrecord, rowCont, nS)
                  elif report == 'STA_STATUS:3':
                     addStationRecord3XML(doc, XMLrecord, rowCont, nS)
                  else:
                     addStationRecordXML(doc, XMLrecord, rowCont, nS)
               elif cdqualParams['outFormat'] == 'json':
                  print jrs
                  if report == 'STA_STATISTICS' \
                     or report == 'STA_STATISTICS:2':
                     rowCont = (station, maxExpTimeMax,
                                # Minimum channels ------------------
                                minValue[0], minValue[1], minValue[2],
                                minValue[3], minValue[4], minValue[5],
                                minValue[6], minValue[7],
                                # Eligible channels ------------------
                                stationStatsT[T_RECVD_ELI],
                                stationStatsT[T_RECVD_ELI + A__],
                                stationStatsT[T_RECVD_ELI + AS_],
                                stationStatsT[T_RECVD_ELI + T__],
                                stationStatsT[T_RECVD_ELI + AT_],
                                stationStatsT[T_RECVD_ELI + ATS],
                                # Geophysical channels ------------------
                                stationStatsT[T_RECVD_GEO],
                                stationStatsT[T_RECVD_GEO + A__],
                                stationStatsT[T_RECVD_GEO + AS_],
                                stationStatsT[T_RECVD_GEO + T__],
                                stationStatsT[T_RECVD_GEO + AT_],
                                stationStatsT[T_RECVD_GEO + ATS],
                                # All channels ------------------
                                stationStatsT[T_RECVD_ALL],
                                stationStatsT[T_RECVD_ALL + A__],
                                stationStatsT[T_RECVD_ALL + AS_],
                                stationStatsT[T_RECVD_ALL + T__],
                                stationStatsT[T_RECVD_ALL + AT_],
                                stationStatsT[T_RECVD_ALL + ATS])
                  elif report == 'STA_STATUS:2' or  report == 'STA_STATUS:3':
                     # Station status V2.0 and V3.0. Differ only in header
                     rowCont = (station, maxExpTimeMax,
                                # Minimum channels ------------------
                                minValue[2], minValue[5], minValue[7],
                                # Geophysical channels ------------------
                                stationStatsT[T_RECVD_GEO],
                                stationStatsT[T_RECVD_GEO + A__],
                                stationStatsT[T_RECVD_GEO + AS_])
                  else:
                     # Station status. If minimum channel values should
                     # be used minValues instead of stationStatsT here
                     rowCont = (station, maxExpTimeMax,
                                stationStatsT[T_RECVD_ELI],
                                stationStatsT[T_RECVD_ELI + A__],
                                stationStatsT[T_RECVD_ELI + T__],
                                minValue[6],
                                stationStatsT[T_RECVD_ELI + U__])
                  debug(3, cdqualParams, (rowFormat, rowCont))
                  print rowFormat % rowCont
                  # Close the record
                  jrs = '  },\n  {'
               else:
                  print actLine

         # END for station in stations:
      # END for report in splitReport():

      if cdqualParams['fileMode']:
         # Close files in file mode
         if cdqualParams['outFormat'] == 'xml':
            # xml.dom.ext.PrettyPrint(doc, sys.stdout, encoding="utf-8")
            uglyXml = doc.toprettyxml(indent="  ", encoding="utf-8")
            text_re = re.compile('>\n\s+([^<>\s].*?)\n\s+</', re.DOTALL)
            prettyXml = text_re.sub('>\g<1></', uglyXml)
            print prettyXml
            # doc.writexml(sys.stdout, '   ', '   ', '\n', 'utf-8')
         elif cdqualParams['outFormat'] == 'json':
            print "  }\n ]\n}"
         fd.close()
         sys.stdout = saveStdout
   # END for arg in cdqualParams['args']:

   if not cdqualParams['fileMode']:
      if cdqualParams['outFormat'] == 'xml':
         # xml.dom.ext.PrettyPrint(doc, sys.stdout, encoding="utf-8")
         uglyXml = doc.toprettyxml(indent="  ", encoding="utf-8")
         text_re = re.compile('>\n\s+([^<>\s].*?)\n\s+</', re.DOTALL)
         prettyXml = text_re.sub('>\g<1></', uglyXml)
         print prettyXml
         # doc.writexml(sys.stdout, '   ', '   ', '\n', 'utf-8')
      elif cdqualParams['outFormat'] == 'json':
         # Close JSON report
         print "  }\n ]\n}"
# -----------------------------------------------------------------------------

R__ = 0  # No data gap time / percentage received (based on time)
A__ = 1  # R__ + bad data gap time / percentage available (based on time)
AS_ = 2  # AS_ + no surety gap time / available with surety (based on time)
T__ = 3  # R__ + late data gap time / received timely (based on time)
AT_ = 4  # T__ + bad data gap time / available timely (based on time)
ATS = 5  # AT_ + no surety gap time / available timely with surety (based ...)
U__ = 6  # R__ + unauthenticated data gap time / received unauthenticated (... )
CC_ = 7  # Maximum expected time for channel all channels

T_RECVD_ALL = 8   # Percentage received all channel set (based on time)
T_RECVD_GEO = 16  # Percentage received geophysical channel set (based on time)
T_RECVD_ELI = 24  # Percentage received eligible channel set (based on time)

# Minimum channel set ------------------------------------
P_RECVD = 0       # Number of samples received
P_BAD_SAMPLES = 1 # Number of bad samples received
P_TOO_LATE = 2    # Number of late samples received not counting bad samples
P_MISSION = 3     # Number of samples received for mission capability
P_UNTRUSTED = 4   # Number of untrusted samples received
P_EXPECTED = 5    # Number of samples expected
P_TIME_EXPECTED = 6    # time period expected
# All channel set ------------------------------------
P_RECVD_ALL = 7   # Number of samples received
V_RECVD_ALL = 7   # Percentage received (based on samples)
P_BAD_SAMPLES_ALL = 8 # Number of bad samples received
V_AVAILABLE_ALL = 8   # Percentage available (based on samples)
P_TOO_LATE_ALL = 9 # Number late samples received
V_TIMELY_ALL = 9   # Percentage timely received (based on samples)
P_TOO_LATE_WO_BAD_ALL = 10 # late samples received not counting bad samples
V_TIMELY_AVAIL_ALL = 10    # Percentage available timely (based on samples)
P_UNTRUSTED_WO_BAD_ALL = 11   # Number of untrusted samples without bad
V_AVAIL_SURE_ALL = 11    # Percentage available with surety (based on samples)
P_UNTRUSTED_LATE_WO_BAD_ALL = 12   # Number of untrusted and late samples
                                   # without bad ones
V_TIMELY_AVAIL_SURE_ALL = 12   # Percentage timely available with surety
                               # based on samples
P_EXPECTED_ALL = 13   # Number of samples expected
# Geophysical channel set ------------------------------------
P_RECVD_GEO = 14   # Number of samples received
V_RECVD_GEO = 14
P_BAD_SAMPLES_GEO = 15 # Number of bad samples received
P_TOO_LATE_GEO = 16    # Number late samples received
P_TOO_LATE_WO_BAD_GEO = 17 # late samples received not counting bad samples
P_UNTRUSTED_WO_BAD_GEO = 18   # Number of untrusted samples without bad
P_UNTRUSTED_LATE_WO_BAD_GEO = 19   # Number of untrusted and late samples
                                   # without bad ones
P_EXPECTED_GEO = 20   # Number of samples expected
# Eligible channel set ------------------------------------
P_RECVD_ELI = 21   # Number of samples received
V_RECVD_ELI = 21
P_BAD_SAMPLES_ELI = 22 # Number of bad samples received
P_TOO_LATE_ELI = 23    # Number late samples received
P_TOO_LATE_WO_BAD_ELI = 24 # late samples received not counting bad samples
P_UNTRUSTED_WO_BAD_ELI = 25   # Number of untrusted samples without bad
P_UNTRUSTED_LATE_WO_BAD_ELI = 26   # Number of untrusted and late samples
                                   # without bad ones
P_EXPECTED_ELI = 27   # Number of samples expected

# -----------------------------------------------------------------------------
# @short Get Infrasound Statistics
#
# This function computes the statistics for a given
# infrasound station. It takes a list of site/channel
# combinations and a list of gaps to compute the mission
# capability of the station
#
# @param cdqualParams  dictionary containing name value pairs
# @param cStatArray    list of statistics returned by getChannelStatistics()
# @param stationStats  station statistics list
# @param station       station name
#
# @return time where the minimum required number of channels were not
#         available
# -----------------------------------------------------------------------------

def getInfrasoundStatistics(cdqualParams, cStatArray, stationStats, station):
   # Rules Infrasound: at least 70% (rounded up) of the array elements must
   #                   be operational
   # Known channel names: BD[AF], LDA, LK[IO], [MB]KO, [MBL]W[DS], LE[IM],
   # LG[DS], LIO, LRH, LEP?
   # Processed: BDF
   # Meteorological: MKO,BKO, LKO (Temperature outside),
   #                 MWD, BWD, LWD (Wind direction),
   #                 MWS, BWS, LWS (Wind speed)
   # SOH: LEI (Digitizer voltage), LEM (Met station battery
   #      voltage), LKI (vault temperature), MDA, BDA, LDA (barometric
   #      Pressure), LIO (Humidity), LRH (Rainfall, could also be
   #      Relative humidity), LGS (Gust direction), LGD (Gust speed)
   channelsToConsider = ('BDF')
   channelCnt = 0

   # First pass: sort out all channels not used for mission capability
   filteredStatistics = []
   for cStat in cStatArray:
      if cStat[CHANNEL] in channelsToConsider:
         filteredStatistics.append(cStat)
         channelCnt += 1
         # Sum up % data received, % constant data, late data
         # and data received unauthenticated
         stationStats[P_RECVD] += cStat[NBR_SAMPLES]
         stationStats[P_BAD_SAMPLES] += cStat[NBR_BAD_SAMPLES]
         stationStats[P_TOO_LATE] += cStat[NBR_LATE_SAMPLES_WO_BAD]
         stationStats[P_UNTRUSTED] += cStat[NBR_UNTRUSTED_SAMPLES]
         # Sum up also number of expected samples for percentage computation
         stationStats[P_EXPECTED] += round(cStat[MAX_EXP_TIME] * \
                                           cStat[SAMP_RATE])
         stationStats[P_TIME_EXPECTED] += cStat[MAX_EXP_TIME]
      else:
         debug(2, cdqualParams, ('drop SOH channel ', cStat[CHANNEL]))

   # Second pass: consolidate gaps and add channel count to gaps for
   #              channels of interest, remove all gaps where the
   #              minimum number of required channels is available
   #              and sum up gap time
   #
   # Keep in mind that "bad" data (e.g. too late arrived, constant data
   # and data which is flagged by a particular status bit) have been
   # transformed into "virtual gaps"  before. The same algorithm is applied
   # to these types of gaps as for real gaps

   # To consider backup rules here, consolidate backup group to one
   # gap list, decrease overall channel number by number of backup
   # channels and remove all gaps with G_CNT <= number of backup
   # channels and subtract number backup channels from channel count
   # in remaining gaps

   # Check 70% rule first
   minChannels = ceil(channelCnt * 0.7)

   # No channels left for calculation of minimal values
   if minChannels == 0:
      return []

   # Compute minimum channel values received for BDF channels.
   # The default 70 percent rule is used for the computation.
   ## ---> Uncomment next to lines out for alternative computation of
   ## minValues = [ 0, 0, 0, 0, 0, 0, 0, 0 ]
   ## getMinValues(cdqualParams, filteredStatistics, minValues, minChannels)
   overallGapTime = combineGapsMin(cdqualParams, filteredStatistics,
                                   minChannels, channelCnt)
   # Now overall gap time weighted by gap count is divided by minimum
   # channel count
   for ii in range(8):
      overallGapTime[ii] /= minChannels

   # Compute mission gaps for default 70% rule first
   gapResult = combineGaps3(cdqualParams, filteredStatistics, minChannels,
                            channelCnt)

   # Third pass: check for station specific rule set and do the same
   #             as in pass two for specified rule set. The 70% rule
   #             has also to be applied
   ruleResult = loadMissionRuleSet(station, cdqualParams, filteredStatistics,
                                   'infrasound')
   if ruleResult is not None and len(ruleResult[GAP_LIST]) != 0:
      filteredStatistics = []
      # Create virtual channel to be able to reprocess the gaps
      percent70 = createStatisticRec('_VIR_', '_M_', 1.0)
      percent70[GAP_LIST] = gapResult
      filteredStatistics.append(ruleResult)
      filteredStatistics.append(percent70)
      # All off the (virtual) channels have to be operational. It is safe to
      # use combineGaps2 here.
      gapResult = combineGaps2(cdqualParams, filteredStatistics, ALL_GT)

   # Sum up the overall gap time and compute values for Mission Capability
   overallGapTimeMission = getGapTimes(cdqualParams, gapResult)
   overallGapTime[6] = overallGapTimeMission[6]
   overallGapTime[7] = overallGapTimeMission[7]

   ## ---> Comment next to lines out for alternative computation of
   ## minimum values
   ## for ii in range(0, 6):
   ##   overallGapTime[ii] = minValues[ii]

   debug(2, cdqualParams, ('overall gap time: ', overallGapTime))
   return overallGapTime

# -----------------------------------------------------------------------------
# @short Sum up gap times
#
# This function sums up gap times by gap gap categories
# It takes a list of gaps and compute the gap times for
# any gap category
#
# @param cdqualParams  dictionary containing name value pairs
# @param gapResult     list of gaps
#
# @return list of accumulated gap times
# -----------------------------------------------------------------------------

def getGapTimes(cdqualParams, gapResult):
   overallGapTime = [ 0, 0, 0, 0, 0, 0, 0, 0, 0 ]

   # Sum up the overall gap time and compute minimum values
   # and Mission Capability
   for gap in gapResult:
      gapDiff = gap[G_END] - gap[G_BEG]
      if (gap[G_TYP] & NO_DATA_GAP_T) > 0:
         overallGapTime[0] += gapDiff
      if (gap[G_TYP] & AVAIL_GAPS_T) > 0:
         overallGapTime[1] += gapDiff
      if (gap[G_TYP] & AVAIL_SUR_GAPS_T) > 0:
         overallGapTime[2] += gapDiff
      if (gap[G_TYP] & TIM_DATA_GAPS_T) > 0:
         overallGapTime[3] += gapDiff
      if (gap[G_TYP] & TIM_AVAIL_GAPS_T) > 0:
         overallGapTime[4] += gapDiff
      if (gap[G_TYP] & TIM_AVAIL_SUR_GAPS_T) > 0:
         overallGapTime[5] += gapDiff
      if (gap[G_TYP] & MISSION_GAPS_T) > 0:
         overallGapTime[6] += gapDiff
      if (gap[G_TYP] & MISSION_SUR_GAPS_T) > 0:
         overallGapTime[7] += gapDiff
      if (gap[G_TYP] & AUTH_FAIL_GAP_T) > 0:
         overallGapTime[8] += gapDiff

   debug(2, cdqualParams, ('overall gap times: ', overallGapTime))
   return overallGapTime

# -----------------------------------------------------------------------------
# @short Get minimum values
#
# This function computes the minimum values using the samples
# received as basis for the calculation
#
# @param cdqualParams  dictionary containing name value pairs
# @param channels      list of channels
# @param minValues     List containing minimum values
#
# @return list of accumulated minimum values as percentage
# -----------------------------------------------------------------------------

def getMinValues(cdqualParams, channels, minValues, minChannels):

   for cStat in channels:
      # Sum up % data received, % constant data, late data
      # and data received unauthenticated
      expSamples = round(cStat[MAX_EXP_TIME] * cStat[SAMP_RATE])
      # Percentage data received
      samples = cStat[NBR_SAMPLES]
      minValues[0] += getPercent(expSamples, samples);
      # Percentage data available are samples received minus bad data
      samples -=  cStat[NBR_BAD_SAMPLES]
      minValues[1] += getPercent(expSamples, samples);
      # Percentage data available with surety are samples received minus
      # bad data minus untrusted, not bad data
      samples -=  cStat[NBR_UNTRUSTED_SAMPLES_WO_BAD]
      minValues[2] += getPercent(expSamples, samples);
      # Percentage data received timely are samples received minus too late
      # samples
      samples = cStat[NBR_SAMPLES] - cStat[NBR_LATE_SAMPLES]
      minValues[3] += getPercent(expSamples, samples);
      # Percentage data available timely are samples received minus bad
      # samples minus too late, but not bad samples
      samples = cStat[NBR_SAMPLES] - cStat[NBR_BAD_SAMPLES] - \
                cStat[NBR_LATE_SAMPLES_WO_BAD];
      minValues[4] += getPercent(expSamples, samples);
      # Percentage data available timely with surety are samples received
      # minus bad samples minus too late, but not bad samples, minus
      # untrusted but not bad and minus untrusted too late samples without
      # bad samples
      samples -= cStat[NBR_UNTRUSTED_LATE_SAMPLES_WO_BAD]
      samples -= cStat[NBR_UNTRUSTED_SAMPLES_WO_BAD]
      minValues[5] += getPercent(expSamples, samples);

   if minChannels > 0:
      # Trim minimum values greater than minimum channels * 100 (only 70%
      # of the elements must be operational)
      for ii in range(0, 6):
         if minValues[ii] > (minChannels * 100):
            minValues[ii] = (minChannels * 100)
         minValues[ii] /= minChannels

   debug(2, cdqualParams, ('min values: ', minValues))

# -----------------------------------------------------------------------------
# @short Get Seismic Statistics
#
# This function computes the statistics for a given
# seismic station. It takes a list of site/channel
# combinations and a list of gaps to compute the mission
# capability of the station
#
# @param cdqualParams  dictionary containing name value pairs
# @param cStatArray    list of statistics returned by getChannelStatistics()
# @param stationStats  station statistics list
# @param station       station name
#
# @return time where the minimum required number of channels were not
#         available
# -----------------------------------------------------------------------------

def getSeismicStatistics(cdqualParams, cStatArray, stationStats, station):
   # Rules Seismic: single 3C-station: all of the (broadband) components
   #                array: all three components of a 3C broadband seismometer
   #                       and at least 80% of the short period or broadband
   #                       elements (e.g. WRA). If more than one 3C broadband
   #                       than only one of them must be operational (e.g. ZAL)
   #  Known channel names: BH[12], [BMSHE]H[ENZ], [lbms][nez]
   nonSOHchannels = r'^([BMSH]H[12]|[BMSHE]H[ENZ]|[lbms][nez])$'
   overallGapTime = [ 0, 0, 0, 0, 0, 0, 0, 0 ]

   # First pass: sort out all channels not used for mission capability
   # Drop SOH and channels not used for mission capability. Only short period
   # and broadband channels are considered
   filteredStatistics = []
   for cStat in cStatArray:
      if re.match( nonSOHchannels, cStat[CHANNEL], 0):
         if re.match( r'^([BHSME]H[ENZ12]|[bms][nez])$', cStat[CHANNEL], 0):
            filteredStatistics.append(cStat)
            # Sum up % data received, % constant data, late data
            # and data received unauthenticated
            stationStats[P_RECVD] += cStat[NBR_SAMPLES]
            stationStats[P_BAD_SAMPLES] += cStat[NBR_BAD_SAMPLES]
            stationStats[P_TOO_LATE] += cStat[NBR_LATE_SAMPLES_WO_BAD]
            stationStats[P_UNTRUSTED] += cStat[NBR_UNTRUSTED_SAMPLES]
            # Sum up also number of expected samples for percentage computation
            stationStats[P_EXPECTED] += round(cStat[MAX_EXP_TIME] * \
                                              cStat[SAMP_RATE])
            stationStats[P_TIME_EXPECTED] += cStat[MAX_EXP_TIME]
         else:
            debug(2, cdqualParams, ('drop long period channel ',
                                     cStat[CHANNEL]))
      else:
         debug(2, cdqualParams, ('drop SOH channel ', cStat[CHANNEL]))

   # Second pass: consolidate gaps and add channel count to gaps for
   #              channels of interest, remove all gaps where the
   #              minimum number of required channels is available
   #              and sum up gap time
   #
   # Keep in mind that "bad" data (e.g. too late arrived, constant data
   # and data which is flagged by a particular status bit) have been
   # transformed into "virtual gaps"  before. The same algorithm is applied
   # to these types of gaps as for real gaps
   # Group channels by site
   sites = {}
   threeCsites = []
   otherChannels = []

   # Sort channels by site
   for channel in filteredStatistics:
      if sites.has_key(channel[SITE]):
         sites[channel[SITE]].append(channel)
      else:
         sites[channel[SITE]] = [channel, ]

   if len(sites) == 0:
      sys.exit("Error: %s. Station %s seems not to be seismic. %s" \
                % ('No channels left', cStatArray[0][0], 'Abort processing'))

   ## ----> minValues = [ 0, 0, 0, 0, 0, 0, 0, 0 ]
   # Sort out all channels not used for mission capability
   # and try to determine if site is a 3C seismometer
   for site in sites.keys():

      # Go through all the channels of the site
      haveMZ = haveMN = haveME = haveSZ = haveSN = haveSE = 0
      haveBZ = haveBN = haveBE = 0
      haveHZ = haveHN = haveHE = 0
      haveEZ = 0
      channels2remove = []

      for channel in sites[site]:

         # Filtering is done bases on channel name
         cn = channel[CHANNEL]
         debug(2, cdqualParams, "Processing " + site + " --> " + cn)

         if cn == 'BHZ' or cn == 'bz':
            haveBZ = 1
            continue
         if cn == 'BHN' or cn == 'BH2' or cn == 'bn':
            haveBN = 1
            continue
         if cn == 'BHE' or cn == 'BH1' or cn == 'be':
            haveBE = 1
            continue
         if cn == 'MHZ' or cn == 'mz':
            haveMZ = 1
            continue
         if cn == 'MHN' or cn == 'MH2' or cn == 'mn':
            haveMN = 1
            continue
         if cn == 'MHE' or cn == 'MH1' or cn == 'me':
            haveME = 1
            continue
         if cn == 'SHZ' or cn == 'sz':
            haveSZ = 1
            continue
         if cn == 'SHN' or cn == 'SH2' or cn == 'sn':
            haveSN = 1
            continue
         if cn == 'SHE' or cn == 'SH1' or cn == 'se':
            haveSE = 1
            continue
         if cn == 'HHZ':
            haveHZ = 1
            continue
         if cn == 'HHN' or cn == 'HH2' or cn == 'hn':
            haveHN = 1
            continue
         if cn == 'HHE' or cn == 'HH1' or cn == 'he':
            haveHE = 1
            continue
         if cn == 'EHZ':
            haveEZ = 1
            continue

         # No Broadband or short period channel, drop channel
         debug(2, cdqualParams, "Drop non mission channel " + site + "/" + cn)
         # (sites[site]).remove(channel) does not work!!
         channels2remove.append(channel)

      # Remove non mission channels from site
      for channel in channels2remove[:]:
          sites[site].remove(channel)
      channels2remove = []

      # Check for no channels left
      if len(sites[site]) == 0:
         continue

      channels2stay = []

      # Now check remaining channels
      if haveBZ and haveBN and haveBE:
         # Looks like a 3C Broadband, scan for channels not starting
         # with 'B' or 'b' and move them to the 'other' channels
         debug(2, cdqualParams, "Detect 3C broadband at " + site)
         channels2stay = [ 'B', 'b' ]
# Short period 3C do NOT contribute to mission capability (according to GP)
#    elif haveSZ and haveSN and haveSE:
         # Looks like a 3C Broadband, scan for channels not starting
         # with 'S' or 's' and move them to the 'other' channels
#         debug(2, cdqualParams, "Detect 3C short period at " + site)
#         channels2stay = [ 'S', 's' ]
      elif haveMZ and haveMN and haveME and haveSZ and haveSN and haveSE:
         # Looks like a 3C Broadband which consists of short and medium
         # period seismometers, scan for channels starting
         # not with 'M', 'S', 'm' or 's' and move them to the 'other' channels
         debug(2, cdqualParams, "Detect 3C mixed (short + medium) at " + site)
         channels2stay = [ 'M', 'S', 's', 'm' ]
      elif haveHZ and haveHN and haveHE:
         # Looks like a 3C high Broadband, scan for channels not starting
         # with 'H' and move them to the 'other' channels
         debug(2, cdqualParams, "Detect 3C high broadband at " + site)
         channels2stay = [ 'H', ]

      # Move channels from site which are not part of 3C to otherChannels
      for channel in sites[site]:
         if channel[CHANNEL][0] not in channels2stay:
            # Add statistics record to "other" channel list and remove
            # it from site
            # Only vertical channels are considered
            if channel[CHANNEL][-1] == 'Z' or  channel[CHANNEL][-1] == 'z':
               otherChannels.append(channel)
               debug(2, cdqualParams,
                     "Move %s/%s to otherChannels" % (site, channel[CHANNEL]))
            channels2remove.append(channel)
         else:
           debugGaps2(5, cdqualParams, ('gaps 3C: ', channel[GAP_LIST]))

      # Remove channels from site which are not part of 3C
      for channel in channels2remove[:]:
          sites[site].remove(channel)
          debug(2, cdqualParams, "Remove %s/%s" % (site, channel[CHANNEL]))
      channels2remove = []

      # Check for no channels left
      if len(sites[site]) == 0:
         continue

      ## Compute percentage received for channels of 3C
      ## and add them to minimum values
      ## ----> getMinValues(cdqualParams, sites[site], minValues, -1)

      # Only broadband channels should be left in 'site'
      # Consolidate all channels of the 3C broadband seismometer
      # All off the channels have to be operational. It is safe to
      # use combineGaps2() here
      gapResult = combineGaps2(cdqualParams, sites[site], ALL_GT)

      # All channels must be operational. No gaps are removed in
      # this case. Add the gap list to a statistics record to be
      # able to reprocess the gaps
      combinedChannels = createStatisticRec(site, '_C_', 1.0)
      ## combinedChannels.append(gapResult)
      combinedChannels[GAP_LIST] = gapResult
      threeCsites.append(combinedChannels)

   # Add the combined gap list to a new statistics record to be
   # able to reprocess the gaps
   result3C = createStatisticRec(site, '_C3', 1.0)

   ## Trim minimum values greater than 300% to 300% (only one
   ## of the 3C must be operational)
   ## ----> for ii in range(0, 6):
   ##      if minValues[ii] > 300:
   ##         minValues[ii] = 300

   # Consolidate all 3C sites to a single gap list
   channelCnt = len(threeCsites)
   debug(2, cdqualParams, "Number of 3C sites: " + str(channelCnt))
   if channelCnt > 1:
      # Remove all gaps where the minimum number of required channels
      # is available if more than one 3C element was detected. Here each
      # gap type has to be considered separately (all channels must be
      # operational).
      gapResult = combineGaps3(cdqualParams, threeCsites, 1, channelCnt)
      result3C[GAP_LIST] = gapResult

      # Computing the minimum values for seismic is tricky for periods where
      # none of the 3C sites where mission capable. In this case the 3C site
      # with the best data received (for this period) has to be considered.
      ### ===> gapTime3C = combineGaps3C(cdqualParams, sites)
      gapTime3C = getGapTimes(cdqualParams, result3C[GAP_LIST])
      for ii in range(0, 6):
         gapTime3C[ii] *= 3
   elif channelCnt == 1:
      # Only one 3C left, no need to consolidate sites
      result3C[GAP_LIST] = threeCsites[0][GAP_LIST]

      # Only one 3C site. One run of combineGapsMin() will do the job,
      # but first the 3C site has to be identified
      for site in sites.keys():
         if len(sites[site]) < 3:
            debug(2, cdqualParams, ('skip site: ', site))
            continue

         gapTime3C = combineGapsMin(cdqualParams, sites[site], 3, 3)
         break
   else:
      # Create gap for whole period if no site left
      gL = []
      # NOTE: this breaks minimum rules for auxiliary stations,
      #       but how to "weight" something which is not
      #       not requested is not defined yet. Possible solution
      #       might by to take a request from another channel.
      gL.append((cdqualParams['startTime'], cdqualParams['endTime'],
                 0, NO_DATA_GAP_T))
      result3C[GAP_LIST] = gL

      # No Site left. Creating a gap list for the whole
      # period is sufficient.
      gapTime3C = getGapTimes(cdqualParams, result3C[GAP_LIST])
      for ii in range(0, 6):
         gapTime3C[ii] *= 3

   debugGaps2(2, cdqualParams, ('Result 3C gaps: ', result3C[GAP_LIST]))
   ### gapTime3C = getGapTimes(cdqualParams, result3C[GAP_LIST])

   # Add the gap list to a statistics record to be able to reprocess
   # the gaps
   resultOther = createStatisticRec(site, '_C0', 1.0)

   # Consolidate all other channels to a single gap list
   # To consider backup rules here, consolidate backup group to one
   # gap list, decrease overall channel number by number of backup
   # channels and remove all gaps with G_CNT <= number of backup
   # channels and subtract number backup channels from channel count
   # in remaining gaps
   channelCnt = len(otherChannels)
   debug(2, cdqualParams, "Number of other sites: " + str(channelCnt))
   if channelCnt > 1:
      # Try to load mission rule set
      ruleResult = loadMissionRuleSet(station, cdqualParams, filteredStatistics,
                                      'seismic')
      if ruleResult is None:
         # Calculate minimum values first
         minChannels = ceil(channelCnt * 0.8)
         gapTimeOther = combineGapsMin(cdqualParams, otherChannels,
                                       minChannels, channelCnt)
         # No mission rule set found. Use default rule set.
         # Remove all gaps where the minimum number of required channels
         # is available if more than one channel left
         gapResult = combineGaps3(cdqualParams, otherChannels, minChannels,
                                  channelCnt)
         resultOther[GAP_LIST] = gapResult
      else:
         # NOTE:
         # Calculate minimum values first. To do minimum channel calculation
         # here might be incorrect if rule set also effects minimum values
         minChannels = ceil(channelCnt * 0.8)
         gapTimeOther = combineGapsMin(cdqualParams, otherChannels,
                                       minChannels, channelCnt)
         # Take gap list from rule set result (if any). Seismic stations
         # have to be treaded different than infrasound stations!! No default
         # rule set has to be checked, too!
         resultOther[GAP_LIST] = ruleResult[GAP_LIST]

   elif channelCnt == 1:
      # Only one channel, no need to consolidate channels
      resultOther[GAP_LIST] = otherChannels[0][GAP_LIST]
      minChannels = 1
      gapTimeOther = getGapTimes(cdqualParams, resultOther[GAP_LIST])
   else:
      # Create gap for whole period if no site left
      # Not sure if the rule for seismic stations should allow this
      gL = []
      # According to JG 80% of nothing is OK. Uncomment the next
      # two lines and set minChannels to 1 if this is not true
      # gL.append((cdqualParams['startTime'], cdqualParams['endTime'],
      #            0, NO_DATA_GAP_T))
      resultOther[GAP_LIST] = gL
      minChannels = 0
      gapTimeOther = [ 0, 0, 0, 0, 0, 0, 0, 0 ]

   # Compute minimum channel values for vertical broadband channels
   # The default 80 percent rule is used for the computation.
   ## ---> Uncomment next to lines out for alternative computation of
   ## minValues2 = [ 0, 0, 0, 0, 0, 0, 0, 0 ]
   ## getMinValues(cdqualParams, otherChannels, minValues2, minChannels)
   ## for ii in range(0, 6):
   ##   minValues[ii] += minValues2[ii]
   ##   minValues[ii] /= (minChannels + 3)
   ## debug(2, cdqualParams, ('all min values: ', minValues))

   debugGaps2(3, cdqualParams, ('Result other gaps: ', resultOther[GAP_LIST]))
   ### gapTimeOther = getGapTimes(cdqualParams, resultOther[GAP_LIST])

   # To compute minimum, but not mission values, give each gap list
   # a weight according to the number of channels required in the
   # minimum set. For a 3C the weight is 3, for other channels the
   # weight is the minimum number of required channels
   ### for ii in range(0, 6):
   ###   gapTime3C[ii] *= 3
   ###
   ### for ii in range(0, 6):
   ###    gapTimeOther[ii] *= minChannels

   minChannels += 3
   for ii in range(0, 6):
      overallGapTime[ii] = (gapTime3C[ii] + gapTimeOther[ii]) / minChannels

   # Consolidate result of 'other' channels and 3C Site
   allResult = []
   allResult.append(result3C)
   allResult.append(resultOther)

   # Consolidate virtual channels of the 3C broadband seismometer
   # and vertical broadband seismometer.
   # All off the channels have to be operational. It is safe to
   # use combineGaps2() here
   gapResult = combineGaps2(cdqualParams, allResult, ALL_GT)

   # Sum up the overall gap time and compute minimum values
   # and Mission Capability
   for gap in gapResult:
      gapDiff = gap[G_END] - gap[G_BEG]
      if (gap[G_TYP] & MISSION_GAPS_T) > 0:
         overallGapTime[6] += gapDiff
      if (gap[G_TYP] & MISSION_SUR_GAPS_T) > 0:
         overallGapTime[7] += gapDiff

   ## ---> Comment next to lines out for alternative computation of
   ## minimum values
   ## for ii in range(0, 6):
   ##   overallGapTime[ii] = minValues[ii]

   debug(2, cdqualParams, ('overall gap time: ', overallGapTime))
   return overallGapTime

# -----------------------------------------------------------------------------
# @short Get Hydro Statistics
#
# This function computes the statistics for a given
# hydro station. It takes a list of site/channel
# combinations and a list of gaps to compute the mission
# capability of the station
#
# @param cdqualParams   dictionary containing name value pairs
# @param cStatArray    list of statistics returned by getChannelStatistics()
# @param stationStats  station statistics list
#
# @return time where the minimum required number of channels were not
#         available
# -----------------------------------------------------------------------------

def getHydroStatistics(cdqualParams, cStatArray, stationStats):

   # Rules Hydrometer: minimum of one sensor per cable. Same station definition
   #                   is used as in processing (H10N and H10S are treated as
   #                   different stations)
   # Known channel names: EDH, LEA, LEV
   channelsToConsiderHydrometer = ('EDH', )
   # Rules T-Phase: minimum of one vertical sensor (horizontal produces too
   #                much noise)
   # Known channel names: [EH]H[ENZ], ez??
   channelsToConsiderTP = ('EHZ', 'HHZ')
   overallGapTime = [ 0, 0, 0, 0, 0, 0, 0, 0 ]
   isTP = 0
   channelCnt = 0

   # First pass: sort out all channels not used for mission capability
   filteredStatistics = []
   for cStat in cStatArray:
      if cStat[CHANNEL] in channelsToConsiderHydrometer:
         isTP = 0
      elif cStat[CHANNEL] in channelsToConsiderTP:
         isTP = 1
      else:
         debug(2, cdqualParams, ('drop SOH channel ', cStat[CHANNEL]))
         continue

      filteredStatistics.append(cStat)
      channelCnt += 1
      # Sum up % data received, % constant data, late data
      # and data received unauthenticated
      stationStats[P_RECVD] += cStat[NBR_SAMPLES]
      stationStats[P_BAD_SAMPLES] += cStat[NBR_BAD_SAMPLES]
      stationStats[P_TOO_LATE] += cStat[NBR_LATE_SAMPLES_WO_BAD]
      stationStats[P_UNTRUSTED] += cStat[NBR_UNTRUSTED_SAMPLES]
      # Sum up also number of expected samples for percentage computation
      stationStats[P_EXPECTED] += round(cStat[MAX_EXP_TIME] * \
                                        cStat[SAMP_RATE])
      stationStats[P_TIME_EXPECTED] += cStat[MAX_EXP_TIME]

   # Second pass: consolidate gaps and add channel count to gaps for
   #              channels of interest, remove all gaps where the
   #              minimum number of required channels is available
   #              and sum up gap time
   #
   # Keep in mind that "bad" data (e.g. too late arrived, constant data
   # and data which is flagged by a particular status bit) have been
   # transformed into "virtual gaps"  before. The same algorithm is applied
   # to these types of gaps as for real gaps

   # To consider backup rules here, consolidate backup group to one
   # gap list, decrease overall channel number by number of backup
   # channels and remove all gaps with G_CNT <= number of backup
   # channels and subtract number backup channels from channel count
   # in remaining gaps


   # Compute minimum values
   ## ---> Uncomment next to lines out for alternative computation of
   ## minValues = [ 0, 0, 0, 0, 0, 0, 0, 0 ]
   ## getMinValues(cdqualParams, filteredStatistics, minValues, 1)
   overallGapTime = combineGapsMin(cdqualParams, filteredStatistics,
                                   1, channelCnt)

   # Compute mission values
   gapResult = combineGaps3(cdqualParams, filteredStatistics, 1, channelCnt)

   # Sum up the overall gap time and compute Mission Capability
   overallGapTimeMission = getGapTimes(cdqualParams, gapResult)
   overallGapTime[6] = overallGapTimeMission[6]
   overallGapTime[7] = overallGapTimeMission[7]

   ## ---> Uncomment next to lines out for alternative computation of
   ## minimum values
   ## for ii in range(0, 6):
   ##    overallGapTime[ii] = minValues[ii]

   debug(2, cdqualParams, ('overall gap time: ', overallGapTime))
   return overallGapTime

# -----------------------------------------------------------------------------
# @short Combine Gaps with type
#
# This function consolidates the gap list stored in the statistics
# record for each (relevant) channel to one single gap list
# with the number of channels contributing to a gap as a gap
# attribute and considering the number of channel which must be operational.
# Gap consolidation is done for each gap type separately.
# This function requires that constant data gaps,
# which are part of a late data gap has been split before!
# CL gaps are not skipped!
#
# @param cdqualParams dictionary containing name value pairs
# @param fStatistics  list of statistics returned by getChannelStatistics()
# @param minCCount    minimum channel number required
# @param chanCount    total number of channels
#
# @return list containing combined gaps
# -----------------------------------------------------------------------------

def combineGaps3(cdqualParams, fStatistics, minCCount, chanCount):

   sGC = []
   maxChannelGapCnt = chanCount - minCCount

   # Process each gap type separately
   for gapType in (NO_DATA_GAP_T, LATE_GAP_T, BAD_DATA_GAP_T,
                   AUTH_FAIL_GAP_T, SURETY_GAP_T):
      # Consolidate channels in respect to current gap type. Ignore
      # gaps with other gap type. The 'no data gap' type has to be
      # considered for all other gap types or the gap count might become
      # too low if no data have been received on one or more channels
      gapResult = combineGaps2(cdqualParams, fStatistics,
                               gapType | NO_DATA_GAP_T)
      debugGaps2(3, cdqualParams, ('Result gaps by GT: ', gapResult))
      getGapTimes(cdqualParams, gapResult)
      for gap in gapResult[:]:
         # Pure no data gaps have to be considered only if gap type is
         # no data gap. Otherwise the 'no data gap' will cause too high
         # gap times
         # C3-1 NNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNN
         # C3-2 NNNUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU
         # C3-3 NNNUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU
         # Clear other gap types than current gap type
         if gapType != NO_DATA_GAP_T and gap[G_TYP] == NO_DATA_GAP_T:
            gapResult.remove(gap)
            continue

         gap[G_TYP] = gapType;
         # Remove all gaps where the minimum number of required channels
         # is available
         if gap[G_CNT] <= maxChannelGapCnt:
            gapResult.remove(gap)

      getGapTimes(cdqualParams, gapResult)
      debugGaps2(3, cdqualParams, ('Result gaps by GT after: ', gapResult))
      # Add the gap list to a statistics record to be
      # able to reprocess the gaps
      combinedChannels = createStatisticRec('Virt0', '_G_' + str(gapType), 1.0)
      ## combinedChannels.append(gapResult)
      combinedChannels[GAP_LIST] = gapResult
      sGC.append(combinedChannels)

   # No merge the gaps of all gap types
   gapResult = combineGaps2(cdqualParams,sGC, ALL_GT)
   getGapTimes(cdqualParams, gapResult)
   debugGaps2(3, cdqualParams, ('Result gaps by GT after all: ', gapResult))
   return(gapResult)


# -----------------------------------------------------------------------------
# @short Compute minimum values for 3C sites
#
# This function computes the minimum value for 3C sites. It splits data
# received at all 3C sites into periods where no data or not perfect data
# have been received at least at one channel. For each of these periods,
# it determines which of the 3C site has the best data received and uses
# this value for computing the minimal values. It could happen, that
# for different metrics, different 3C sites could be used
#
# DDDDDDDDD-DDDDDDDDDDDDD-DDDDDDDDDDDDDDDDDDDDD-DDDDD-DDDDDDDDDDD \
# --DDDLLDD-DDDDUUUUUUUDD-DDDDDDDDDDDDDDDDDBBBD-------DDDDDDDDDDD  > Site A
# --DDDLLDD-DDDDDDDDDDDDD-DDDDDDDDDDDDDDDDDBBBD-------DDDDDDDDDDD /
#
# D-DDDDDDD-DDDDDDDDDDDDDDDDDDDDD--------DDDDDD----DD-DDDDDDDDDDD \
# D-DDDLLDD-DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD----DD-DDDDDDDDDDD  > Site B
# --DDDDDDD-D--------DDDD-DDDDDDDDDDDDDDDDDDDDD-------DDDDDDDDDDD /
#
# BAAAABBAA-AAAAAAAAAAAAAB-AAAAAAAAAAAAAAAABBBA-AAABB-AAAAAAAAAAA Site taken
#
# @param cdqualParams dictionary containing name value pairs
# @param threeCsites  sites array
#
# @return list containing gap times weighted by gap count
# -----------------------------------------------------------------------------

def combineGaps3C(cdqualParams, sites):

   sGC = []
   overallGapTime = [ 0, 0, 0, 0, 0, 0, 0, 0 ]
   curValue = 0

   debug(2, cdqualParams, ('In combineGaps3C', ))
   # Merge channels from all sites into one virtual site
   for site in sites.keys():
      if len(sites[site]) < 3:
         debug(2, cdqualParams, ('skip site: ', site))
         continue

      for channel in sites[site]:
         debug(2, cdqualParams, ('append site/channel: ', site,
                  channel[CHANNEL]))
         sGC.append(channel)

   gapResult = combineGaps2(cdqualParams, sGC, ALL_GT)
   debugGaps2(3, cdqualParams, ('Result all gaps merged: ', gapResult))

   # gapResult contains a gap list with the combined gaps of all channels
   # This gap list is used to split data of all 3C sites into
   # portions of the same periods to easily compare periods after
   for gap in gapResult:
      gap[G_TYP] = ALL_GT

   gapChannel = createStatisticRec(site, '_C3G_', 1.0)
   gapChannel[GAP_LIST] = gapResult

   allGapList = {}
   # For each 3C site
   for site in sites.keys():
      sGC = []
      if len(sites[site]) < 3:
         continue

      debug(2, cdqualParams, ('merge site: ', site))
      # Merge channels from one site plus gap channel into one virtual site
      for channel in sites[site]:
         sGC.append(channel)
      sGC.append(gapChannel)

      # Process each minimum value separately
      for gapType in (NO_DATA_GAP_T, AVAIL_GAPS_T, AVAIL_SUR_GAPS_T,
                      TIM_DATA_GAPS_T, TIM_AVAIL_GAPS_T, TIM_AVAIL_SUR_GAPS_T):
         # Consolidate channels in respect to current gap type. Ignore
         # gaps with other gap type
         gapResult = combineGaps2(cdqualParams, sGC, gapType)
         debugGaps2(3, cdqualParams, ('Result gaps by GT: ', gapResult))
         getGapTimes(cdqualParams, gapResult)
         listName = str(gapType)

         # Save gap list for later processing
         if allGapList.has_key(listName):
            allGapList[listName].append(gapResult)
         else:
            allGapList[listName] = [gapResult, ]

   # Now a gap list for each metric and each site is available. Next step
   # is too compare the gap counts for each site and metric and take the
   # value form the site which the lowest gap count (= best value) to
   # compute the minimum value
   for gapType in (NO_DATA_GAP_T, AVAIL_GAPS_T, AVAIL_SUR_GAPS_T,
                   TIM_DATA_GAPS_T, TIM_AVAIL_GAPS_T, TIM_AVAIL_SUR_GAPS_T):
      listName = str(gapType)
      bestGaps = []
      debug(2, cdqualParams, ('merge site: ', site))
      for gapResult in allGapList[listName]:
          if len(bestGaps) == 0:
             bestGaps = gapResult
             continue

          if len(bestGaps) != len(gapResult):
             debugGaps2(1, cdqualParams, ('best gaps: ', bestGaps))
             debugGaps2(1, cdqualParams, ('current gaps: ', gapResult))

          for ii in range(len(gapResult)):
             if bestGaps[ii][G_CNT] > gapResult[ii][G_CNT]:
                bestGaps[ii][G_CNT] = gapResult[ii][G_CNT]

      # Iterate through the gap list and sum up gap times for minimum
      # values
      for gap in bestGaps:
         gapCnt = gap[G_CNT] - 1 # minus 1 is the gap channel
         gapDiff = gap[G_END] - gap[G_BEG]
         overallGapTime[curValue] += (gapDiff * gapCnt)

      # compute next minimum channel value
      curValue += 1

   debug(2, cdqualParams, ('overall gap times combine3C: ', overallGapTime))
   return(overallGapTime)


# -----------------------------------------------------------------------------
# @short Combine Gaps with type for minimum values
#
# This function consolidates the gap list stored in the statistics
# record for each (relevant) channel to one single gap list
# with the number of channels contributing to a gap as a gap
# attribute and considering the number of channel which must be operational.
# Gap consolidation is done for each minimum value separately.
# This function requires that constant data gaps,
# which are part of a late data gap has been split before!
# CL gaps are not skipped!
#
# @param cdqualParams dictionary containing name value pairs
# @param fStatistics  list of statistics returned by getChannelStatistics()
# @param minCCount    minimum channel number required
# @param chanCount    total number of channels
#
# @return list containing gap times weighted by gap count
# -----------------------------------------------------------------------------

def combineGapsMin(cdqualParams, fStatistics, minCCount, chanCount):

   sGC = []
   overallGapTime = [ 0, 0, 0, 0, 0, 0, 0, 0 ]
   curValue = 0
   debug(2, cdqualParams, ('In combineGapsMin', ))

   # Process each minimum value separately
   for gapType in (NO_DATA_GAP_T, AVAIL_GAPS_T, AVAIL_SUR_GAPS_T,
                   TIM_DATA_GAPS_T, TIM_AVAIL_GAPS_T, TIM_AVAIL_SUR_GAPS_T):
      # Consolidate channels in respect to current gap type. Ignore
      # gaps with other gap type
      gapResult = combineGaps2(cdqualParams, fStatistics, gapType)
      debugGaps2(3, cdqualParams, ('Result gaps by GT: ', gapResult))
      # getGapTimes(cdqualParams, gapResult)

      maxChannelNotNeeded = chanCount - minCCount
      # Iterate through the gap list and sum up gap times for minimum
      # and mission values
      for gap in gapResult:
         # Minimum rule:
         # Decrease the number of gaps by the number of channels not
         # needed. If the number of channels not needed is higher
         # than the gap count don't count the gap.
         if gap[G_CNT] > maxChannelNotNeeded:
            gapCnt = gap[G_CNT] - maxChannelNotNeeded
            gapDiff = gap[G_END] - gap[G_BEG]
            # Minimum rule:
            # Sum up the gap time weighted by the gap count
            overallGapTime[curValue] += (gapDiff * gapCnt)

            # Gaps for TIM_AVAIL_GAPS_T and TIM_AVAIL_SUR_GAPS_T are also
            # used for computing the mission values.
            # Mission rule:
            # If the gap count is higher than the number of
            # channels not needed weight the gap with the minimum number
            # of channels required
            if gapType == TIM_AVAIL_GAPS_T:
               overallGapTime[6] += (gapDiff * minCCount)
            elif gapType == TIM_AVAIL_SUR_GAPS_T:
               overallGapTime[7] += (gapDiff * minCCount)

      # compute next minimum channel value
      curValue += 1

   # Now overall gap time weighted by gap count is divided by minimum
   # channel count
   # DON'T: time is needed as it is by seismic

   debug(2, cdqualParams, ('overall gap times minimum: ', overallGapTime))
   return(overallGapTime)


# -----------------------------------------------------------------------------
# @short Combine Gaps with type
#
# This function consolidates the gap list stored in the statistics
# record for each (relevant) channel to one single gap list
# with the number of channels contributing to a gap as a gap
# attribute. This function requires that constant data gaps,
# which are part of a late data gap has been split before!
# CL gaps are not skipped!
# Gap consolidation is done for the requested gap type.
#
# @param cdqualParams dictionary containing name value pairs
# @param fStatistics  list of statistics returned by getChannelStatistics()
# @param gapType2Con  gap type to consider
#
# @return list containing combined gaps
# -----------------------------------------------------------------------------

def combineGaps2(cdqualParams, channelStatistics, gapType2Con):
   isFirst = 1
   combinedGapList = []
   tmpGapList = []

   for cStat in channelStatistics:
       debug(2, cdqualParams, ('Site = %s, channel = %s' % (cStat[SITE],
                                cStat[CHANNEL])))
       debug(2, cdqualParams, ('NSR = %f, RSR = %s gt=%d' % \
                (cStat[NOM_SAMP_RATE], cStat[SAMP_RATE], gapType2Con)))

       if len(cStat[GAP_LIST]) == 0:
          continue;

       debugGaps2(5, cdqualParams, ('Gap list in: ', cStat[GAP_LIST]))

       if 1 == isFirst:
          # Copy gap list of first channel to consolidated gap list
          # and set gap channel count to 1
          for gapCombined in cStat[GAP_LIST]:
             # Consider gaps only with requested type
             if (gapCombined[G_TYP] & gapType2Con) == 0:
                continue;
             # Reset counter
             lgap = list(gapCombined)
             lgap[G_CNT] = 1
             combinedGapList.append(lgap)

          # Algorithm requires that combinedGapList is not empty
          if len(combinedGapList) > 0:
             isFirst = 0
             debugGaps2(5, cdqualParams, ('Combined gaps: ', combinedGapList))
          continue


       # Gaps are created as tuples, but the algorithm requires that gaps
       # can be modified. So a temporary gap list is created for the
       # new gaps (A)
       currentGapList = []
       for gap in cStat[GAP_LIST]:
           # Consider gaps only with requested type
           if (gap[G_TYP] & gapType2Con) == 0:
              continue;
           # Reset counter
           lgap = list(gap)
           lgap[G_CNT] = 1
           currentGapList.append(lgap)

       # Gap end time is END time of last sample and not start time of last
       # sample. This is more useful for overall gap time calculation!

       # Now add current gap list to consolidated gap list
       for gapC in combinedGapList:
          debug(5, cdqualParams, ('gapC +---- ', gapC))
          gapCprocessed = 0
          # Keep in mind that list.remove(element) won't work when the
          # list is being iterated!! Iterate trough a copy instead
          for gapA in currentGapList[:]:
             # Combined:1-----1111-----111111-----1111------111------11111
             # Gaps:    --xx-x--xxx--xxxx----xx---xxxx---xxxxxxxx---------
             # Result:  1-11-111221--1122111111---2222---11122211----11111
             # ===========================================================
             # New gap --> copy gapA
             # GLC:     ------1111-----oooooo-o---oooo---oooo---ooooo--ooo
             # GLA:     AA---------x-xxxxxxxxxx----xx---xxx-----xxxxxxxxx-
             # GLA(new):-------------xxxxxxxxxx----xx---xxx-----xxxxxxxxx-
             # Result:  11------------------------------------------------
             if gapA[G_END] <= gapC[G_BEG]:
                debug(5, cdqualParams, ('gapA    +-:', gapA, 'NA'))
                tmpGapList.append(gapA)
                # This gap has been processed complete.
                currentGapList.remove(gapA)
                # Continue with next gapA
                continue
             # New gap --> copy gapC
             # GLC:     ------1111-----oooooo-o---oooo---oooo---ooooo--ooo
             # GLA:     -----------A-xxxxxxxxxx----xx---xxx-----xxxxxxxxx-
             # GLA(new):-------------xxxxxxxxxx----xx---xxx-----xxxxxxxxx-
             # Result:  ------1111----------------------------------------
             ## if gapA[G_BEG] > gapC[G_BEG] and gapA[G_END] >= gapC[G_END]:
             if gapA[G_BEG] >= gapC[G_END]:
                debug(5, cdqualParams, ('gapA    +-:', gapA, 'NC'))
                tmpGapList.append(gapC)
                # Break to continue with next gapC
                gapCprocessed = 1
                break
             # Duplicate --> copy gapC with gap_count+1
             # GLC:     ------1111-----oooooo-o---oooo---oooo---ooooo--ooo
             # GLA:     ------AAAA---xxxxxxxxxx----xx---xxx-----xxxxxxxxx-
             # GLA(new):-------------xxxxxxxxxx----xx---xxx-----xxxxxxxxx-
             # Result:  ------2222----------------------------------------
             if gapA[G_BEG] == gapC[G_BEG] and gapA[G_END] == gapC[G_END]:
                debug(5, cdqualParams, ('gapA    +-:', gapA, 'D'))
                gapC[G_CNT] += 1
                gapC[G_TYP] = gapC[G_TYP] | gapA[G_TYP]
                tmpGapList.append(gapC)
                # These gaps have been processed complete
                currentGapList.remove(gapA)
                # Break to continue with next gapC
                gapCprocessed = 1
                break
             # GapA complete overlaps gapC --> create 1 new gap, copy gapC
             # with gap_count+1 and modify start of gapA
             # GLC:     ------oooo-----111111-o---oooo------ooo------ooooo
             # GLA:     -------------AAAAAAAAAA----xx---xxx-------xxxxxxxx
             # GLA(new):---------------------xx----xx---xxx-------xxxxxxxx
             # Result:  -------------11222222-----------------------------
             if gapA[G_BEG] < gapC[G_BEG] and gapA[G_END] > gapC[G_END]:
                debug(5, cdqualParams, ('gapA    +-:', gapA, 'OCC'))
                # End time of tmp gap is start time of C gap
                tmpGapList.append([gapA[G_BEG], gapC[G_BEG], 1, gapA[G_TYP]])
                gapC[G_CNT] += 1
                gapC[G_TYP] = gapC[G_TYP] | gapA[G_TYP]
                tmpGapList.append(gapC)
                # Rest of gapA is treated in the next iteration
                # Start time of modified A gap is end time of C gap
                gapA[G_BEG] = gapC[G_END]
                # Break to continue with next gapC
                gapCprocessed = 1
                break
             # GapA overlaps GapC but same end --> create 1 new gap, copy
             # gapC with gap_count+1 and delete gapA
             # GLC:     ------oooo-----oooooo-1---oooo------ooo------ooooo
             # GLA:     ---------------------AA----xx---xxx-------xxxxxxxx
             # GLA(new):---------------------------xx---xxx-------xxxxxxxx
             # Result:  ---------------------12---------------------------
             if gapA[G_BEG] < gapC[G_BEG] and gapA[G_END] == gapC[G_END]:
                debug(5, cdqualParams, ('gapA    +-:', gapA, 'OCCE'))
                # End time of tmp gap is start time of C gap
                tmpGapList.append([gapA[G_BEG], gapC[G_BEG], 1, gapA[G_TYP]])
                gapC[G_CNT] += 1
                gapC[G_TYP] = gapC[G_TYP] | gapA[G_TYP]
                tmpGapList.append(gapC)
                # These gaps have been processed complete
                currentGapList.remove(gapA)
                # Break to continue with next gapC
                gapCprocessed = 1
                break
             # GapA overlaps gapC but same start --> copy gapC with gap_count+1
             # and modify start of gapA
             # GLC:     ------oooo-----oooooo-1-oooooo------ooo------ooooo
             # GLA:     ----------------------AAAA-xx---xxx-------xxxxxxxx
             # GLA(new):-----------------------xxx-xx---xxx-------xxxxxxxx
             # Result:  ----------------------2---------------------------
             if gapA[G_BEG] == gapC[G_BEG] and gapA[G_END] > gapC[G_END]:
                debug(5, cdqualParams, ('gapA    +-:', gapA, 'OCCB'))
                gapC[G_CNT] += 1
                gapC[G_TYP] = gapC[G_TYP] | gapA[G_TYP]
                tmpGapList.append(gapC)
                # Rest of gapA is treated the next iteration
                # Start time of modified gap A is end time of gap C
                gapA[G_BEG] = gapC[G_END]
                # Break to continue with next gapC
                gapCprocessed = 1
                break
             # GapA overlaps gapC on the left side --> split gapA, increase
             # gap_count by 1 on second part of gapA, modify start of gapC
             # and delete gapA
             # GLC:     ------oooo-----oooooo-o-111111------ooo------ooooo
             # GLA:     -----------------------AAA-xx---xxx-------xxxxxxxx
             # GLA(new):---------------------------xx---xxx-------xxxxxxxx
             # GLC(new) ------oooo-----oooooo-o---1111------ooo------ooooo
             # Result:  -----------------------122------------------------
             if gapA[G_BEG] < gapC[G_BEG] and gapA[G_END] < gapC[G_END]:
                debug(5, cdqualParams, ('gapA    +-:', gapA, 'OCL'))
                # End time of tmp gap is start time of C gap
                tmpGapList.append([gapA[G_BEG], gapC[G_BEG], 1, gapA[G_TYP]])
                tmpGapList.append([gapC[G_BEG], gapA[G_END], gapC[G_CNT] + 1,
                                   gapC[G_TYP] | gapA[G_TYP]])
                # Rest of gapC is treated in the next iteration
                # Start time of modified gap C is end time of gap A
                gapC[G_BEG] = gapA[G_END]
                # This gap has been processed complete
                currentGapList.remove(gapA)
                # Continue with next gapA
                continue
             # GapA overlaps gapC on the right side --> split gapA, increase
             # gap_count by 1 on first part of gapA and modify start of gapA
             # GLC:     ------oooo-----oooooo-o-111111------ooo------ooooo
             # GLA:     ---------------------------AAAA-xxx-------xxxxxxxx
             # GLA(new):------------------------------x-xxx-------xxxxxxxx
             # Result:  ------------------------111222--------------------
             if gapA[G_BEG] > gapC[G_BEG] and gapA[G_END] > gapC[G_END]:
                debug(5, cdqualParams, ('gapA    +-:', gapA, 'OCR'))
                # End time of tmp gap is start time of A gap
                tmpGapList.append([gapC[G_BEG], gapA[G_BEG], gapC[G_CNT],
                                   gapC[G_TYP]])
                tmpGapList.append([gapA[G_BEG], gapC[G_END], gapC[G_CNT] + 1,
                                   gapC[G_TYP] | gapA[G_TYP]])
                # Rest of gapA is treated in the next iteration
                # Start time of modified gap A is end time of gap C + 1 sample
                gapA[G_BEG] = gapC[G_END]
                # Break to continue with next gapC
                gapCprocessed = 1
                break
             # GapC complete overlaps gapA --> create 1 new gap, increase gap
             # count of gapC by 1, modify start of gapC and remove gapA
             # GLC:     ------oooo-----oooooo-o---22222222--ooo------ooooo
             # GLA:     ---------------------------AA---xxx-------xxxxxxxx
             # GLC(new):------oooo-----oooooo-o------22222--ooo------ooooo
             # GLA(new):--------------------------------xxx-------xxxxxxxx
             # Result:  --------------------------233---------------------
             if gapA[G_BEG] > gapC[G_BEG] and gapA[G_END] < gapC[G_END]:
                debug(5, cdqualParams, ('gapA    +-:', gapA, 'OCI'))
                # End time of tmp gap is start time of A gap
                tmpGapList.append([gapC[G_BEG], gapA[G_BEG], gapC[G_CNT],
                                   gapC[G_TYP]])
                tmpGapList.append([gapA[G_BEG], gapA[G_END], gapC[G_CNT] + 1,
                                   gapC[G_TYP] | gapA[G_TYP]])
                # Rest of gapC is tested in the next iteration
                # Start time of modified gap C is end time of gap A
                gapC[G_BEG] = gapA[G_END]
                # This gap has been processed complete
                currentGapList.remove(gapA)
                # Continue with next gapA
                continue
             # GapC overlaps gapA but same start --> copy gapA with gap_count+1,
             # modify start of gapC remove gapA
             # GLC:     ------oooo-----oooooo-o---22222222--ooo------ooooo
             # GLA:     --------------------------AAA---xxx-------xxxxxxxx
             # GLC(new):------oooo-----oooooo-o------22222--ooo------ooooo
             # GLA(new):--------------------------------xxx-------xxxxxxxx
             # Result:  --------------------------333---------------------
             if gapA[G_BEG] == gapC[G_BEG] and gapA[G_END] < gapC[G_END]:
                debug(5, cdqualParams, ('gapA    +-:', gapA, 'OAL'))
                tmpGapList.append([gapA[G_BEG], gapA[G_END], gapC[G_CNT] + 1,
                                   gapC[G_TYP] | gapA[G_TYP]])
                # Rest of gapC is tested in the next iteration
                # Start time of modified gap C is end time of gap A
                gapC[G_BEG] = gapA[G_END]
                # This gap has been processed complete
                currentGapList.remove(gapA)
                # Continue with next gapA
                continue
             # GapC overlaps gapA but same end --> copy gapA with gap_count+1,
             # modify start of gapC, remove gapA and
             # copy gapC with end time = start time gapA
             # GLC:     ------oooo-----oooooo-o---22222222--ooo------ooooo
             # GLA:     -------------------------------AAA--------xxxxxxxx
             # GLC(new):------oooo-----oooooo-o---oooooooo--ooo------ooooo
             # GLA(new):------------------------------------------xxxxxxxx
             # Result:  --------------------------22222333----------------
             if gapA[G_BEG] > gapC[G_BEG] and gapA[G_END] == gapC[G_END]:
                debug(5, cdqualParams, ('gapA    +-:', gapA, 'OAR'))
                # End time of tmp gap is start time of A gap
                tmpGapList.append([gapC[G_BEG], gapA[G_BEG], gapC[G_CNT],
                                   gapC[G_TYP]])
                tmpGapList.append([gapA[G_BEG], gapA[G_END], gapC[G_CNT] + 1,
                                   gapC[G_TYP] | gapA[G_TYP]])
                # This gap has been processed complete
                currentGapList.remove(gapA)
                # Break to continue with next gapC
                gapCprocessed = 1
                break

             # GapA not processed!! Stop application
             sys.exit("Error: GapA %s-%s not processed (GapC: %s-%s) %s/%s" \
                   % (gapA[G_BEG], gapA[G_END], gapC[G_BEG], gapC[G_END],
                       cStat[SITE], cStat[CHANNEL]))

          # End for gapA in currentGapList:
          # Check if combined gap has been processed complete
          if not gapCprocessed:
             tmpGapList.append(gapC)

       # End for gapC in combinedGapList:
       # Check for one gapA left
       for gapA in currentGapList[:]:
          tmpGapList.append(gapA)

       # Make the temporary gap list the combined gap list if it
       # contain gaps, otherwise leave the combined gap list untouched!!
       if len(tmpGapList) > 0:
          combinedGapList = tmpGapList
       tmpGapList = []
       debugGaps2(5, cdqualParams, ('Combined gaps: ', combinedGapList))

   debugGaps2(5, cdqualParams, ('Result gaps: ', combinedGapList))
   return combinedGapList

# -----------------------------------------------------------------------------
# @short Combine Gaps
#
# This function consolidates the gap list stored in the statistics
# record for each (relevant) channel to one single gap list
# with the number of channels contributing to a gap as a gap
# attribute
#
# @param cdqualParams dictionary containing name value pairs
# @param fStatistics  list of statistics returned by getChannelStatistics()
#                     excluding all SOH and other auxiliary channels
#
# @return list containing combined gaps
# -----------------------------------------------------------------------------

def combineGapsXX(cdqualParams, channelStatistics):
   isFirst = 1
   combinedGapList = []
   tmpGapList = []

   for statistics in channelStatistics:
       debug(2, cdqualParams, ('Site = %s, channel = %s' % (statistics[SITE],
                                statistics[CHANNEL])))
       debug(2, cdqualParams, ('NSR = %f, RSR = %s' % \
                (statistics[NOM_SAMP_RATE], statistics[SAMP_RATE])))
       debugGaps2(3, cdqualParams, ('Gap list: ', statistics[GAP_LIST]))
       if 1 == isFirst:
          # Copy gap list of first channel to consolidated gap list
          # and set gab channel count to 1
          isFirst = 0
          for gapCombined in statistics[GAP_LIST]:
             # Skip constant data gaps which are also late data to not
             # count gap time twice later
             if 'CL' == gapCombined[2]:
                continue
             combinedGapList.append([gapCombined[0], gapCombined[1], 1])
          continue

       debugGaps2(5, cdqualParams, ('Combined gaps: ', combinedGapList))

       # Gaps are created as tuples, but the algorithm requires that gaps
       # can be modified. So a temporary gap list is created for the
       # new gaps (A)
       currentGapList = []
       for gap in statistics[GAP_LIST]:
           # Skip constant data gaps which are also late data to not
           # count gap time twice later
           if 'CL' == gap[2]:
              continue

           currentGapList.append([gap[0], gap[1], 1])

       # Gap end time is END time of last sample and not start time of last
       # sample. This is more useful for overall gap time calculation!

       # Now add current gap list to consolidated gap list
       for gapC in combinedGapList:
          debug(5, cdqualParams, ('gapC +---- ', gapC))
          # Keep in mind that list.remove(element) won't work when the
          # list is being iterated!! Iterate trough a copy instead
          for gapA in currentGapList[:]:
             # Combined:1-----1111-----111111-----1111------111------11111
             # Gaps:    --xx-x--xxx--xxxx----xx---xxxx---xxxxxxxx---------
             # Result:  1-11-111221--1122111111---2222---11122211----11111
             # ===========================================================
             # New gap --> copy gapA
             # GLC:     ------1111-----oooooo-o---oooo---oooo---ooooo--ooo
             # GLA:     AA---------x-xxxxxxxxxx----xx---xxx-----xxxxxxxxx-
             # GLA(new):-------------xxxxxxxxxx----xx---xxx-----xxxxxxxxx-
             # Result:  11------------------------------------------------
             if gapA[G_END] <= gapC[G_BEG]:
                debug(5, cdqualParams, ('gapA    +-:', gapA, 'NA'))
                tmpGapList.append([gapA[G_BEG], gapA[G_END], 1])
                # This gap has been processed complete.
                currentGapList.remove(gapA)
                # Continue with next gapA
                continue
             # New gap --> copy gapC
             # GLC:     ------1111-----oooooo-o---oooo---oooo---ooooo--ooo
             # GLA:     -----------A-xxxxxxxxxx----xx---xxx-----xxxxxxxxx-
             # GLA(new):-------------xxxxxxxxxx----xx---xxx-----xxxxxxxxx-
             # Result:  ------1111----------------------------------------
             ## if gapA[G_BEG] > gapC[G_BEG] and gapA[G_END] >= gapC[G_END]:
             if gapA[G_BEG] >= gapC[G_END]:
                debug(5, cdqualParams, ('gapA    +-:', gapA, 'NC'))
                tmpGapList.append([gapC[G_BEG], gapC[G_END], gapC[G_CNT]])
                # Break to continue with next gapC
                break
             # Duplicate --> copy gapC with gap_count+1
             # GLC:     ------1111-----oooooo-o---oooo---oooo---ooooo--ooo
             # GLA:     ------AAAA---xxxxxxxxxx----xx---xxx-----xxxxxxxxx-
             # GLA(new):-------------xxxxxxxxxx----xx---xxx-----xxxxxxxxx-
             # Result:  ------2222----------------------------------------
             if gapA[G_BEG] == gapC[G_BEG] and gapA[G_END] == gapC[G_END]:
                debug(5, cdqualParams, ('gapA    +-:', gapA, 'D'))
                tmpGapList.append([gapC[G_BEG], gapC[G_END], gapC[G_CNT] + 1])
                # These gaps have been processed complete
                currentGapList.remove(gapA)
                # Break to continue with next gapC
                break
             # GapA complete overlaps gapC --> create 1 new gap, copy gapC
             # with gap_count+1 and modify start of gapA
             # GLC:     ------oooo-----111111-o---oooo------ooo------ooooo
             # GLA:     -------------AAAAAAAAAA----xx---xxx-------xxxxxxxx
             # GLA(new):---------------------xx----xx---xxx-------xxxxxxxx
             # Result:  -------------11222222-----------------------------
             if gapA[G_BEG] < gapC[G_BEG] and gapA[G_END] > gapC[G_END]:
                debug(5, cdqualParams, ('gapA    +-:', gapA, 'OCC'))
                # End time of tmp gap is start time of C gap
                tmpGapList.append([gapA[G_BEG], gapC[G_BEG], 1])
                tmpGapList.append([gapC[G_BEG], gapC[G_END], gapC[G_CNT] + 1])
                # Rest of gapA is treated in the next iteration
                # Start time of modified A gap is end time of C gap
                gapA[G_BEG] = gapC[G_END]
                # Break to continue with next gapC
                break
             # GapA overlaps GapC but same end --> create 1 new gap, copy
             # gapC with gap_count+1 and delete gapA
             # GLC:     ------oooo-----oooooo-1---oooo------ooo------ooooo
             # GLA:     ---------------------AA----xx---xxx-------xxxxxxxx
             # GLA(new):---------------------------xx---xxx-------xxxxxxxx
             # Result:  ---------------------12---------------------------
             if gapA[G_BEG] < gapC[G_BEG] and gapA[G_END] == gapC[G_END]:
                debug(5, cdqualParams, ('gapA    +-:', gapA, 'OCCE'))
                # End time of tmp gap is start time of C gap
                tmpGapList.append([gapA[G_BEG], gapC[G_BEG], 1])
                tmpGapList.append([gapC[G_BEG], gapC[G_END], gapC[G_CNT] + 1])
                # These gaps have been processed complete
                currentGapList.remove(gapA)
                # Break to continue with next gapC
                break
             # GapA overlaps gapC but same start --> copy gapC with gap_count+1
             # and modify start of gapA
             # GLC:     ------oooo-----oooooo-1-oooooo------ooo------ooooo
             # GLA:     ----------------------AAAA-xx---xxx-------xxxxxxxx
             # GLA(new):-----------------------xxx-xx---xxx-------xxxxxxxx
             # Result:  ----------------------2---------------------------
             if gapA[G_BEG] == gapC[G_BEG] and gapA[G_END] > gapC[G_END]:
                debug(5, cdqualParams, ('gapA    +-:', gapA, 'OCCB'))
                tmpGapList.append([gapC[G_BEG], gapC[G_END], gapC[G_CNT] + 1])
                # Rest of gapA is treated the next iteration
                # Start time of modified gap A is end time of gap C
                gapA[G_BEG] = gapC[G_END]
                # Break to continue with next gapC
                break
             # GapA overlaps gapC on the left side --> split gapA, increase
             # gap_count by 1 on second part of gapA, modify start of gapC
             # and delete gapA
             # GLC:     ------oooo-----oooooo-o-111111------ooo------ooooo
             # GLA:     -----------------------AAA-xx---xxx-------xxxxxxxx
             # GLA(new):---------------------------xx---xxx-------xxxxxxxx
             # GLC(new) ------oooo-----oooooo-o---1111------ooo------ooooo
             # Result:  -----------------------122------------------------
             if gapA[G_BEG] < gapC[G_BEG] and gapA[G_END] < gapC[G_END]:
                debug(3, cdqualParams, ('gapA    +-:', gapA, 'OCL'))
                # End time of tmp gap is start time of C gap
                tmpGapList.append([gapA[G_BEG], gapC[G_BEG], 1])
                tmpGapList.append([gapC[G_BEG], gapA[G_END], gapC[G_CNT] + 1])
                # Rest of gapC is treated in the next iteration
                # Start time of modified gap C is end time of gap A
                gapC[G_BEG] = gapA[G_END]
                # This gap has been processed complete
                currentGapList.remove(gapA)
                # Continue with next gapA
                continue
             # GapA overlaps gapC on the right side --> split gapA, increase
             # gap_count by 1 on first part of gapA and modify start of gapA
             # GLC:     ------oooo-----oooooo-o-111111------ooo------ooooo
             # GLA:     ---------------------------AAAA-xxx-------xxxxxxxx
             # GLA(new):------------------------------x-xxx-------xxxxxxxx
             # Result:  ------------------------111222--------------------
             if gapA[G_BEG] > gapC[G_BEG] and gapA[G_END] > gapC[G_END]:
                debug(5, cdqualParams, ('gapA    +-:', gapA, 'OCR'))
                # End time of tmp gap is start time of A gap
                tmpGapList.append([gapC[G_BEG], gapA[G_BEG], gapC[G_CNT]])
                tmpGapList.append([gapA[G_BEG], gapC[G_END], gapC[G_CNT] + 1])
                # Rest of gapA is treated in the next iteration
                # Start time of modified gap A is end time of gap C + 1 sample
                gapA[G_BEG] = gapC[G_END]
                # Break to continue with next gapC
                break
             # GapC complete overlaps gapA --> create 1 new gap, increase gap
             # count of gapC by 1, modify start of gapC and remove gapA
             # GLC:     ------oooo-----oooooo-o---22222222--ooo------ooooo
             # GLA:     ---------------------------AA---xxx-------xxxxxxxx
             # GLC(new):------oooo-----oooooo-o------22222--ooo------ooooo
             # GLA(new):--------------------------------xxx-------xxxxxxxx
             # Result:  --------------------------233---------------------
             if gapA[G_BEG] > gapC[G_BEG] and gapA[G_END] < gapC[G_END]:
                debug(5, cdqualParams, ('gapA    +-:', gapA, 'OCI'))
                # End time of tmp gap is start time of A gap
                tmpGapList.append([gapC[G_BEG], gapA[G_BEG], gapC[G_CNT]])
                tmpGapList.append([gapA[G_BEG], gapA[G_END], gapC[G_CNT] + 1])
                # Rest of gapC is tested in the next iteration
                # Start time of modified gap C is end time of gap A
                gapC[G_BEG] = gapA[G_END]
                # This gap has been processed complete
                currentGapList.remove(gapA)
                # Continue with next gapA
                continue
             # GapC overlaps gapA but same start --> copy gapA with gap_count+1,
             # modify start of gapC remove gapA
             # GLC:     ------oooo-----oooooo-o---22222222--ooo------ooooo
             # GLA:     --------------------------AAA---xxx-------xxxxxxxx
             # GLC(new):------oooo-----oooooo-o------22222--ooo------ooooo
             # GLA(new):--------------------------------xxx-------xxxxxxxx
             # Result:  --------------------------333---------------------
             if gapA[G_BEG] == gapC[G_BEG] and gapA[G_END] < gapC[G_END]:
                debug(5, cdqualParams, ('gapA    +-:', gapA, 'OAL'))
                tmpGapList.append([gapA[G_BEG], gapA[G_END], gapC[G_CNT] + 1])
                # Rest of gapC is tested in the next iteration
                # Start time of modified gap C is end time of gap A
                gapC[G_BEG] = gapA[G_END]
                # This gap has been processed complete
                currentGapList.remove(gapA)
                # Continue with next gapA
                continue
             # GapC overlaps gapA but same end --> copy gapA with gap_count+1,
             # modify start of gapC remove gapA
             # GLC:     ------oooo-----oooooo-o---22222222--ooo------ooooo
             # GLA:     -------------------------------AAA--------xxxxxxxx
             # GLC(new):------oooo-----oooooo-o---oooooooo--ooo------ooooo
             # GLA(new):------------------------------------------xxxxxxxx
             # Result:  --------------------------22222333----------------
             if gapA[G_BEG] > gapC[G_BEG] and gapA[G_END] == gapC[G_END]:
                debug(5, cdqualParams, ('gapA    +-:', gapA, 'OAR'))
                # End time of tmp gap is start time of A gap
                tmpGapList.append([gapC[G_BEG], gapA[G_BEG], gapC[G_CNT]])
                tmpGapList.append([gapA[G_BEG], gapA[G_END], gapC[G_CNT] + 1])
                # This gap has been processed complete
                currentGapList.remove(gapA)
                # Break to continue with next gapC
                break

             # GapA not processed!! Stop application
             sys.exit("Error: GapA %s-%s not processed (GapC: %s-%s) %s/%s" \
                   % (gapA[0], gapA[1], gapC[0], gapC[1],
                       statistics[SITE], statistics[CHANNEL]))

          # End for gapA in currentGapList:
       # End for gapC in combinedGapList:

       # Make the temporary gap list the current one
       combinedGapList = tmpGapList
       tmpGapList = []

   debugGaps2(5, cdqualParams, ('Result gaps: ', combinedGapList))
   return combinedGapList


# -----------------------------------------------------------------------------
# @short Create channel statistics record
#
# This function created and initializes a channel statistics
# record for a given site/channel combination
#
# @param site    site name
# @param chan    channel name
# @param chan    nominal sample rate
#
# @return list containing net(empty) (0), site( 1), channel (2),
#                         %recvd = 0 (3), %avail = 0 (4),
#                         nbr. of gaps = 0 (5), nbr. of samples = 0 (6),
#                         nbr of constant values = 0 (7), mean = 0 (8),
#                         RMS (9), real sample rate = 0 (10) and
#                         nominal sample rate (11), max. expected time = 0 (12),
#                         (empty) list of gaps (13), (empty) request list (14),
#                         number of wfdiscs = 0 (15), number of quality
#                         records = 0 (16), number of late samples = 0 (17),
#                         number of untrusted samples = 0 (18), number of
#                         untrusted and late samples = 0 (19), (empty)
#                         authentication gap list (20),
#                         frames received = 0 (21),
NET = 0
SITE = 1
CHANNEL = 2
NBR_LATE_SAMPLES_WO_BAD = 3
NBR_UNTRUSTED_SAMPLES = 4
NBR_GAPS = 5
NBR_SAMPLES = 6
NBR_BAD_SAMPLES = 7
MEAN = 8
RMS = 9
SAMP_RATE = 10
NOM_SAMP_RATE = 11
MAX_EXP_TIME = 12
GAP_LIST = 13
REQ_LIST = 14
WFDISCS = 15
QUAL_RECS = 16
NBR_LATE_SAMPLES = 17
NBR_UNTRUSTED_SAMPLES_WO_BAD = 18
NBR_UNTRUSTED_LATE_SAMPLES_WO_BAD = 19
AUTH_GAP_LIST = 20
CSF_RCVD = 21
CSF_CHECKED = 22  # CSF checked for authentication
CSF_FAILED = 23
NBR_SAMPLES_QUAL = 24


def createStatisticRec(site, chan, nomSampRate):
   # Set up default record
   return ['', site, chan, 0, 0, 0, 0, 0, 0.0, 0.0, 0.0,
           nomSampRate, 0, [], [], 0, 0, 0, 0, 0, [], 0, 0, 0, 0]

# -----------------------------------------------------------------------------
# @short Debug channel statistics
#
# This function prints all channel values for a given channel
# omitting gap and request list
#
# @param debugLevel   debug level
# @param cdqualParams dictionary containing name value pairs
# @param cStat        channel record
#
# @return nothing
# -----------------------------------------------------------------------------

def debugChannel(debugLevel, cdqualParams, cStat):

   # Set up default record
   record = debug(debugLevel, cdqualParams,
                  "%5s %5s %3s %8dL %8dU %5dG %8dS %8dB %7.5f, %7.5f" \
                  % (cStat[0], cStat[1], cStat[2], cStat[3], cStat[4],
                     cStat[5], cStat[6], cStat[7], cStat[10], cStat[11]))

# -----------------------------------------------------------------------------
# @short Get Channel Statistics
#
# This function computes the statistics for a given
# site/channel combination and a given time period
#
# @param siteChan      list containing site and channel
# @param cdqualParams  dictionary containing name value pairs
# @param reqList       list with requested data time periods
# @param frames        list with frame lines
# @param NCDStation    is non-continuous data (auxiliary) station
#
# @return list containing net(empty) (0), site( 1), channel (2),
#                         %recvd (3), %avail (4),
#                         nbr. of gaps (5), nbr. of samples (6),
#                         nbr of constant values (7), mean (8), RMS (9),
#                         real sample rate (10), nominal sample rate (11),
#                         max. expected time (12), list (13) of gaps with
#                             gap_start_time,  gap_end_time and gap type
#                         (empty) request list (14),
#                         number of wfdiscs (15), number of quality
#                         records (16), number of late samples (17),
#                         number of untrusted samples (18), number of
#                         untrusted and late samples (19),
#                         list of authentication gaps (20) with
#                             gap_start_time,  gap_end_time and gap type
#                         frames received including this channel (21)
# -----------------------------------------------------------------------------

def getChannelStatistics(siteChan, cdqualParams, reqList, frames, NCDStation):

   schema = cdqualParams.get('schemaPrefix', '')

   # Set up default record
   record = createStatisticRec(siteChan[0], siteChan[1], siteChan[2])
   sampleRate = [0.0, 0]
   realSampleRate = record[NOM_SAMP_RATE]
   gaps = []
   authGaps = []
   frameList = []
   mean = [0.0, 0.0, 0]
   nbrWfdiscs = 0
   nbrQualRecs = 0
   FIRST_SEL_OPS = 1
   FIRST_OPS = 2
   TRY_ARCHIVE = 3
   ARCHIVE = 4
   OPS = 5
   REQ_START = 0
   REQ_END = 1
   REQ_TYPE = 3

   for request in reqList:
      lastEnd = request[REQ_START]
      firstStart = request[REQ_START]
      queryState = FIRST_SEL_OPS

      # Next lines are only for auxiliary stations
      # which use different algorithm to calculate timeliness
      lastGapEndAux = request[REQ_START]
      if request[REQ_TYPE] == REQUEST_GAP_TD:
         isLateAux = 1
      else:
         isLateAux = 0

      # ---------------------------------------------------------------------
      # Set up first database query. Search all wfdiscs which start or end time
      # is within the requested period. If the requested period is split
      # between OPS and ARCHIVE, query ARCHIVE first and than OPS. For
      # performance reasons OPS is queried first and only if the start time
      # of the first wfdisc is after the start time of the requested data,
      # ARCHIVE and than OPS is queried
      for db in ('dbConOps', 'dbConArchive', 'dbConOps'):
         debug(2, cdqualParams, 'Query ' + db)
         curCursor = cdqualParams[db].cursor()

##         sqlStr = "select time, endtime, nsamp, samprate \
## from %swfdisc where sta = '%s' and chan = '%s' and \
## endtime > %f and time < %f order by time" % (schema,
##                siteChan[0], siteChan[1], firstStart, request[REQ_END])

         # For use of prepared statement, build static part of SQL string first
         sqlStr = "SELECT time, endtime, nsamp, samprate \
 FROM %swfdisc WHERE sta=? AND chan=? AND \
 time BETWEEN ? AND ? AND endtime > ? \
 ORDER BY time" % (schema)

         # Bind parameters as strings to avoid performance drop when DB
         # data type does not match python type in where clause
         firstStartSm = str(firstStart - 14500)
         requestEndS = str(request[REQ_END])
         firstStartS = str(firstStart)

         # NOTE: This select does not consider on and off dates for channels!
         debug(3, cdqualParams, (sqlStr, siteChan[0], siteChan[1],
                                 firstStartSm, requestEndS,
                                 firstStartS))

         # NOTE: This select does not consider on and off dates for channels!
         curCursor.execute(sqlStr, (siteChan[0], siteChan[1],
                           firstStartSm, requestEndS, firstStartS))
         debug(3, cdqualParams, "select finished")
         for row in curCursor:
             debug(3, cdqualParams, '.')
             debug(3, cdqualParams, row)
             debug(5, cdqualParams, ('lastend = ', lastEnd))
             nbrWfdiscs += 1
             # Now check if the wfdiscs are complete within the requested
             # time period and calculate requested values.
             if request[REQ_START] < row[0]:
                if FIRST_SEL_OPS == queryState:
                   # This is the first query in the OPS database and wfdisc
                   # start after 'startTime'. In this case ARCHIVE is
                   # checked for older records
                   queryState = TRY_ARCHIVE
                   curCursor.close()
                   break
                tStart = row[0]
             else:
                # If the waveform data must be read here don't forget to
                # calculate the offset in the wfdisc here (see
                # readWaveformData() from wave API)
                tStart = request[REQ_START]

             if FIRST_SEL_OPS == queryState:
                # Data has been found in OPS
                queryState = FIRST_OPS

             if request[REQ_END] > row[1]:
                tEnd = row[1]
             else:
                tEnd = request[REQ_END]

             # Compute end time. Keep in mind that endtime
             # is the start time of the last sample in the segment!
             tEnd += 1./row[3]

             # Check for overlapped data. If end time of current record
             # is before last end time, it is a complete overlapped record.
             # Gap detection does not work for overlap records, so
             # they are not considered further
             if tEnd > lastEnd:

                # If start time of current record is lower than last end
                # time it is a partial overlap. Adjust time in this case
                if lastEnd > tStart:
                   tStart = lastEnd

                tdiff = tEnd - tStart

                # Number of samples stored in a wfdisc is not used here,
                # because it could happen that only parts of the wfdisc is
                # used. NOTE: If overlapped data must be considered here, check
                # if last end date is higher than start and/or end date
                # and modify the number of samples accordingly
                # To get consistent values for samples and constant samples
                # use the same rounding HERE and when processing the
                # quality record below
                nSamples = int(round(tdiff * row[3]))
                debug(3, cdqualParams,
                   'Sampratewfdisc: %7.5f, samples: %d, tdiff: %f' % (row[3], nSamples, tdiff))
                record[NBR_SAMPLES] += nSamples

                # Check for a gap. It is a gap when the start or the end of
                # the data ST--|DDDDDDDDDDDD|--ET is missing
                if (tStart - lastEnd) > (1./row[3]):

                   record[NBR_GAPS] += 1
                   # Gap end time is END time of last sample to have a chance
                   # to compute overall gap time
                   gaps.append((lastEnd, tStart, 0, NO_DATA_GAP_T))

                # Sum up sample rate and number of sample rates
                # sampleRate[0] += row[3]
                # sampleRate[1] += 1
                sampleRate[0] += row[3] * tdiff
                sampleRate[1] += tdiff

                # Check for overlapped data. If start time of record
                # is lower than last end time it is an overlap record.
                # Gap detection does not work for overlap records, so
                # they are not considered further
                if tEnd > lastEnd:
                   # Remember the end of the wfdisc for gap detection
                   lastEnd = tEnd

         # End of wfdisc processing for particular channel
         # -------------------------------------------------------------------
         debugGaps2(5, cdqualParams, ('gaps after wfdisc: ', gaps))

         if FIRST_SEL_OPS == queryState or TRY_ARCHIVE == queryState:
            # First record found in OPS older than requested start time
            # or no record found at all. Check ARCHIVE
            queryState = ARCHIVE
            continue

         # Try to compute real sample rate by computing the mean of all
         # sample rates found in the wfdisc records
         if sampleRate[1] > 0:
            realSampleRate = sampleRate[0] / sampleRate[1]

         record[SAMP_RATE] = realSampleRate
         debug(2, cdqualParams, ("sampRate=", realSampleRate))

         if not ARCHIVE == queryState:
            # Check for gap between endtime of last segment and report end time
            if (request[REQ_END] - lastEnd) > (1./realSampleRate):
               record[NBR_GAPS] += 1
               gaps.append((lastEnd, request[REQ_END], 0, NO_DATA_GAP_T))

         if ARCHIVE == queryState and 0 == nbrWfdiscs:
            # No wfdiscs found in archive. No need to check
            # cdquality records. Keep in mind that inconsistency between
            # wfdisc and cdquality will not show up longer
            queryState = OPS
            curCursor.close()
            continue

         if 0 == nbrWfdiscs:
            # No wfdiscs found in second try of ops. No need to check
            # cdquality records
            break


         # ?? Quality Record issues:
         #   o) Number of constant data or 'no input from sensor'
         #      samples is not known (Must not spawn always the whole frame)
         #   o) Differential too large not checked
         #   o) No chance to compute mean and RMS when requested time shorter
         #      than record without re-computing it
         #   o) Have to query the wfdisc records to get location of
         #      waveform files

         # ---------------------------------------------------------------------
         # Set up second database query. Search for all quality records which
         # start or end time is within the requested period. If the requested
         # period is split between OPS and ARCHIVE, query OPS first

##         sqlStr = "select time, endtime, nsamp, qualityflags, meancounts, \
##    m2 from %scdquality where sta = '%s' and chan = '%s' and \
##    endtime > %f and time < %f" % (schema, siteChan[0],
##                   siteChan[1], firstStart, request[REQ_END])

         sqlStr = "SELECT time, endtime, nsamp, qualityflags, meancounts, \
 m2, chanstatus FROM %scdquality WHERE sta=? AND chan=? AND \
 time BETWEEN ? AND ? AND endtime > ? \
 ORDER BY time" % (schema)


         # Does not work without closing the cursor first
         curCursor.close()
         curCursor = cdqualParams[db].cursor()

         # Bind parameters as strings to avoid performance drop when DB
         # data type does not match python type in where clause
         firstStartSm = str(firstStart - 86400)
         requestEndS = str(request[REQ_END])
         firstStartS = str(firstStart)

         # NOTE: This select does not consider on and off dates for channels!
         debug(3, cdqualParams, (sqlStr, siteChan[0], siteChan[1],
                                 firstStartSm, requestEndS,
                                 firstStartS))

         curCursor.execute(sqlStr, (siteChan[0], siteChan[1],
                           firstStartSm, requestEndS, firstStartS))
         debug(3, cdqualParams, "q select finished")

         for row in curCursor:
            debug(4, cdqualParams, '+')
            debug(3, cdqualParams, row)
            nbrQualRecs += 1
            # Now check if the quality records are complete within the
            # requested time period and calculate requested values.
            if request[REQ_START] < row[0]:
               tStart = row[0]
            else:
               # If the waveform data must be read here don't forget to
               # calculate the offset in the wfdisc (see readWaveformData()
               # from wave API
               tStart = request[REQ_START]

            if request[REQ_END] > row[1]:
               tEnd = row[1]
            else:
               tEnd = request[REQ_END]

            # Compute end time. Keep in mind that endtime
            # is the start time of the last sample in the segment!
            tEndS = tEnd + 1./realSampleRate
            tdiff = tEndS - tStart
            # To get consistent values for samples and constant samples
            # use the same rounding HERE and when processing the
            # wfdisc record above
            nSamples = int(round(tdiff * realSampleRate))
            debug(3, cdqualParams,
                  'Sampratequal: %7.5f, samples: %d tdiff: %f' % (realSampleRate, nSamples, tdiff))
            debug(5, cdqualParams, ("samples=", nSamples, " st=",
                                    tStart, " et=", tEndS,
                                    "realSamp=", realSampleRate))
            record[NBR_SAMPLES_QUAL] += nSamples
            badSamples = [ 0, 0 ]

            # Set virtual gap type
            authGapType = gapType = NO_GAP_T
            qualityFlag = int(row[3])

            # If station is an auxiliary station, timeliness is calculated
            # based on the information in the request table instead of
            # receiving time of frames.
            debug(5, cdqualParams, request)
            if NCDStation:
               if isLateAux:
                  gapType = LATE_GAP_T
            elif qualityFlag & QUA_NOT_TIMELY:
               gapType = LATE_GAP_T

            # If surety considers only channel authentication
            # use the following line
            if qualityFlag & QUA_AUTH_FAILURE:
            # If surety considers frame and channel authentication
            # use this one:
            ## if qualityFlag & (QUA_AUTH_FAILURE | QUA_FR_AUTH_FAILURE):
               gapType |= AUTH_FAIL_GAP_T
            if int(row[6]) & SURETY_CHANNEL_ATTRIBUTES:
               gapType |= SURETY_GAP_T
            if qualityFlag & QUA_NO_INPUT:
               gapType |= BAD_DATA_GAP_T
               record[NBR_BAD_SAMPLES] += nSamples
               badSamples[0] = nSamples

            # Set virtual authentication gap type
            authGapType = qualityFlag & (QUA_AUTH_FAILURE | \
                                         QUA_FR_AUTH_FAILURE)

            # Check for "constant data" samples. If "no input from
            # sensor" set is set, "constant data" flag must not set in
            # quality flags!!
            if qualityFlag & QUA_CONST_DATA:
               # Sum up number of samples in constant data or no
               # input from sensor
               badSamples = getBadSampleNumber(db, siteChan, cdqualParams,
                                               tStart, tEndS, nSamples)
               debug(4, cdqualParams, ("Bad samples ", badSamples, "Samples",
                                        nSamples))
               # Count 'no input from sensor' and 'constant data' as bad
               record[NBR_BAD_SAMPLES] += (badSamples[0] + badSamples[1])
               # Add constant data gaps to gap list now
               gapTypeC = gapType
               gapTypeC |= BAD_DATA_GAP_T

               # It might happen that not the whole frame contains constant
               # data, so samples containing constant data are converted
               # into 'virtual' gaps. This is the reason why the number
               # of virtual gaps MIGHT BE HIGHER than the number of quality
               # records
               lastGapEnd = tStart
               for cDataGap in badSamples[2]:
                  # Check if the whole segment is marked as "bad" like late
                  # data. Create virtual gap with another gap type between
                  # the constant data gaps in this case
                  if lastGapEnd != cDataGap[0] and gapType != NO_GAP_T:
                     gaps.append((lastGapEnd, cDataGap[0], 0, gapType))
                  gaps.append((cDataGap[0], cDataGap[1], 0, gapTypeC))
                  lastGapEnd = cDataGap[1]

               if lastGapEnd != tEndS and gapType != NO_GAP_T:
                  gaps.append((lastGapEnd, tEndS, 0, gapType))

               # Virtual gaps with other type than constant data are created
               # already!
               gapType = NO_GAP_T

            # Check for too late arrived samples but do not count "bad"
            # data. Needed for station report. See example there
            if qualityFlag & QUA_NOT_TIMELY:
               record[NBR_LATE_SAMPLES_WO_BAD] += nSamples - badSamples[0]
               # Add too late arrived samples for station statistics
               record[NBR_LATE_SAMPLES] += nSamples

            # Check for unauthenticated samples
            # If authentication is switched off data is treated
            # as unauthenticated
            if qualityFlag & QUA_AUTH_FAILURE:
               record[NBR_UNTRUSTED_SAMPLES_WO_BAD] += nSamples - badSamples[0]
               # Add too late arrived samples for station status
               record[NBR_UNTRUSTED_SAMPLES] += nSamples

            # Check for too late arrived samples but do not count "bad"
            # data. Needed for station report. See example there
            if (qualityFlag & QUA_AUTH_FAILURE) \
                and (qualityFlag & QUA_NOT_TIMELY):
               record[NBR_UNTRUSTED_LATE_SAMPLES_WO_BAD] += nSamples - \
                                                            badSamples[0]

            # No need to check for SURETY_GAP_T gaps here if unsure samples
            # are not counted.

            # Sum up mean, RMS and number of samples
            mean[0] += (row[4] * nSamples) # mean
            mean[1] += row[5]              # m2
            mean[2] += nSamples

            if gapType != NO_GAP_T:
               # Create "gap list" for late arrived and/or unauthenticated
               # and/or unsure data
               gaps.append((tStart, tEndS, 0, gapType))

            if authGapType != NO_GAP_T:
               # Create "gap list" for unauthenticated data by type
               authGaps.append([tStart, tEndS, 0, authGapType])

            # Remember end of last gap for late auxiliary stations
            lastGapEndAux = tEndS

         # Does not work without closing the cursor first
         curCursor.close()

         # End of quality record processing for particular channel
         # -----------------------------------------------------------------
         debugGaps2(5, cdqualParams, ('gaps after cdquality: ', gaps))

         # If all the data has been found in OPS we can stop here
         if FIRST_OPS == queryState or FIRST_SEL_OPS == queryState:
            break
         elif ARCHIVE == queryState:
            # This works only if related wfdisc and quality records are
            # archived synchronously
            firstStart = lastEnd
            queryState = OPS

      # End of OPS/ARCHIVE processing for particular channel
      # ---------------------------------------------------------------------
      # Copy frames which cover the request time to temporary list
      channelIndex = 0
      tmpFrames = []
      channelSearchStr = '\|' + siteChan[0] + '/' + siteChan[1] + '\|'
      for frame in frames:
         if frame[F_TYPE] == 3: # Data frame
            if frame[F_NOM_TIME] < request[REQ_END] and \
               frame[F_ENDTIME] > request[REQ_START]:
               try:
                  if (frame[F_MASK][channelIndex] == '1'):
                     tmpFrames.append(frame)
               except IndexError:
                  print >> sys.stderr, \
                  "Warning! Channel index too high " + frame[F_MASK] + \
                    ", index = " + str(channelIndex)

         elif frame [F_TYPE] == 2: # Virtual Channel Name frame
            if re.search(channelSearchStr, frame[F_MASK], 0):
               channelIndex = frame[F_INDEX]
         else: # Virtual Connect frame
            channelIndex = 0

      # Sort frames by nominal time to avoid counting duplicates
      # (frames with same nominal time). Works only if duplicate
      # frames have same duration
      tmpFrames.sort(key=lambda tmpFrame: tmpFrame[F_NOM_TIME])
      isFirst = 1
      # Remove duplicate frames. First frame received wins
      for frame in tmpFrames:
         if isFirst:
            isFirst = 0
            lowestRecTime = frame[F_RECVD_TIME]
            lastNomTime = frame[F_NOM_TIME]
            lastFrame = frame

         if lastNomTime < frame[F_NOM_TIME]:
            frameList.append(lastFrame)
            lowestRecTime = frame[F_RECVD_TIME]
            lastNomTime = frame[F_NOM_TIME]
            lastFrame = frame
         else:
            if lowestRecTime > frame[F_RECVD_TIME]: # older frame found
               lowestRecTime = frame[F_RECVD_TIME]
               lastFrame = frame
      # 'frameList' contains now only frames which contribute to
      # channel data

      if isLateAux:
         if lastGapEndAux == request[REQ_START]:
            # No quality record found, make the whole request late);
            gaps.append((request[REQ_START], request[REQ_END], 0, LATE_GAP_T))

         # Information if the request has been delayed is not longer
         # needed
         request[REQ_TYPE] = REQUEST_GAP_T

   # End of request loop
   # ---------------------------------------------------------------------
   # Compute mean and RMS of all mean values found in the quality records
   if mean[2] > 0:
      record[MEAN] = round(mean[0]/mean[2],1)
      record[RMS] = round(sqrt(mean[1]/mean[2]),4)

   # Sort the gap list
   gaps.sort(key=lambda gap: gap[0])
   # Sort the authentication gap list
   authGaps.sort(key=lambda gap: gap[0])

   # Determine authentication state of channel sub-frames
   ij = 0
   lenAuthGapList = len(authGaps)
   debug(4, cdqualParams, ("site", record[SITE], "channel", record[CHANNEL],
         "Out gaps:", authGaps, "Len: ", lenAuthGapList));
   for frame in frameList:
      record[CSF_RCVD] += 1
      # If no authentication gap has been found,
      # CSF passed
      record[CSF_CHECKED] += 1
      frame[F_AUTH_STATE] = 0
      for ii in range(ij, lenAuthGapList):
         debug(3, cdqualParams, ("fs=", frame[F_NOM_TIME], " ge=",
               authGaps[ii][G_END], " fe=", frame[F_ENDTIME],
               "gs=", authGaps[ii][G_BEG], record[CSF_CHECKED],
               record[CSF_FAILED]))
         if frame[F_ENDTIME] < authGaps[ii][G_BEG]:
            # Frame ends before the first gap. No need to scan gaps
            debug(2, cdqualParams, ("before --------"))
            break
         elif frame[F_NOM_TIME] >= authGaps[ii][G_END]:
            # Frame starts after this gap. Try next one
            ij = ii
            debug(2, cdqualParams, ("after --------"))
            continue
         else:
            if frame[F_NOM_TIME] < authGaps[ii][G_END] and \
               frame[F_ENDTIME] > authGaps[ii][G_BEG]:
                  # and check channel authentication status
               if authGaps[ii][G_TYP] & QUA_NOT_AUTHENTICATED:
                  record[CSF_CHECKED] -= 1
               elif authGaps[ii][G_TYP] & QUA_AUTH_FAILURE:
                  record[CSF_FAILED] += 1
               else:
                  debug(2, cdqualParams, ("if --------"))
               # Remember frame authentication status in the frame line
               # The default is OK
               if authGaps[ii][G_TYP] & QUA_FR_NOT_AUTHENTICATED:
                  frame[F_AUTH_STATE] = QUA_FR_NOT_AUTHENTICATED
               elif authGaps[ii][G_TYP] & QUA_FR_AUTH_FAILURE:
                  frame[F_AUTH_STATE] = QUA_FR_AUTH_FAILURE

            else:
               # No authentication interval found. Frame and channel
               # authentication passed
               debug(2, cdqualParams, ("else --------"))

            ij = ii
            break

   # Add the gap list
   record[GAP_LIST] = gaps
   record[AUTH_GAP_LIST] = authGaps

   # Merge consecutive authentication gaps
   mergeAuthGaps(cdqualParams, record)

   record[WFDISCS] = nbrWfdiscs
   record[QUAL_RECS] = nbrQualRecs

   # Return channel statistics
   return record


# -----------------------------------------------------------------------------
# @short Create report
#
# This function creates a report for a given list of
# stations. A station can be a single station, an array
# or a "network" in the affiliation list
#
# @param reportType    type of the report (CHAN_STATUS, STA_STATUS, OUTAGE)
# @param cdqualParams  dictionary containing name value pairs
#
# @return dictionary containing name value pairs
# -----------------------------------------------------------------------------

def createReport(reportType, cdqualParams):

   debug(3, cdqualParams, 'Surety bits: %x' % (SURETY_CHANNEL_ATTRIBUTES))

   # Check if the list of stations/arrays/network/channels is correct
   unknown = []
   for station in cdqualParams['args']:
      # Check if station name contains also a channel name
      # Syntax: station/channel
      uncheckedChannels = ''
      mustCheckChannels = 1

      try:
         (station, uncheckedChannels) = station.split('/', 1)
      except ValueError, why:
         mustCheckChannels = 0

      debug(3, cdqualParams, 'Station [' + station + \
            '], unchecked channels [' + uncheckedChannels + ']')
      # Station could be a single station, an array or a "network"
      # Single stations and arrays are covered by stationOrArrayToSites(),
      # but the check for the "network" has to be done here!
      stations = networkToStation(station, cdqualParams)
      debug(2, cdqualParams, stations)
      if len(stations) == 0:
         unknown.append(station)
         continue

      # Station part is OK
      knownChannels = []
      if mustCheckChannels:
         if reportType == 'STA_STATUS' or reportType == 'STA_STATUS:2' or \
            reportType == 'STA_STATISTICS' or reportType == 'STA_STATISTICS:2':
            print >> sys.stderr, \
                  "Warning! Providing a channel for report " + reportType + \
                  " makes no sense. " + station + "/" + uncheckedChannels + \
                  " skipped .."
            return
         chanNum = 0
         for actStation in stations:
            # Get sites and channels of station
            siteChannels = stationOrArrayToSites(actStation, cdqualParams)

            # If the user has provided channels, check them
            # siteChannels is modified in filterSiteChan()
            debug(3, cdqualParams, siteChannels)
            # Create list of known channels
            for siteChannel in siteChannels:
                knownChannels.append(siteChannel[1])

         try:
            chanList = uncheckedChannels.split('/')
         except ValueError, why:
            chanList = [ uncheckedChannels ]

         for actChan in chanList:
            if actChan not in knownChannels:
               unknown.append(station + '/' + actChan)

   if 0 != len(unknown):
      sys.exit("Error: Can't find %s in database" % (unknown))
   # ===== end check ====

   # Create requested report for all names found in the CLI
   processArgs(cdqualParams, reportType)


# -----------------------------------------------------------------------------
# @short Get station names for network
#
# This function creates a list of stations for a given network
# If network is a single station return only the station name
#
# @param network       network of station name
# @param cdqualParams  dictionary containing name value pairs
#
# @return list of stations or station
# -----------------------------------------------------------------------------

def networkToStation(network, cdqualParams):

   staticTab = cdqualParams.get('staticTablesPrefix', '')
   ops = cdqualParams['dbConOps']
   opsCursor = ops.cursor()

   # Set up database query. Check if "network" is known in channame
   # For use of prepared statement, build static part of SQL string first
   sqlStr = "SELECT COUNT(DISTINCT c.station) FROM\
 %ssensor s, %sinstrument i, %schanname c WHERE s.inid = i.inid AND\
 c.intern_sta = s.sta AND c.intern_chan = s.chan AND c.station=? AND\
 (%f > s.time AND %f < s.endtime) AND c.revision = (SELECT MAX(revision)\
 FROM %schanname WHERE intern_sta = s.sta AND intern_chan = s.chan)\
 GROUP BY c.station ORDER BY c.station" % (staticTab, staticTab,
                                          staticTab,
                                          cdqualParams['startTime'],
                                          cdqualParams['endTime'], staticTab)
   count = 0
   debug(3, cdqualParams, (sqlStr, network))
   opsCursor.execute(sqlStr, (network,))
   for row in opsCursor:
       count = row[0]

   if count != 0:
      # network name found in channame table. This is no "network"
      return [network, ]

   # Cursor most be closed
   opsCursor.close()
   opsCursor = ops.cursor()
   # Set up database query. Check if "network" is known in affiliation
   sqlStr = "SELECT DISTINCT sta FROM %saffiliation WHERE net=?\
             ORDER BY sta" % (staticTab)
   stationList = []
   debug(3, cdqualParams, (sqlStr, network))
   opsCursor.execute(sqlStr, (network,))
   for row in opsCursor:
       stationList.append(row[0])

   return stationList


# -----------------------------------------------------------------------------
# @short Get list of sites for given station or array
#
# This function creates a list of sites for a given station or
# array.
#
# @param station      name of the station or array
# @param cdqualParams dictionary containing name value pairs
#
# @return list containing site/channel pairs or empty list
# -----------------------------------------------------------------------------

def stationOrArrayToSites(station, cdqualParams):

   staticTab = cdqualParams.get('staticTablesPrefix', '')
   ops = cdqualParams['dbConOps']
   opsCursor = ops.cursor()

   # Set up database query
   sqlStr = "SELECT DISTINCT c.station, c.stream, s.sta, c.extern_sta, s.chan, \
 s.chanid, i.instype, i.ncalib, i.ncalper, i.samprate, s.time, s.endtime, \
 MAX(c.revision), c.position FROM\
 %ssensor s, %sinstrument i, %schanname c WHERE s.inid = i.inid AND\
 c.intern_sta = s.sta AND c.intern_chan = s.chan AND c.station=? AND\
 (%f > s.time AND %f < s.endtime) AND c.revision = (SELECT MAX(revision)\
 FROM %schanname WHERE intern_sta = s.sta AND intern_chan = s.chan)\
 GROUP BY c.station, c.stream, s.sta, c.extern_sta,\
 s.chan, s.chanid, i.instype, i.ncalib, i.ncalper, i.samprate,\
 s.time, s.endtime, c.position\
 ORDER BY s.sta, c.extern_sta, s.chan" % (staticTab, staticTab,
                                          staticTab,
                                          cdqualParams['startTime'],
                                          cdqualParams['endTime'], staticTab)
   siteList = []
   debug(3, cdqualParams, (sqlStr, station))
   opsCursor.execute(sqlStr, (station,))
   for row in opsCursor:
       siteList.append((row[2], row[4], row[9]))

   return siteList
